#
# Copyright (c) Ensim Corporation, 2003   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#
import os
import pwd
import urllib
import urlparse
import deployables_lib as dl

dl.load_msgs_module("gallery-1.3.4", symtab = globals())

dir_moves = [('gallery', '')]
file_mods = [{'path' : 'tmp/', 'create' : '', 'delete' : ''},
             {'path' : 'albums/', 'create' : '', 'delete' : ''},
             {'path' : 'config.php', 'delete' : ''},
             {'path' : '.htaccess', 'delete' : ''}]
secure_files = [('config.php', 0600, 0600),
                ('.htaccess', 0644, 0644),
                ('setup', 0, 0)]
instance_patches = []

htaccess_pat = "^\s*(?P<key>Options)\s+(?P<leading>.*?)(?P<value>\+(FollowSymLinks|SymLinksIfOwnerMatch))(?P<trailing>.*)"
htaccess_sub = "%(key)s %(leading)s %(value)s %(trailing)s"

# Note that there can't be any spaces after the quotes and the
# beginning of the string; otherwise, the space is considered
# output, and will cause PHP to send its headers.
config_data = """\
<?php
/*
 * Protect against very old versions of 4.0 (like 4.0RC1) which
 * don't implicitly create a new stdClass() when you use a variable
 * like a class.
 */
if (!isset($gallery)) {
        $gallery = new stdClass();
}
if (!isset($gallery->app)) {
        $gallery->app = new stdClass();
}

/* Version  */
$gallery->app->config_version = 35;

/* Features */
$gallery->app->feature["zip"] = 1;
$gallery->app->feature["rewrite"] = 1;
$gallery->app->feature["mirror"] = 0; // (missing <i>mirrorSites</i> -- it's optional)

/* Constants */
$gallery->app->galleryTitle = "Gallery";
$gallery->app->graphics = "ImageMagick";
$gallery->app->pnmDir = "%(im_path)s";
$gallery->app->pnmtojpeg = "pnmtojpeg";
$gallery->app->ImPath = "/usr/bin";
$gallery->app->highlight_size = "200";
$gallery->app->zipinfo = "/usr/bin/zipinfo";
$gallery->app->unzip = "/usr/bin/unzip";
// optional <i>use_exif</i> missing
$gallery->app->movieThumbnail = "%(install_loc)s/images/movie.thumb.jpg";
$gallery->app->albumDir = "%(install_loc)s/albums";
$gallery->app->tmpDir = "%(install_loc)s/tmp";
$gallery->app->photoAlbumURL = "%(url_loc)s";
$gallery->app->albumDirURL = "%(url_loc)s/albums";
// optional <i>mirrorSites</i> missing
$gallery->app->showAlbumTree = "no";
$gallery->app->autorotate = "yes";
$gallery->app->cacheExif = "no";
$gallery->app->jpegImageQuality = "95";
$gallery->app->timeLimit = "30";
$gallery->app->debug = "no";
$gallery->app->use_flock = "yes";
$gallery->app->expectedExecStatus = "0";
$gallery->app->sessionVar = "gallery_session";
$gallery->app->userDir = "%(install_loc)s/albums/.users";

/* Defaults */
$gallery->app->default["bordercolor"] = "black";
$gallery->app->default["border"] = "1";
$gallery->app->default["font"] = "arial";
$gallery->app->default["cols"] = "3";
$gallery->app->default["rows"] = "3";
$gallery->app->default["thumb_size"] = "150";
$gallery->app->default["resize_size"] = "640";
$gallery->app->default["fit_to_window"] = "no";
$gallery->app->default["use_fullOnly"] = "no";
$gallery->app->default["print_photos"] = "shutterfly";
$gallery->app->default["returnto"] = "yes";
$gallery->app->default["showOwners"] = "no";
$gallery->app->default["albumsPerPage"] = "5";
$gallery->app->default["showSearchEngine"] = "yes";
$gallery->app->default["useOriginalFileNames"] = "yes";
$gallery->app->default["display_clicks"] = "yes";
$gallery->app->default["public_comments"] = "yes";
$gallery->app->default["extra_fields"] = "Description";
?>
"""

def install(site, site_conf, instance_conf, msgs):
    actions = ['resolve_location', 'check_disk_quota', 'check_pass',
               'check_install_loc_owner', '']
    args_dict = {}

    dl.do_install_actions(site, site_conf, instance_conf, msgs,
                              globals(), actions, args_dict)
    if msgs.get_severity() != msgs.ERROR :
        actions = ['archive_unpack', 'check_index', 'exec_php_script',
                   'set_php_cgi', 'handle_security_mode',
                   'edit_conf_file', 'access_url']
        
        if site_conf['apache'].get('jail', '0') == '0':
            vars = [('Options', '+FollowSymLinks')]
        else:
            vars = [('Options', '+SymLinksIfOwnerMatch')]

        local_config_data = {'url_loc' : instance_conf['url_loc'],
                             'install_loc' : dl.site_fs_jail(site, site_conf) + \
                             instance_conf['install_loc']}
            
        # Assume that the location of convert (which belongs to ImageMagick)
        # in the site mirrors its location at the root level.
        if os.path.exists(dl.site_fs(site) + "/usr/bin/convert"):
            local_config_data['im_path'] = "/usr/bin"
        else:
            local_config_data['im_path'] = "/usr/X11R6/bin"
            
        local_config = urllib.quote(config_data % local_config_data)

        args_dict = {'archive_unpack' : {'dir_moves':dir_moves,
                                         'file_mods':file_mods},
                     'handle_security_mode' : {'files': secure_files},
                     'exec_php_script' :
                         {'script' : 'setup/index.php',
                          'args' : {'go_next' : '1',
                                    'next_page' : 'write',
                                    'editPassword' : instance_conf['instance_pass1'],
                                    'init_mod_rewrite' : '1',
                                    'ampersandbroken' : '',
                                    'GALLERY_PHP_VALUE_OK' : '1',
                                    'data' : local_config},
                          'failure_re' : '.*There are errors'},
                     'edit_conf_file' : { 'file' : '.htaccess',
                                          'lang' : 'apache_config',
                                          'pat' : htaccess_pat,
                                          'sub' : htaccess_sub,
                                          'vars' : vars},
                     'access_url' : {'user' : 'admin'}}
        dl.do_install_actions(site, site_conf, instance_conf, msgs,
                              globals(), actions, args_dict)

def reconfigure(context, site, site_conf, old_site_conf,
                instance_conf, old_instance_conf, msgs):
    # Edit the Configuration file.                                       
    old_domain = old_site_conf['siteinfo']['domain']
    new_domain = site_conf['siteinfo']['domain']
    def url_sub(key, url):
        if context == dl.CONTEXT_SITE_EDIT:
            return dl.url_net_sub(url, new_domain, old_domain)
        else:
            return dl.url_sub(url, instance_conf['url_loc'],
                              old_instance_conf['url_loc'])
    def rewrite_sub(key, value):
        return dl.url_path_sub(value, instance_conf['url_loc'],
                               old_instance_conf['url_loc'])

    old_loc = "%s%s" % (dl.site_fs_jail(site, old_site_conf),
                        old_instance_conf['install_loc'])
    new_loc = "%s%s" % (dl.site_fs_jail(site, site_conf),
                        instance_conf['install_loc'])

    pat = "^\s*\$gallery->app->(?P<key>\S+\w)\s*=\s*(?P<value>['\"].*['\"])\s*;\s*"
    sub = "$gallery->app->%(key)s = %(value)s;"
    vars = [ ('photoAlbumURL',  url_sub),
             ('albumDirURL',    url_sub),
             ('movieThumbnail', new_loc + '/images/movie.thumb.jpg',
                                old_loc + '/images/movie.thumb.jpg'),
             ('albumDir',       new_loc + '/albums', old_loc + '/albums'),
             ('tmpDir',         new_loc + '/tmp',    old_loc + '/tmp'),
             ('userDir',        new_loc + '/albums/.users',
                                old_loc + '/albums/.users')]

    # Handle ImageMagick moving
    if os.path.exists(dl.site_fs(site) + "/usr/bin/convert"):
        vars.append(('pnmDir', "/usr/bin", "/usr/X11R6/bin"))
    else:
        vars.append(('pnmDir', "/usr/X11R6/bin", "/usr/bin"))

    if site_conf['apache'].get('jail', "0") == '0':
        vars2 = [('Options', '+FollowSymLinks')]
    else:
        vars2 = [('Options', '+SymLinksIfOwnerMatch')]        

    rewrite_pat = "^\s*(?P<key>Rewrite\w+)\s+(?P<value>.*)"
    rewrite_sub_str = "%(key)s %(value)s"
    vars3 = [('RewriteBase', rewrite_sub), ('RewriteRule', rewrite_sub)]
             
    actions = ['handle_url_loc', 'resolve_location', '',
               'archive_unpack',
               'handle_username_change','edit_conf_file',
               'edit_conf_file2', 'edit_conf_file3', 'apply_patch',
               'handle_security_mode']
    args_dict = {
        'archive_unpack': { 'dir_moves' : dir_moves,
                            'fileinfo' :
                               {'excluded_files' : ['config.php']}},
        'edit_conf_file': { 'file':'config.php', 'lang':'php',
                            'pat' : pat, 'sub': sub, 'vars':vars },
        'edit_conf_file2' : {'file' : '.htaccess', 'lang' : 'apache_config',
                             'pat' : htaccess_pat, 'sub' : htaccess_sub,
                             'vars' : vars2},
        'edit_conf_file3' : {'file' : '.htaccess', 'lang' : 'apache_config',
                             'pat' : rewrite_pat, 'sub' : rewrite_sub_str,
                             'vars' : vars3},
        'handle_security_mode': {'files': secure_files },
        'apply_patch' : {'patches' : instance_patches}
        }
    dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                                  instance_conf, old_instance_conf,msgs,
                                  globals(), actions, args_dict )

def remove(site, site_conf, instance_conf, msgs):
    dl.install_loc_cleanup(site, site_conf, instance_conf, dir_moves,
                           file_mods = file_mods)

def get_instance_configuration(context, site, site_conf, new_instance_conf,
                               old_instance_conf, locale, msgs):
    instance = dl.dsp_create_instance(new_instance_conf['shortname'],
                                          new_instance_conf['version'])

    dl.dsp_fill_instance(context, instance, site, site_conf,
                             new_instance_conf, old_instance_conf,
                             ['username', 'url_net', 'url_path',
                              'domain_to_path'])
    dl.enable_patches(instance, instance_patches)
    return instance

def regression_test(action, index, test_data):
    tests = [{'disp' : 'Logging in to gallery (with bad password)',
              'test' : 'login'},
             {'disp' : 'Verifying expected page contents',
              'test' : 'check_output2'}
             ]

    ret = dl.regression_check_index(action, index, test_data,
                                    '<!-- album table begin -->')
    if ret == '':
        index = index - test_data['regression_check_index']
        if action == dl.REGRESSION_GET_TEST_STRING:
            if index >= len(tests):
                return ''
            else:
                return tests[index]['disp']
        else:
            status = -1
            if tests[index]['test'] == 'login':
                url = dl.add_slash(test_data['url_loc']) + 'login.php'
                opts = [('-F', 'uname=admin'), ('-F', 'gallerypassword=ensim2'),
                        ('-F', 'submit=Login')]
                status, test_data['output2'] = dl.exec_curl_cmd(url, opts)
            elif tests[index]['test'] == 'check_output2':
                test_data['success_re2'] = 'Invalid username or password' 
                status = dl.check_output(test_data['output2'],
                                         test_data['success_re2'])
            return status
    else:
        return ret
