#
# Copyright (c) Ensim Corporation, 2003   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#
import deployables_lib as dl

dl.load_msgs_module("invboard-1.1.2", symtab = globals())

dir_moves = [('invboard1_1_2/upload', ''),
             ('invboard1_1_2', 'docs')]
file_mods = [{'path' : 'install.lock', 'delete' : ''}]
patches = [{'file' : 'invboard.patch.ensim'}]
instance_patches = []

def install(site, site_conf, instance_conf, msgs):

    actions = ['resolve_location', 'check_accept_lic', 'check_mysql_enabled',
               'check_pass', 'check_disk_quota', 'check_install_loc_owner', '']
    args_dict = {}
    dl.do_install_actions(site, site_conf, instance_conf, msgs,
                          globals(), actions, args_dict)
    if msgs.get_severity() != msgs.ERROR:
        actions = ['db_create', 'db_probe', 'archive_unpack', 'check_index',
                   'exec_php_script', 'exec_php_script2', 'exec_php_script3',
                   'handle_security_mode', 'access_url']
        args_dict = {
            'archive_unpack': { 'dir_moves': dir_moves, 'patches' : patches },
            'handle_security_mode': { 'files': ['conf_global.php'] },
            'exec_php_script' : {
                 'script' : 'sm_install.php',
                 'args' : {'a' : '2',
                           'board_url' : instance_conf['url_loc'],
                           'sql_host' : dl.get_mysql_host(site),
                           'sql_database' : instance_conf['dbname'],
                           'sql_user' : site_conf['mysql']['dbaseadmin'],
                           'sql_pass' : instance_conf['dbpass'],
                           'sql_tbl_prefix' : '',
                           'adminname' : instance_conf['instance_admin'],
                           'adminpassword' : instance_conf['instance_pass1'],
                           'adminpassword2' : instance_conf['instance_pass1'],
                           'email' : instance_conf['instance_admin_email']},
                 'success_re' : '.*Success!'},
            'exec_php_script2' : {'script' : 'sm_install.php',
                                  'args' : {'a' : 'templates'},
                                  'success_re' : '.*Success!'},
            'exec_php_script3' : {'script' : 'sm_install.php',
                                  'args' : {'a' : '3'},
                                  'success_re' :
                                      '.*The installation is now complete!'},
            }
        dl.do_install_actions(site, site_conf, instance_conf, msgs,
                              globals(), actions, args_dict)
        

# The following instance configuration parameters can change:
# url_net/url_path: if the new values don't line up with the existing
#                   install_loc, then this is a failure
# dbname
# username
#
# The following site configuration parameters can change, and affect
# the instance parameters mentioned in parentheses
# site_conf['mysql']['dbaseprefix'] (dbname)
# site_conf['mysql']['dbaseadmin']  (internal configuration)
# site_conf['siteinfo']['domain']   (url_loc)
# site_conf['siteinfo']['admin_user'] (username)
# site_conf['apache']['jail'] (affects perm settings on files)
def reconfigure(context, site, site_conf, old_site_conf, instance_conf,
                old_instance_conf, msgs):
    actions = ['handle_url_loc', 'resolve_location', 'require_url_loc', '',
               'archive_unpack', 'handle_username_change',
               'handle_db_changes']
    args_dict = {
        'archive_unpack':       { 'dir_moves' : dir_moves,
                                  'patches' : patches,
                                  'fileinfo' :
                                  {'excluded_files' : ['conf_global.php']}},
        }
    if context != dl.CONTEXT_SITE_EDIT and instance_conf.get('dbpass'):
        actions.insert(0, 'db_probe')
        args_dict['db_probe'] = {'use_old' : 1}
    dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                              instance_conf, old_instance_conf, msgs,
                              globals(), actions, args_dict)
    if msgs.get_severity() != msgs.ERROR:
        old_domain = old_site_conf['siteinfo']['domain']
        new_domain = site_conf['siteinfo']['domain']
        def url_sub(key, url):
            if context == dl.CONTEXT_SITE_EDIT:
                return dl.url_net_sub(url, new_domain, old_domain)
            else:
                return dl.url_sub(url, instance_conf['url_loc'],
                                  old_instance_conf['url_loc'])
        
        pat = "^\s*\$INFO\['(?P<key>\S+)'\]\s*=\s*(?P<value>'.*')\s*;\s*$"
        sub = "$INFO['%(key)s'] = %(value)s;"

        old_root = "%s%s" % (dl.site_fs_jail(site, old_site_conf),
                             instance_conf['install_loc'] + '/')
        new_root = "%s%s" % (dl.site_fs_jail(site, site_conf),
                             instance_conf['install_loc'] + '/')        
        
        vars = [('board_url', url_sub), ('html_url', url_sub),
                ('upload_url', url_sub),
                ('html_dir', new_root + 'html/', old_root + 'html/'),
                ('base_dir', new_root, old_root),
                ('upload_dir', new_root + 'uploads', old_root + 'uploads')]
        if dl.mysql_installed(site_conf, old_site_conf):
            vars.extend([('sql_database', instance_conf['dbname'],
                          old_instance_conf['dbname']),
                         ('sql_user', site_conf['mysql']['dbaseadmin'],
                          old_site_conf['mysql']['dbaseadmin'])])
        if context != dl.CONTEXT_SITE_EDIT:
            # We will only change the mysql host if a user specifically
            # invokes reconfigure; we don't want this change to happen
            # during upgrade, when all the sites are edited, but the
            # mysql.sock file hasn't moved yet.
            vars.append(('sql_host', dl.get_mysql_host(site)))
            if instance_conf.get('dbpass'):
                vars.append(('sql_pass', instance_conf['dbpass']))
        
        actions = ['edit_conf_file', 'apply_patch', 'handle_security_mode']
        args_dict = {
            'edit_conf_file':       { 'file': 'conf_global.php', 'lang': 'php',
                                      'pat': pat, 'sub': sub, 'vars': vars },
            'handle_security_mode': { 'files': ['conf_global.php'] },
            'apply_patch' : {'patches' : instance_patches}
            }
        dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                                  instance_conf, old_instance_conf, msgs,
                                  globals(), actions, args_dict)
        
       
def remove(site, site_conf, instance_conf, msgs):
    dl.db_drop(site, site_conf, instance_conf)
    dl.install_loc_cleanup(site, site_conf, instance_conf, dir_moves, patches,
                           file_mods)
    
def get_instance_configuration(context, site, site_conf, instance_conf,
                               old_instance_conf, locale, msgs):
    instance = dl.dsp_create_instance(instance_conf['shortname'],
                                      instance_conf['version'])
    dl.dsp_fill_instance(context, instance, site, site_conf,
                         instance_conf, old_instance_conf,
                         ['dbname', 'username', 'url_net', 'url_path',
                          'domain_to_path', 'instance_admin_email'])
    dl.enable_patches(instance, instance_patches)
    return instance

def regression_test(action, index, test_data):
    tests = [{'disp' : 'Logging in to Invision Board',
              'test' : 'verify_login'},
             {'disp' : 'Verifying expected page contents',
              'test' : 'check_output'}
             ]
    ret = dl.regression_check_index(action, index, test_data,
                                    'IBForums latest news')
    if ret == '':
        index = index - test_data['regression_check_index']
        if action == dl.REGRESSION_GET_TEST_STRING:
            if index >= len(tests):
                return ''
            else:
                return tests[index]['disp']
        else:
            status = -1
            if tests[index]['test'] == 'verify_login':
                url = dl.add_slash(test_data['url_loc']) + 'index.php'
                opts = [('-F', 'UserName=admin'), ('-F', 'PassWord=ensim'),
                        ('-F', 'act=Login'), ('-F', 'CODE=01'),
                        ('-F', 'CookieDate=1')]
                status, test_data['output'] = dl.exec_curl_cmd(url, opts)
            elif tests[index]['test'] == 'check_output':
                test_data['success_re'] = 'Please wait while we transfer you...'
                status = dl.check_output(test_data['output'],
                                         test_data['success_re'])
            return status
    else:
        return ret
        
        
        
