<?xml version="1.0"?>
<!DOCTYPE xsl:stylesheet [
<!ENTITY nbsp "&#160;">
]>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

  <xsl:output method="html" omit-xml-declaration="yes"/>

  <xsl:param name="locale" select="'en_US'"/>

  <xsl:variable name="trans">
    <xsl:variable name="file" select="concat('/usr/lib/ensim/deployables/lib/locale/',                                              $locale, '/LC_MESSAGES/strings.xml')"/>
    <xsl:choose>
      <xsl:when test="$locale='en_US'">
        <!-- Don't try to translate the original strings -->
        <xsl:value-of select="''"/>
      </xsl:when>
      <xsl:when test="document($file)">
        <xsl:value-of select="$file"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="''"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:key name="parent-index" match="option" use="@parent"/>
  <xsl:key name="option-fail-index" match="msg" use="@option"/>

  <xsl:param name="show-indent" select="0"/>
  <xsl:variable name="fieldwidth" select="220"/>
  <xsl:variable name="indentsize" select="20"/>

  <xsl:template match="/">
    <xsl:apply-templates select="dsp/msgs"/>
    <xsl:if test="not(dsp[@context='remove']) and (not(dsp/msgs/msg) or dsp/msgs/msg[@severity='2'])">
      <xsl:apply-templates select="dsp/dspDescription"/>
    </xsl:if>
  </xsl:template>

  <xsl:template match="dspDescription">
    <xsl:call-template name="generatePage"/>
  </xsl:template>

  <xsl:template name="generatePage">
    <xsl:variable name="shortname" select="normalize-space(shortname)"/>
    <xsl:variable name="longname" select="normalize-space(longname)"/>
    <xsl:variable name="version" select="normalize-space(version)"/>
    <xsl:variable name="dapiversion" select="normalize-space(dapiversion)"/>
    <xsl:variable name="dapirelease" select="normalize-space(dapirelease)"/>
    <xsl:variable name="old_shortname" select="normalize-space(upgradeFrom/shortname)"/>
    <xsl:variable name="old_longname" select="normalize-space(upgradeFrom/longname)"/>
    <xsl:variable name="old_version" select="normalize-space(upgradeFrom/version)"/>
    <xsl:variable name="old_dapiversion" select="normalize-space(upgradeFrom/dapiversion)"/>
    <xsl:variable name="old_dapirelease" select="normalize-space(upgradeFrom/dapirelease)"/>
    <xsl:variable name="fullname" select="concat($longname, ' ', $version, ' (', $dapiversion, '-', $dapirelease, ')')"/>
    <xsl:variable name="old_fullname" select="concat($old_longname, ' ', $old_version, ' (', $old_dapiversion, '-', $old_dapirelease, ')')"/>
        <table width="640" border="0" cellspacing="0" cellpadding="7">
          <tr>
            <td class="head3" align="left" colspan="4">
              <xsl:choose>
                <xsl:when test="upgradeFrom">
                  <xsl:variable name="upgrade-string">
                    <xsl:call-template name="gettext">
                      <xsl:with-param name="default" select="'Upgrade %(old_version)s to %(new_version)s'"/>
                    </xsl:call-template>
                  </xsl:variable>
                  <xsl:variable name="upgrade-string2">
                    <xsl:call-template name="search-and-replace">
                      <xsl:with-param name="input" select="$upgrade-string"/>
                      <xsl:with-param name="search-string" select="'%(old_version)s'"/>
                      <xsl:with-param name="replace-string" select="$old_fullname"/>
                    </xsl:call-template>
                  </xsl:variable>
                  <xsl:variable name="upgrade-string3">
                    <xsl:call-template name="search-and-replace">
                      <xsl:with-param name="input" select="$upgrade-string2"/>
                      <xsl:with-param name="search-string" select="'%(new_version)s'"/>
                      <xsl:with-param name="replace-string" select="$fullname"/>
                    </xsl:call-template>
                  </xsl:variable>
                  <xsl:value-of select="$upgrade-string3"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="$fullname"/>
                </xsl:otherwise>
              </xsl:choose>
            </td>
          </tr>
	 <tr height="48">
	   <td colspan="4" class="help" align="center">
	     &nbsp;
	     <img src="view_image?dsp={$shortname}&amp;version={$version}" align="absmiddle"/>
             &nbsp;
	   </td>
	 </tr>

         </table>
	 <xsl:call-template name="generateOptions"/>
       <table width="640" border="0" cellspacing="0" cellpadding="7">
	 <tr height="30">
	   <td class="help" colspan="4" align="center">
             <xsl:if test="upgradeFrom">
               <input type="hidden" name="old_shortname" value="{$old_shortname}"/>
	       <input type="hidden" name="old_version" value="{$old_version}"/>
	       <input type="hidden" name="old_dapiversion" value="{$old_dapiversion}"/>
	       <input type="hidden" name="old_dapirelease" value="{$old_dapirelease}"/>
             </xsl:if>
	     <input type="hidden" name="shortname" value="{$shortname}"/>
	     <input type="hidden" name="version" value="{$version}"/>
	     <input type="hidden" name="dapiversion" value="{$dapiversion}"/>
	     <input type="hidden" name="dapirelease" value="{$dapirelease}"/>
             <input type="hidden" name="context" value="{normalize-space(../@context)}"/>
             <xsl:variable name="ButtonText">
               <xsl:choose>
                 <xsl:when test="../@context = 'install'">
                   <xsl:call-template name="gettext">
                     <xsl:with-param name="default" select="'Install'"/>
                   </xsl:call-template>
                 </xsl:when>
                 <xsl:when test="../@context = 'reconfigure'">
                   <xsl:call-template name="gettext">
                     <xsl:with-param name="default" select="'Reconfigure'"/>
                   </xsl:call-template>
                 </xsl:when>
                 <xsl:when test="../@context = 'upgrade'">
                   <xsl:call-template name="gettext">
                     <xsl:with-param name="default" select="'Upgrade'"/>
                   </xsl:call-template>
                 </xsl:when>
                 <xsl:otherwise>
                   <xsl:call-template name="gettext">
                     <xsl:with-param name="default" select="'Action'"/>
                   </xsl:call-template>
                 </xsl:otherwise>
               </xsl:choose>
             </xsl:variable>
             <xsl:variable name="cancelText">
               <xsl:call-template name="gettext">
                 <xsl:with-param name="default" select="'Cancel'"/>
               </xsl:call-template>
             </xsl:variable>
             <xsl:variable name="resetText">
               <xsl:call-template name="gettext">
                 <xsl:with-param name="default" select="'Reset'"/>
               </xsl:call-template>
             </xsl:variable>
	     <input type="submit" value="{$ButtonText}"/>
	     <img src="/webhost/spacer_gif" height="0" width="15"/>
	     <input type="button" value="{$cancelText}" onClick="location.href='list_installed_instances?shortname={$shortname}&amp;version={$version}&amp;dapiversion={$dapiversion}&amp;dapirelease={$dapirelease}'"/>
	     <img src="/webhost/spacer_gif" height="0" width="15"/>
             <input type="reset" value="{$resetText}"/>
	   </td>
	 </tr>
       </table>
         <xsl:if test="upgradesTo and ../@context = 'reconfigure'">
	<br/>
        <table width="640" border="0" cellspacing="0" cellpadding="7">
          <tr>
            <td class="head3" align="left" colspan="4">
              <xsl:call-template name="gettext">
                <xsl:with-param name="default" select="'Available updates'"/>
              </xsl:call-template>
            </td>
           </tr>
           <tr height="30">
	     <td class="help" colspan="4" align="center">
               <xsl:apply-templates select="upgradesTo"/>
	     </td>
	   </tr>
       </table>
         </xsl:if>
   </xsl:template>

  <xsl:template match="upgradesTo">
    <xsl:if test="newversion and ../../@context = 'reconfigure'">
      <script language="JavaScript">
          var upgradedb = new Object();
          var currentShortname = "<xsl:value-of select="normalize-space(../shortname)"/>";
          var currentVersion = "<xsl:value-of select="normalize-space(../version)"/>";
          var currentDAPIVersion = "<xsl:value-of select="normalize-space(../dapiversion)"/>";
          var currentDAPIRelease = "<xsl:value-of select="normalize-space(../dapirelease)"/>";
          var upgradeShortname;
          var upgradeVersion;
          var upgradeDAPIVersion;
          var upgradeDAPIRelease;
          var currentId = "<xsl:value-of select="../@instance_id"/>";
        <xsl:for-each select="newversion">
          <xsl:variable name="shortname" select="normalize-space(shortname)"/>
          <xsl:variable name="version" select="normalize-space(version)"/>
          <xsl:variable name="dapiversion" select="normalize-space(dapiversion)"/>
          <xsl:variable name="dapirelease" select="normalize-space(dapirelease)"/>
          <xsl:variable name="fullname" select="concat($shortname,'-',$version,'-',$dapiversion,'-',$dapirelease)"/>
          <xsl:text>upgradedb["</xsl:text><xsl:value-of select="$fullname"/><xsl:text>"] = </xsl:text>
          <xsl:text>{shortname:"</xsl:text><xsl:value-of select="$shortname"/><xsl:text>",</xsl:text>
          <xsl:text>version:"</xsl:text><xsl:value-of select="$version"/><xsl:text>",</xsl:text>
          <xsl:text>dapiversion:"</xsl:text><xsl:value-of select="$dapiversion"/><xsl:text>",</xsl:text>
          <xsl:text>dapirelease:"</xsl:text><xsl:value-of select="$dapirelease"/><xsl:text>"};</xsl:text>
        </xsl:for-each>
          function upgradeChange(select) {
            select = (select) ? select : event;
            var target = select.options[select.selectedIndex].value;
            var db = upgradedb[target];
            upgradeShortname = db.shortname;
            upgradeVersion = db.version;
            upgradeDAPIVersion = db.dapiversion;
            upgradeDAPIRelease = db.dapirelease;
          }
      </script>
      
      <img src="/webhost/spacer_gif" height="0" width="15"/>
      <input type="button" name="action" value="Upgrade To" onClick="location.href='form_upgrade?shortname='+currentShortname+'&amp;version='+currentVersion+'&amp;dapiversion='+currentDAPIVersion+'&amp;dapirelease='+currentDAPIRelease+'&amp;upgradeShortname='+upgradeShortname+'&amp;upgradeVersion='+upgradeVersion+'&amp;upgradeDAPIVersion='+upgradeDAPIVersion+'&amp;upgradeDAPIRelease='+upgradeDAPIRelease+'&amp;instance_id='+currentId"/>
      <img src="/webhost/spacer_gif" height="0" width="15"/>
      <select name="upgradeList" id="upgradeList" onClick="upgradeChange(this)">
        <xsl:for-each select="newversion">
          <xsl:sort select="longname"/>
          <xsl:sort select="version"/>
          <xsl:sort select="dapiversion"/>
          <xsl:sort select="dapirelease"/>
          <xsl:variable name="shortname" select="normalize-space(shortname)"/>
          <xsl:variable name="longname" select="normalize-space(longname)"/>
          <xsl:variable name="version" select="normalize-space(version)"/>
          <xsl:variable name="dapiversion" select="normalize-space(dapiversion)"/>
          <xsl:variable name="dapirelease" select="normalize-space(dapirelease)"/>
          <xsl:variable name="fullname" select="concat($shortname,'-',$version,'-',$dapiversion,'-',$dapirelease)"/>
          <xsl:if test="not(preceding-sibling::shortname)">
            <script language="JavaScript">
              upgradeShortname     = "<xsl:value-of select="$shortname"/>";
              upgradeVersion       = "<xsl:value-of select="$version"/>";
              upgradeDAPIVersion   = "<xsl:value-of select="$dapiversion"/>";
              upgradeDAPIRelease   = "<xsl:value-of select="$dapirelease"/>";
            </script>
          </xsl:if>
          <option value="{$fullname}">
            <xsl:value-of select="$longname"/>
            <xsl:text> </xsl:text>
            <xsl:value-of select="$version"/>
            <xsl:text> (</xsl:text>
            <xsl:value-of select="$dapiversion"/>
            <xsl:text>-</xsl:text>
            <xsl:value-of select="$dapirelease"/>
            <xsl:text>)</xsl:text>
          </option>
        </xsl:for-each>
      </select>
    </xsl:if>
  </xsl:template>

  <xsl:template match="msgs">
    <xsl:if test="count(msg) &gt; 0">
      <table width="640" border="0" cellspacing="0" cellpadding="7">
      <tr>
        <td class="head3">
          <xsl:value-of select="../dspDescription/longname"/>
        </td>
      </tr>
      <tr>
        <td class="listeven"><ul>
          <xsl:for-each select="msg">
            <xsl:variable name="msgVal">
              <xsl:value-of select="normalize-space(string)"/>
            </xsl:variable>
            <xsl:choose>
              <xsl:when test="@severity='1'">
                <li>
                  <font color="red">
                    <xsl:call-template name="gettext">
                      <xsl:with-param name="default" select="'WARNING'"/>
                    </xsl:call-template>
                    <xsl:text>: </xsl:text>
                    <xsl:value-of select="$msgVal"/>
                  </font>
                </li>
              </xsl:when>
              <xsl:when test="@severity='2'">
                <li>
                  <font color="red">
                    <xsl:call-template name="gettext">
                      <xsl:with-param name="default" select="'ERROR'"/>
                    </xsl:call-template>
                    <xsl:text>: </xsl:text>
                    <xsl:value-of select="$msgVal"/>
                  </font>
                </li>
              </xsl:when>
              <xsl:otherwise>
                <li>
                  <xsl:call-template name="gettext">
                    <xsl:with-param name="default" select="'SUCCESS'"/>
                  </xsl:call-template>
                  <xsl:text>: </xsl:text>
                  <xsl:value-of select="$msgVal"/>
                </li>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:for-each>
        </ul></td>
      </tr>
      </table>
      <p>&nbsp;</p>
      <p>&nbsp;</p>
    </xsl:if>
  </xsl:template>

  <xsl:template name="generateOptions">
    <xsl:for-each select="option">
      <xsl:choose>
        <xsl:when test="@parent"/>
        <xsl:otherwise>
	  <xsl:call-template name="handleOption">
            <xsl:with-param name="indent" select="0"/>
          </xsl:call-template>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
  </xsl:template>

  <xsl:template name="handleOption">
    <xsl:param name="indent"/>
      <xsl:if test="$show-indent">
        Current: <xsl:value-of select="name(current())"/>
        Name: <xsl:value-of select="@name"/>
        Indent: <xsl:value-of select="$indent"/>
      </xsl:if>
    <xsl:apply-templates select=".">
      <xsl:with-param name="indent" select="$indent"/>
      <xsl:with-param name="fromHandleOption" select="true()"/>
    </xsl:apply-templates>
    <xsl:for-each select="key('parent-index',@name)">
      <xsl:call-template name="handleOption">
        <xsl:with-param name="indent" select="$indent + 1"/>
      </xsl:call-template>
    </xsl:for-each>
  </xsl:template>

  <xsl:template name="handleTitle">
    <xsl:param name="elem" select="."/>
    <xsl:variable name="elem_title" select="normalize-space($elem/title)"/>
    <xsl:choose>
      <xsl:when test="key('option-fail-index', $elem/@name)">
        <font color="red">
          <xsl:value-of select="$elem_title"/>
        </font>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$elem_title"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="removePrefixSuffix">
    <xsl:param name="prefix"/>
    <xsl:param name="suffix"/>
    <xsl:param name="string"/>
    <xsl:variable name="minusprefix">
      <xsl:choose>
        <xsl:when test="$prefix and starts-with($string, $prefix)">
          <xsl:value-of select="substring($string, string-length($prefix) + 1)"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$string"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="$suffix and ends-with($minusprefix, $suffix)">
        <xsl:value-of select="substring($minusprefix, 1, string-length($minusprefix) - string-length($suffix))"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$minusprefix"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

<!--
    To create a more selective template for a given option,
    create a template that matches against "option[@name='name']"
-->

  <xsl:template match="option[Boolean]" name="Boolean">
    <xsl:param name="elem" select="."/>
    <xsl:param name="indent"/>
    <xsl:param name="triggerFunc" select="''"/>
      <xsl:if test="$show-indent">
        Boolean
        OptName: <xsl:value-of select="$elem/@name"/>
        Indent: <xsl:value-of select="$indent"/>
      </xsl:if>
    <xsl:choose>
      <xsl:when test="$elem[@hidden='1']">
        <input type="hidden" id="{$elem/@name}" name="{$elem/@name}" value="{$elem/@value}"/>
      </xsl:when>
      <xsl:otherwise>
    <table width="640" border="0" cellspacing="0" cellpadding="7">
    <tr>
      <td width="{$fieldwidth}" class="listeven">
        <xsl:call-template name="handleIndent">
          <xsl:with-param name="indent" select="$indent"/>
        </xsl:call-template>
        <xsl:call-template name="handleTitle">
          <xsl:with-param name="elem" select="$elem"/>
        </xsl:call-template>
      </td>
      <td class="listeven">
        <input id="{$elem/@name}" type="checkbox" name="{$elem/@name}" value="{$elem/@value}" onclick="{$triggerFunc}"/>
        <xsl:call-template name="handleHelp">
          <xsl:with-param name="elem" select="$elem"/>
        </xsl:call-template>
      </td>
    </tr>
    </table>
    <xsl:call-template name="FieldHelp">
      <xsl:with-param name="elem" select="$elem"/>
    </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="handleHelp">
    <xsl:param name="elem" select="."/>
    <xsl:if test="$elem/helpmsg">
      <img align="absmiddle" src="help_navbar_gif" onClick="{$elem/@name}_help_toggle()"/>
    </xsl:if>
  </xsl:template>

  <xsl:template name="handleIndent">
    <xsl:param name="indent" select="0"/>
    <xsl:variable name="indentby" select="$indentsize * $indent"/>
    <xsl:if test="$indent &gt; 0">
      <img src="spacer_gif" height="1" width="{$indentby}" border="0"/>
    </xsl:if>
  </xsl:template>

  <xsl:template match="option[ExclusiveSelect]" name="ExclusiveSelect">
    <xsl:param name="elem" select="."/>
    <xsl:param name="indent"/>
    <xsl:param name="triggerFunc" select="''"/>
      <xsl:if test="$show-indent">
        ExclusiveSelect
        OptName: <xsl:value-of select="$elem/@name"/>
        Indent: <xsl:value-of select="$indent"/>
      </xsl:if>
    <xsl:choose>
      <xsl:when test="$elem[@hidden='1']">
        <input type="hidden" id="{$elem/@name}" name="{$elem/@name}" value="{normalize-space($elem/ExclusiveSelect/currentValue)}"/>
      </xsl:when>
      <xsl:when test="$elem[@readonly='1']">
        <table width="640" border="0" cellspacing="0" cellpadding="7">
          <tr>
            <td width="{$fieldwidth}" class="listeven">
              <xsl:call-template name="handleIndent">
                <xsl:with-param name="indent" select="$indent"/>
              </xsl:call-template>
              <xsl:call-template name="handleTitle">
                <xsl:with-param name="elem" select="$elem"/>
              </xsl:call-template>
            </td>
            <td class="listeven">
              <input type="input" id="{$elem/@name}" name="{$elem/@name}" readonly="readonly" value="{normalize-space($elem/ExclusiveSelect/currentValue)}"/>
              <img align="absmiddle" src="help_navbar_gif" onClick="{$elem/@name}_help_toggle()"/>
            </td>
          </tr>
        </table>
        <xsl:call-template name="FieldHelp">
          <xsl:with-param name="elem" select="$elem"/>
         </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
    <table width="640" border="0" cellspacing="0" cellpadding="7">
    <tr>
      <td width="{$fieldwidth}" class="listeven">
        <xsl:call-template name="handleIndent">
          <xsl:with-param name="indent" select="$indent"/>
        </xsl:call-template>
        <xsl:call-template name="handleTitle">
          <xsl:with-param name="elem" select="$elem"/>
        </xsl:call-template>
      </td>
      <td class="listeven">
        <xsl:param name="selected" select="$elem/ExclusiveSelect/currentValue"/>
        <select name="{$elem/@name}" id="{$elem/@name}" onChange="{$triggerFunc}">
        <xsl:for-each select="$elem/ExclusiveSelect/value">
          <xsl:variable name="useValue">
            <xsl:choose>
              <xsl:when test="externalValue">
                <xsl:value-of select="externalValue"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="internalValue"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:variable>
          <xsl:choose>
            <xsl:when test="normalize-space(internalValue) = normalize-space($selected) or not(boolean($selected)) and @default = '1'">
              <option selected="selected" value="{normalize-space(internalValue)}"><xsl:value-of select="normalize-space($useValue)"/></option>
            </xsl:when>
            <xsl:otherwise>
              <option value="{normalize-space(internalValue)}"><xsl:value-of select="normalize-space($useValue)"/></option>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:for-each>
        </select><xsl:call-template name="handleHelp"><xsl:with-param name="elem" select="$elem"/></xsl:call-template>

      </td>
    </tr>
    </table>
    <xsl:call-template name="FieldHelp">
      <xsl:with-param name="elem" select="$elem"/>
    </xsl:call-template>
    </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="ExclusiveSelectAsJSArray">
    <xsl:param name="elem" select="."/>
        <script language="JavaScript">
        var <xsl:value-of select="$elem/@name"/> = Array();
        <xsl:for-each select="$elem/ExclusiveSelect/value">
          <xsl:variable name="key" select="normalize-space(externalValue)"/>
          <xsl:variable name="value" select="normalize-space(internalValue)"/>
          <xsl:value-of select="../../@name"/>['<xsl:value-of select="$key"/>'] = '<xsl:value-of select="$value"/>';
        </xsl:for-each>
        </script>
  </xsl:template>

  <xsl:template name="ExclusiveSelectAsJSArrayCache">
    <xsl:param name="elem" select="."/>
    <xsl:param name="default" select="''"/>
    <xsl:param name="selected" select="normalize-space($elem/ExclusiveSelect/currentValue)"/>
        <script language="JavaScript">
        var <xsl:value-of select="$elem/@name"/>_cache_pointer = '';
        var <xsl:value-of select="$elem/@name"/>_cache = Array();
        <xsl:for-each select="$elem/ExclusiveSelect/value">
          <xsl:if test="normalize-space(internalValue) = normalize-space($selected) or not(boolean($selected)) and @default = '1'">
            <xsl:value-of select="$elem/@name"/>_cache_pointer = '<xsl:value-of select="normalize-space(internalValue)"/>';
          </xsl:if>
          <xsl:variable name="key" select="normalize-space(externalValue)"/>
          <xsl:value-of select="../../@name"/>_cache['<xsl:value-of select="$key"/>'] = '<xsl:value-of select="$default"/>';
        </xsl:for-each>
        </script>
  </xsl:template>

  <xsl:template name="declareJSVar">
    <xsl:param name="var"/>
    <xsl:param name="value"/>
    <script language="JavaScript">
      var <xsl:value-of select="$var"/> = <xsl:value-of select="$value"/>;
    </script>
  </xsl:template>

  <xsl:template name="InstallLocJS">
    <xsl:param name="selectList"/>
    <xsl:param name="textBox"/>
    <xsl:param name="install_display"/>
    <xsl:param name="url_display"/>
    <xsl:param name="username"/>
    <xsl:param name="array"/>
    <xsl:param name="funcName"/>
    <script language="JavaScript">
        function <xsl:value-of select="$funcName"/>() {
          var selectList,textBox,domain,path,fullpath;
          var install_loc_div, url_loc_div, title_div;
          selectList = document.getElementById('<xsl:value-of select="$selectList"/>');
          userSelectList = document.getElementById('<xsl:value-of select="$username"/>');
          textBox = document.getElementById('<xsl:value-of select="$textBox"/>');
          textBoxDynPrefix = document.getElementById('<xsl:value-of select="$textBox"/>_dynprefix');
          install_loc_div = document.getElementById('<xsl:value-of select="$install_display"/>');
          url_loc_div = document.getElementById('<xsl:value-of select="$url_display"/>');
          title_div = document.getElementById('<xsl:value-of select="$textBox"/>_title');
          domain = selectList.options[selectList.selectedIndex].value;
          <xsl:value-of select="$array"/>_cache[<xsl:value-of select="$array"/>_cache_pointer] = textBox.value;
          <xsl:value-of select="$array"/>_cache_pointer = domain;
          textBox.value = <xsl:value-of select="$array"/>_cache[domain];
          lookup_path = <xsl:value-of select="$array"/>[domain];
          path = textBox.value;
          if (domain == '/var/www') {
            install_loc_div.innerHTML = '/var/www/' + path;
            title_div.innerHTML = '<xsl:call-template name="gettext">
                                     <xsl:with-param name="default" select="'Full installation path'"/>
                                   </xsl:call-template>';
            url_loc_div.innerHTML = '<xsl:call-template name="gettext">
                                       <xsl:with-param name="default" select="'(unknown)'"/>
                                     </xsl:call-template>';
            if(textBoxDynPrefix.innerHTML.substr(0,9) == '/var/www/') {
                textBoxDynPrefix.innerHTML = textBoxDynPrefix.innerHTML;
            } else {
                textBoxDynPrefix.innerHTML = '/var/www/' + textBoxDynPrefix.innerHTML;
            }
            if('options' in userSelectList) {
                userSelectList.disabled = 0;
            } else {
                userSelectList.readonly = 0;
            }
          } else if (domain == '/home') {
            var user;
            if ('options' in userSelectList) {
                user = userSelectList.options[userSelectList.selectedIndex].value;
            } else {
                user = userSelectList.value;
            }
            install_loc_div.innerHTML = '/home/' + user + '/public_html/' + path;
            title_div.innerHTML = '<xsl:call-template name="gettext">
                                     <xsl:with-param name="default" select="'URL path'"/>
                                   </xsl:call-template>';
            url_loc_div.innerHTML = 'http://' + top_level_domain + '/~' + user + '/' + path;

            if(textBoxDynPrefix.innerHTML.substr(0,9) == '/var/www/') {
                textBoxDynPrefix.innerHTML = textBoxDynPrefix.innerHTML.substr(9);
            } else {
                textBoxDynPrefix.innerHTML = textBoxDynPrefix.innerHTML;
            }

            if('options' in userSelectList) {
                userSelectList.disabled = 0;
            } else {
                userSelectList.readonly = 0;
            }
          } else {
            install_loc_div.innerHTML = lookup_path + path;
            title_div.innerHTML = '<xsl:call-template name="gettext">
                                     <xsl:with-param name="default" select="'URL path'"/>
                                   </xsl:call-template>';
            url_loc_div.innerHTML = 'http://' + domain + '/' + path;

            if(textBoxDynPrefix.innerHTML.substr(0,9) == '/var/www/') {
                textBoxDynPrefix.innerHTML = textBoxDynPrefix.innerHTML.substr(9);
            } else {
                textBoxDynPrefix.innerHTML = textBoxDynPrefix.innerHTML;
            }

            home_re = /^\/home\/([^\/]+)\//;
            result = home_re.exec(lookup_path);
            if(result) {
              var user = result[1];
              userSelectList.value = user;
              if('options' in userSelectList) {
                  userSelectList.disabled = 1;
              } else {
                  userSelectList.readonly = 1;
              }
            } else {
              if('options' in userSelectList) {
                  userSelectList.disabled = 0;
              } else {
                  userSelectList.readonly = 0;
              }
            }
          }
        }
    </script>
  </xsl:template>

  <xsl:template name="ExclusiveSelectAsTrigger">
    <xsl:param name="triggerFunc"/>
    <xsl:param name="indent"/>
      <xsl:if test="$show-indent">
        ExclusiveSelect
        OptName: <xsl:value-of select="@name"/>
        Indent: <xsl:value-of select="$indent"/>
      </xsl:if>
    <xsl:choose>
      <xsl:when test="@hidden='1'">
        <input type="hidden" name="{@name}" value="{normalize-space(ExclusiveSelect/currentValue)}"/>
      </xsl:when>
      <xsl:otherwise>
    <table width="640" border="0" cellspacing="0" cellpadding="7">
    <tr>
      <td width="{$fieldwidth}" class="listeven">
        <xsl:call-template name="handleIndent">
          <xsl:with-param name="indent" select="$indent"/>
        </xsl:call-template>
        <xsl:call-template name="handleTitle">
          <xsl:with-param name="elem" select="."/>
        </xsl:call-template>
      </td>
      <td class="listeven">
        <xsl:param name="selected" select="ExclusiveSelect/currentValue"/>
        <select name="{@name}" id="{@name}" onChange="{$triggerFunc}">
        <xsl:for-each select="ExclusiveSelect/value">
          <xsl:variable name="useValue">
            <xsl:choose>
              <xsl:when test="externalValue">
                <xsl:value-of select="externalValue"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="internalValue"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:variable>
          <xsl:choose>
            <xsl:when test="normalize-space(internalValue) = normalize-space($selected) or not(boolean($selected)) and @default = '1' ">
              <option selected="selected" value="{normalize-space(internalValue)}"><xsl:value-of select="normalize-space($useValue)"/></option>
            </xsl:when>
            <xsl:otherwise>
              <option value="{normalize-space(internalValue)}"><xsl:value-of select="normalize-space($useValue)"/></option>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:for-each>
        </select><xsl:call-template name="handleHelp"><xsl:with-param name="elem" select="$elem"/></xsl:call-template>
      </td>
    </tr>
    </table>
    <xsl:call-template name="FieldHelp"/>
    </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="option[MultiSelect]">
    <xsl:param name="indent"/>
      <xsl:if test="$show-indent">
        MultiSelect
        OptName: <xsl:value-of select="@name"/>
        Indent: <xsl:value-of select="$indent"/>
      </xsl:if>
    <table width="640" border="0" cellspacing="0" cellpadding="7">
    <tr>
      <td width="{$fieldwidth}">
        <xsl:call-template name="handleIndent">
          <xsl:with-param name="indent" select="$indent"/>
        </xsl:call-template>
        <xsl:call-template name="handleTitle">
          <xsl:with-param name="elem" select="."/>
        </xsl:call-template>
      </td>
      <td>
        <select name="{@name}" multiple="multiple">
	  <xsl:for-each select="MultiSelect/value">
	    <xsl:choose>
	      <xsl:when test="normalize-space(internalValue) = normalize-space(../currentValue) or not(../currentValue) and @default = '1' ">
		<option selected="selected" value="{normalize-space(internalValue)}">
		  <xsl:choose>
		    <xsl:when test="externalValue">
		      <xsl:value-of select="externalValue"/>
		    </xsl:when>
		    <xsl:otherwise>
		      <xsl:value-of select="internalValue"/>
		    </xsl:otherwise>
		  </xsl:choose>
                </option>
	      </xsl:when>
	      <xsl:otherwise>
		<option value="{normalize-space(internalValue)}">
		  <xsl:choose>
		    <xsl:when test="externalValue">
		      <xsl:value-of select="externalValue"/>
		    </xsl:when>
		    <xsl:otherwise>
		      <xsl:value-of select="internalValue"/>
		    </xsl:otherwise>
		  </xsl:choose>
                </option>
	      </xsl:otherwise>
	    </xsl:choose>
	  </xsl:for-each>
        </select>
      </td>
    </tr>
    </table>
        <xsl:call-template name="FieldHelp"/>
  </xsl:template>

  <xsl:template name="TextAsTriggerTarget">
    <xsl:param name="name" select="''"/>
    <xsl:param name="title" select="''"/>
    <xsl:param name="value" select="''"/>
    <xsl:param name="indent"/>
        <table width="640" border="0" cellspacing="0" cellpadding="7">
	<tr>
	  <td width="{$fieldwidth}" class="listeven">
            <xsl:call-template name="handleIndent">
              <xsl:with-param name="indent" select="$indent"/>
            </xsl:call-template>
            <xsl:call-template name="handleTitle">
              <xsl:with-param name="elem" select="."/>
            </xsl:call-template>
	  </td>
	  <td class="listeven">
            <p id="{$name}"><xsl:value-of select="$value"/></p>
	  </td>
	</tr>
        </table>
  </xsl:template>

  <xsl:template match="option[SingleInput]" name="SingleInput">
    <xsl:param name="indent"/>
    <xsl:param name="elem" select="."/>
    <xsl:param name="triggerFunc" select="''"/>
      <xsl:if test="$show-indent">
        SingleInput
        OptName: <xsl:value-of select="$elem/@name"/>
        Indent: <xsl:value-of select="$indent"/>
      </xsl:if>
    <xsl:choose>
      <xsl:when test="$elem/@hidden='1'">
        <input type="hidden" name="{$elem/@name}" value="{normalize-space($elem/SingleInput/currentValue)}"/>
      </xsl:when>
      <xsl:otherwise>
        <table width="640" border="0" cellspacing="0" cellpadding="7">
	<tr>
	  <td width="{$fieldwidth}" class="listeven">
            <xsl:call-template name="handleIndent">
              <xsl:with-param name="indent" select="$indent"/>
            </xsl:call-template>
            <xsl:call-template name="handleTitle">
              <xsl:with-param name="elem" select="$elem"/>
            </xsl:call-template>
	  </td>
	  <td class="listeven">
	    <xsl:value-of select="$elem/SingleInput/prefix"/>
            <xsl:variable name="value">
              <xsl:call-template name="removePrefixSuffix">
                <xsl:with-param name="prefix" select="normalize-space($elem/SingleInput/prefix)"/>
                <xsl:with-param name="suffix" select="normalize-space($elem/SingleInput/suffix)"/>
                <xsl:with-param name="string" select="normalize-space($elem/SingleInput/currentValue)"/>              
              </xsl:call-template>
            </xsl:variable>
            <xsl:variable name="type">
              <xsl:choose>
                <xsl:when test="$elem/SingleInput[@password = 1]">
                  <xsl:text>password</xsl:text>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:text>text</xsl:text>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:variable>
	    <xsl:choose>
	      <xsl:when test="$elem/@readonly = 1">
		<input type="hidden" name="{$elem/@name}" value="{$value}"/>
                <xsl:value-of select="$value"/>
	      </xsl:when>
	      <xsl:otherwise>
		<input type="{$type}" name="{$elem/@name}" id="{$elem/@name}" value="{$value}" onKeyUp="{$triggerFunc}"/>
                <xsl:call-template name="handleHelp"><xsl:with-param name="elem" select="$elem"/></xsl:call-template>
	      </xsl:otherwise>
	    </xsl:choose>
	    <xsl:value-of select="$elem/SingleInput/suffix"/>
	  </td>
	</tr>
        </table>
        <xsl:call-template name="FieldHelp"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="FieldHelp">
    <xsl:param name="elem" select="."/>
    <div id="{$elem/@name}_helpdiv" style="display:none;">
      <table width="640" border="0" cellspacing="0" cellpadding="7">
        <tr>
          <td width="{$fieldwidth}" class="listeven">
            &nbsp;
          </td>
          <td class="listeven">
            <font class="divHelp"><xsl:value-of select="$elem/helpmsg"/></font>
          </td>
        </tr>
      </table>
    </div>
    <script language="JavaScript">
      function <xsl:value-of select="$elem/@name"/>_help_toggle() {
        helpdiv = document.getElementById('<xsl:value-of select="$elem/@name"/>' + '_helpdiv');
        if (helpdiv.style.display == "inline") {
          helpdiv.style.display = "none";
        } else {
          helpdiv.style.display = "inline";
        }
      }
    </script>
  </xsl:template>
    
  <xsl:template name="ChangeableTitleSingleInput">
    <xsl:param name="elem" select="."/>
    <xsl:param name="indent"/>
    <xsl:param name="triggerFunc" select="''"/>
      <xsl:if test="$show-indent">
        SingleInput
        OptName: <xsl:value-of select="$elem/@name"/>
        Indent: <xsl:value-of select="$indent"/>
      </xsl:if>
    <xsl:choose>
      <xsl:when test="$elem[@hidden='1']">
        <input type="hidden" name="{$elem/@name}" value="{normalize-space($elem/SingleInput/currentValue)}"/>
      </xsl:when>
      <xsl:otherwise>
        <table width="640" border="0" cellspacing="0" cellpadding="7">
	<tr>
	  <td width="{$fieldwidth}" class="listeven">
	    <xsl:choose>
	      <xsl:when test="key('option-fail-index',$elem/@name)">
		<p id="{$elem/@name}_title"><font color="red"><xsl:value-of select="normalize-space($elem/title)"/></font></p>
	      </xsl:when>
	      <xsl:otherwise>
		<p id="{$elem/@name}_title"><xsl:value-of select="normalize-space($elem/title)"/></p>
	      </xsl:otherwise>
	    </xsl:choose>
	  </td>
	  <td class="listeven">
	    <xsl:value-of select="$elem/SingleInput/prefix"/>
            <xsl:variable name="value">
              <xsl:call-template name="removePrefixSuffix">
                <xsl:with-param name="prefix" select="normalize-space($elem/SingleInput/prefix)"/>
                <xsl:with-param name="suffix" select="normalize-space($elem/SingleInput/suffix)"/>
                <xsl:with-param name="string" select="normalize-space($elem/SingleInput/currentValue)"/>
              </xsl:call-template>
            </xsl:variable>
            <xsl:variable name="type">
              <xsl:choose>
                <xsl:when test="$elem/SingleInput[@password = 1]">
                  <xsl:text>password</xsl:text>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:text>text</xsl:text>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:variable>
            <table border="0" cellspacing="0" cellpadding="0">
              <tr>
                <td id="{$elem/@name}_dynprefix" class="listeven">

                </td>
                <td class="listeven">
	    <xsl:choose>
	      <xsl:when test="$elem[@readonly = 1]">
		<input type="hidden" name="{@name}" value="{$value}"/>
                <xsl:value-of select="$value"/>
	      </xsl:when>
	      <xsl:otherwise>
		<input type="{$type}" name="{$elem/@name}" id="{$elem/@name}" value="{$value}" onKeyUp="{$triggerFunc}"/>
                  <xsl:call-template name="handleHelp">
                    <xsl:with-param name="elem" select="$elem"/>
                  </xsl:call-template>
	      </xsl:otherwise>
	    </xsl:choose>
	    <xsl:value-of select="$elem/SingleInput/suffix"/>
                </td>
              </tr>
            </table>
	  </td>
	</tr>
        </table>
        <xsl:call-template name="FieldHelp">
          <xsl:with-param name="elem" select="$elem"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="option[MultiInput]">
    <xsl:param name="indent"/>
      <xsl:if test="$show-indent">
        MultiInput
        OptName: <xsl:value-of select="@name"/>
        Indent: <xsl:value-of select="$indent"/>
      </xsl:if>
    <table width="640" border="0" cellspacing="0" cellpadding="7">
    <tr>
      <td width="{$fieldwidth}">
        <xsl:call-template name="handleIndent">
          <xsl:with-param name="indent" select="$indent"/>
        </xsl:call-template>
        <xsl:call-template name="handleTitle">
          <xsl:with-param name="elem" select="."/>
        </xsl:call-template>
      </td>
      <td>
        <xsl:value-of select="MultiInput/prefix"/>
        <textarea name="{@name}" cols="50" rows="18">
        <xsl:value-of select="MultiInput/currentValue"/>
        </textarea>
        <xsl:value-of select="MultiInput/prefix"/>
      </td>
    </tr>
    </table>
        <xsl:call-template name="FieldHelp"/>
  </xsl:template>

  <xsl:template match="option[Label]">
    <xsl:param name="indent"/>
      <xsl:if test="$show-indent">
        Label
        OptName: <xsl:value-of select="@name"/>
        Indent: <xsl:value-of select="$indent"/>
      </xsl:if>
    <xsl:value-of select="@name"/>
  </xsl:template>

  <xsl:template name="licenseTextArea">
    <xsl:param name="elem" select="."/>
    <table width="640" border="0" cellspacing="0" cellpadding="7">
    <tr>
      <td class="listodd" align="left" colspan="4">
        <center>
          <xsl:value-of select="$elem/MultiInput/prefix"/>
          <textarea name="{$elem/@name}" cols="70" rows="24" readonly="readonly">
          <xsl:value-of select="$elem/MultiInput/currentValue"/>
          </textarea>
          <xsl:value-of select="$elem/MultiInput/suffix"/>
        </center>
      </td>
    </tr>
    </table>
  </xsl:template>

  <xsl:template name="licenseAccept">
    <xsl:param name="elem" select="."/>
    <xsl:call-template name="ExclusiveSelectAsRadio">
      <xsl:with-param name="elem" select="$elem"/>
      <xsl:with-param name="class" select="'listodd'"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="ExclusiveSelectAsRadio">
    <xsl:param name="elem" select="."/>
    <xsl:param name="class" select="'listeven'"/>
    <table width="640" border="0" cellspacing="0" cellpadding="7">
    <tr>
      <td class="{$class}" align="center" colspan="4">
        <xsl:param name="selected" select="$elem/ExclusiveSelect/currentValue"/>
        <xsl:for-each select="$elem/ExclusiveSelect/value">
          <xsl:variable name="useValue">
            <xsl:choose>
              <xsl:when test="externalValue">
                <xsl:value-of select="externalValue"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="internalValue"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:variable>
          <xsl:choose>
            <xsl:when test="normalize-space(internalValue) = normalize-space($selected) or not(boolean($selected)) and @default = '1' ">
              <input type="radio" checked="checked" name="{../../@name}" value="{normalize-space(internalValue)}"/><xsl:value-of select="normalize-space($useValue)"/>
            </xsl:when>
            <xsl:otherwise>
              <input type="radio" name="{../../@name}" value="{normalize-space(internalValue)}"/><xsl:value-of select="normalize-space($useValue)"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:for-each>
      </td>
    </tr>
    </table>
  </xsl:template>

  <xsl:template name="installToggle">
    <xsl:param name="install_locOption" select="key('parent-index',@name)[@name='install_loc']"/>
    <xsl:param name="url_netOption" select="key('parent-index',@name)[@name='url_net']"/>
    <xsl:param name="url_pathOption" select="key('parent-index',@name)[@name='url_path']"/>
    <xsl:apply-templates select="."/>
    <xsl:apply-templates select="$url_netOption"/>
    <xsl:apply-templates select="$url_pathOption"/>
    <xsl:apply-templates select="$install_locOption"/>
  </xsl:template>

  <xsl:template name="gettext">
    <xsl:param name="id" select="''"/>
    <xsl:param name="default" select="''"/>
    <xsl:choose>
      <xsl:when test="boolean($id) or boolean($default)">
    <xsl:choose>
      <xsl:when test="$trans != ''">
        <xsl:choose>
          <xsl:when test="$id != ''">
            <xsl:variable name="localization">
              <xsl:value-of select="document($trans)/strings/string[@id=$id]/msgstr"/>
            </xsl:variable>
            <xsl:choose>
              <xsl:when test="$localization != ''">
                <xsl:value-of select="$localization"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="$default"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>
          <xsl:otherwise>
            <xsl:variable name="localization">
              <xsl:value-of select="document($trans)/strings/string[string(msgid)=$default]/msgstr"/>
            </xsl:variable>
            <xsl:choose>
              <xsl:when test="$localization != ''">
                <xsl:value-of select="$localization"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="$default"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$default"/>
      </xsl:otherwise>
    </xsl:choose>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$default"/>
    </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="fancySyncFS">
    <xsl:param name="syncfs"/>
    <xsl:param name="overwritefiles"/>
    <xsl:call-template name="Boolean">
      <xsl:with-param name="elem" select="$syncfs"/>
      <xsl:with-param name="triggerFunc" select="'fancySyncFSUpdate()'"/>
    </xsl:call-template>
    <xsl:call-template name="Boolean">
      <xsl:with-param name="elem" select="$overwritefiles"/>
      <xsl:with-param name="indent" select="1"/>
    </xsl:call-template>
    <script language="JavaScript">
      syncBoxElem = document.getElementById('<xsl:value-of select="$syncfs/@name"/>');
      overwriteElem = document.getElementById('<xsl:value-of select="$overwritefiles/@name"/>');
      overwriteElem.disabled = true;
      function fancySyncFSUpdate() {
        if (syncBoxElem.checked) {
          overwriteElem.disabled = false;
        } else {
          overwriteElem.disabled = true;
          overwriteElem.checked = false;
        }
      }
    </script>
  </xsl:template>

  <xsl:template name="fancyInstall">
    <xsl:param name="domain_to_path"/>
    <xsl:param name="url_net"/>
    <xsl:param name="url_path"/>
    <xsl:param name="username"/>
    <xsl:call-template name="declareJSVar">
      <xsl:with-param name="var" select="'top_level_domain'"/>
      <xsl:with-param name="value">
        <xsl:text>'</xsl:text>
        <xsl:value-of select="normalize-space($domain_to_path/ExclusiveSelect/value/externalValue)"/>
        <xsl:text>'</xsl:text>
      </xsl:with-param>
    </xsl:call-template>
    <xsl:call-template name="ExclusiveSelectAsJSArray">
      <xsl:with-param name="elem" select="$domain_to_path"/>
    </xsl:call-template>
    <xsl:call-template name="ExclusiveSelectAsJSArrayCache">
      <xsl:with-param name="elem" select="$domain_to_path"/>
      <xsl:with-param name="default">
        <xsl:value-of select="normalize-space($url_path/SingleInput/currentValue)"/>
      </xsl:with-param>
    </xsl:call-template>
    <xsl:call-template name="InstallLocJS">
      <xsl:with-param name="selectList" select="'url_net'"/>
      <xsl:with-param name="textBox" select="'url_path'"/>
      <xsl:with-param name="install_display" select="'install_loc_display'"/>
      <xsl:with-param name="url_display" select="'url_loc_display'"/>
      <xsl:with-param name="username" select="'username'"/>
      <xsl:with-param name="array" select="'domain_to_path'"/>
      <xsl:with-param name="funcName" select="'update_install_display'"/>
    </xsl:call-template>
    <xsl:call-template name="ExclusiveSelect">
      <xsl:with-param name="elem" select="$url_net"/>
      <xsl:with-param name="triggerFunc" select="'update_install_display()'"/>
    </xsl:call-template>
    <xsl:call-template name="ChangeableTitleSingleInput">
      <xsl:with-param name="elem" select="$url_path"/>
      <xsl:with-param name="triggerFunc" select="'update_install_display()'"/>
    </xsl:call-template>
    <xsl:call-template name="ExclusiveSelect">
      <xsl:with-param name="elem" select="$username"/>
      <xsl:with-param name="triggerFunc" select="'update_install_display()'"/>
    </xsl:call-template>
    <xsl:call-template name="TextAsTriggerTarget">
      <xsl:with-param name="name" select="'url_loc_display'"/>
      <xsl:with-param name="title">
        <xsl:call-template name="gettext">
          <xsl:with-param name="default" select="'URL to access tool'"/>
        </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
    <xsl:call-template name="TextAsTriggerTarget">
      <xsl:with-param name="name" select="'install_loc_display'"/>
      <xsl:with-param name="title">
        <xsl:call-template name="gettext">
          <xsl:with-param name="default" select="'Full installation path'"/>
        </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
    <script language="JavaScript">
      <xsl:text>update_install_display();</xsl:text>
    </script>    
  </xsl:template>

  <xsl:template name="license">
    <xsl:param name="text"/>
    <xsl:param name="accept"/>
    <xsl:call-template name="licenseTextArea">
      <xsl:with-param name="elem" select="$text"/>
    </xsl:call-template>
    <xsl:call-template name="licenseAccept">
      <xsl:with-param name="elem" select="$accept"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template match="option[@name='domain_to_path']">
    <xsl:call-template name="fancyInstall">
      <xsl:with-param name="domain_to_path" select="."/>
      <xsl:with-param name="url_net" select="../option[@name='url_net']"/>
      <xsl:with-param name="url_path" select="../option[@name='url_path']"/>
      <xsl:with-param name="username" select="../option[@name='username']"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template match="option[@name='syncfs']">
    <xsl:call-template name="fancySyncFS">
      <xsl:with-param name="syncfs" select="../option[@name='syncfs']"/>
      <xsl:with-param name="overwritefiles" select="../option[@name='overwritefiles']"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template match="option[@name='license']">
    <xsl:call-template name="license">
      <xsl:with-param name="text" select="."/>
      <xsl:with-param name="accept" select="../option[@name='accept_lic']"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template match="option[@name='overwritefiles' or @name='url_net' or @name='url_path' or @name='accept_lic' or @name='username']"/>

  <xsl:template name="search-and-replace">
    <xsl:param name="input" select="."/>
    <xsl:param name="search-string"/>
    <xsl:param name="replace-string"/>
    <xsl:choose>
      <xsl:when test="$search-string and contains($input, $search-string)">
        <xsl:value-of select="substring-before($input, $search-string)"/>
        <xsl:value-of select="$replace-string"/>
        <xsl:call-template name="search-and-replace">
          <xsl:with-param name="input" select="substring-after($input, $search-string)"/>
          <xsl:with-param name="search-string" select="$search-string"/>
          <xsl:with-param name="replace-string" select="$replace-string"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$input"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
</xsl:stylesheet>
