#
# Copyright (c) Ensim Corporation, 2003   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#
import deployables_lib as dl
import urlparse
import shutil
import os

dl.load_msgs_module("phpbb-2.0.6", symtab = globals())

dir_moves = [('phpBB2', '') ]
file_mods = []
source_patches = [{'file' : 'phpbb-xss.patch.ensim'}, 
                  {'file' : 'phpbb-search.patch.ensim'},
                  {'file' : 'phpbb-xss_group_mod.patch.ensim'}]
upgrade_patches = [{'file' : 'phpbb-xss.patch.ensim', 'args' : '-N -s -p2'},
                   {'file' : 'phpbb-search.patch.ensim', 'args' : '-N -s -p2'},
                   {'file' : 'phpbb-xss_group_mod.patch.ensim',
                    'args' : '-N -s -p2'}]
instance_patches = []

def install(site, site_conf, instance_conf, msgs):

    actions = ['resolve_location', 'check_disk_quota', 'check_mysql_enabled', 
	       'check_pass', 'check_install_loc_owner', '']
    args_dict = {}
    dl.do_install_actions(site, site_conf, instance_conf, msgs,
                          globals(), actions, args_dict)
    if msgs.get_severity() != msgs.ERROR:
        if instance_conf['url_loc']:
            split = urlparse.urlsplit(instance_conf['url_loc'])
            server_name = split[1]
            script_path = split[2]
        else:
            server_name = script_path = ''
        actions = ['db_create', 'db_probe', 'archive_unpack', 'check_index',
                   'exec_php_script', 'handle_security_mode', 'remove_files',
                   'access_url']
        args_dict = {
            'archive_unpack':       { 'dir_moves': dir_moves,
                                      'patches' : source_patches },
            'handle_security_mode': { 'files': ['config.php'] },
            'exec_php_script' : {
                 'script' : 'install/install.php',
                 'args' : {'install_step' : '1',
			   'lang' : 'english',
			   'dbms' : 'mysql',
			   'upgrade' : '0',
                           'server_name' : server_name,
			   'server_port' : '80',
			   'script_path' : script_path,
                           'dbhost' : dl.get_mysql_host(site),
                           'dbname' : instance_conf['dbname'],
                           'dbuser' : site_conf['mysql']['dbaseadmin'],
                           'dbpasswd' : instance_conf['dbpass'],
                           'prefix' : '',
                           'admin_name' : instance_conf['instance_admin'],
                           'admin_pass1' : instance_conf['instance_pass1'],
                           'admin_pass2' : instance_conf['instance_pass1'],
                           'board_email':instance_conf['instance_admin_email']},
                 'success_re' : '.*your basic installation is complete',
                 },
            'remove_files': { 'files': ['install/', 'contrib/'] },
            }
        dl.do_install_actions(site, site_conf, instance_conf, msgs,
                              globals(), actions, args_dict)

def reconfigure(context, site, site_conf, old_site_conf, instance_conf,
                old_instance_conf, msgs):

    actions = ['handle_url_loc', 'resolve_location', '',
               'archive_unpack', 'handle_username_change', 'handle_db_changes']
    args_dict = {
        'archive_unpack': { 'dir_moves': dir_moves,
                            'patches' : source_patches,
                            'fileinfo':
                            { 'excluded_files':
                              ['config.php', 'contrib', 'install'] } },
        }
    if context != dl.CONTEXT_SITE_EDIT:
        # We always require the dbpass for phpbb, since a reconfigure
        # may need to peform database updates
        actions.insert(0, 'db_probe')
        args_dict['db_probe'] = {'use_old' : 1}
    dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                              instance_conf, old_instance_conf, msgs,
                              globals(), actions, args_dict)

    if msgs.get_severity() != msgs.ERROR:
        pat = "^\s*\$(?P<key>\S+)\s*=\s*(?P<value>'.*')\s*;\s*$"
        sub = "$%(key)s = %(value)s;"
        vars = []
        if dl.mysql_installed(site_conf, old_site_conf):
            vars.extend([('dbname', instance_conf['dbname'],
                          old_instance_conf['dbname']),
                         ('dbuser', site_conf['mysql']['dbaseadmin'],
                          old_site_conf['mysql']['dbaseadmin'])])
        if context != dl.CONTEXT_SITE_EDIT:
            # We will only change the mysql host if a user specifically
            # invokes reconfigure; we don't want this change to happen
            # during upgrade, when all the sites are edited, but the
            # mysql.sock file hasn't moved yet.
            vars.append(('dbhost', dl.get_mysql_host(site)))
            vars.append(('dbpasswd', instance_conf['dbpass']))
        actions = ['edit_conf_file', 'apply_patch', 'handle_security_mode']
        args_dict = {
            'edit_conf_file': { 'file': 'config.php', 'lang': 'php',
                                'pat': pat, 'sub': sub, 'vars': vars },
            'apply_patch': { 'patches': instance_patches },
            'handle_security_mode': { 'files': ['config.php'] },
            }
        
        # XXX We want to do this on site edit, too, but we don't have
        # the mysql password. As it stands, chaning the domain name
        # during site edit will break any phpbb instances. The
        # workaround is to reconfigure any phpbb instances afterwards,
        # which will trigger the url_loc change clause below.
        if context != dl.CONTEXT_SITE_EDIT:
            comps = urlparse.urlsplit(instance_conf['url_loc'])
            sql_stmt = [ \
                "UPDATE config SET config_value = '%s' " % comps[1] + \
                "WHERE config_name = 'server_name'",
                "UPDATE config SET config_value = '%s' " % comps[2] + \
                "WHERE config_name = 'script_path'" 
                ]
            actions.append('exec_mysql')
            args_dict['exec_mysql'] = { 'sql': sql_stmt }
        dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                                  instance_conf, old_instance_conf, msgs,
                                  globals(), actions, args_dict)
        
def upgrade(site, site_conf, instance_conf, old_instance_conf, msgs):
    if old_instance_conf['shortname'] == 'phpbb' and \
       old_instance_conf['version'] == '2.0.6' and \
       old_instance_conf['dapiversion'] == '1.0':
        if old_instance_conf['dapirelease'] == '28':
            actions = ['apply_patch']
            args_dict = {'apply_patch' : {'patches' : upgrade_patches}}
            instance_conf['applypatches'] = '1'
            dl.do_upgrade_actions(site, site_conf, instance_conf,
                                  old_instance_conf, msgs,
                                  globals(), actions, args_dict)
        elif old_instance_conf['dapirelease'] == '34':
            actions = ['apply_patch']
            args_dict = {'apply_patch' : {'patches' : upgrade_patches[1:]}}
            instance_conf['applypatches'] = '1'
            dl.do_upgrade_actions(site, site_conf, instance_conf,
                                  old_instance_conf, msgs,
                                  globals(), actions, args_dict)
        elif old_instance_conf['dapirelease'] in ('36', '44'):
            actions = ['apply_patch']
            args_dict = {'apply_patch' : {'patches' : upgrade_patches[2:]}}
            instance_conf['applypatches'] = '1'
            dl.do_upgrade_actions(site, site_conf, instance_conf,
                                  old_instance_conf, msgs,
                                  globals(), actions, args_dict)
      
def remove(site, site_conf, instance_conf, msgs):
    dl.db_drop(site, site_conf, instance_conf)
    dl.install_loc_cleanup(site, site_conf, instance_conf, dir_moves,
                           source_patches)
    
def get_instance_configuration(context, site, site_conf, instance_conf,
                               old_instance_conf, locale, msgs):
    instance = dl.dsp_create_instance(instance_conf['shortname'],
                                      instance_conf['version'])
    dl.dsp_fill_instance(context, instance, site, site_conf,
                         instance_conf, old_instance_conf,
                         ['dbname', 'username', 'url_net', 'url_path',
                          'domain_to_path', 'instance_admin_email'])
    dl.enable_patches(instance, instance_patches)
    return instance

def regression_test(action, index, test_data):
    return dl.regression_check_index(action, index, test_data,
                                     'A _little_ text to describe your forum')
