#
# Copyright (c) Ensim Corporation 2000, 2001   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#

use strict;
package SSL;

BEGIN {
require SSLconf;
push @INC, ($ENV{'OCW_SVCPATH'} || "/usr/lib/opcenter")."/virtualhosting";
$SSL::SSLCmd = $SSLconf::config{'command'};
$SSL::SSLKeyCmd = $SSLconf::config{'key_command'};
$SSL::CertFile = $SSLconf::config{'cert_file'};
$SSL::RequestFile = $SSLconf::config{'req_file'};
$SSL::KeyFile = $SSLconf::config{'key_file'};
$SSL::ConfFile = $SSLconf::config{'conf_file'};
$SSL::KeyModulus = "";
$SSL::CertModulus = "";
}

END {}

return 1;

use lib ($ENV{OCW_SVCPATH} or "/usr/lib/opcenter") . "/apache";
use lib ($ENV{OCW_SVCPATH} or "/usr/lib/opcenter") . "/cmdline_common";

use ERRORS;
use Carper;

sub setDomainInfo(){
  my ($domain) = $_[0];  
  my ($err);
  my ($record,$vroot);
  if ($domain ne "None") {
    require VHConfig;
    require VHConst;
    if ($err = &VHConfig::CheckDomain($domain))       
      {return $err; }
    if ($err = &VHConfig::GetSingleDomainConf($domain,\$record))
      {return $err;} 
    $vroot = "$VHConst::VIRTUALFS/$record->{SITEINFO}->{User}";
    $SSL::RequestFile = "$vroot" . $SSLconf::config{'req_file'};
    $SSL::KeyFile = "$vroot" . $SSLconf::config{'key_file'};
    $SSL::CertFile = "$vroot" .  $SSLconf::config{'cert_file'};
  } else {
    $SSL::RequestFile = $SSLconf::config{'req_file'};
    $SSL::KeyFile = $SSLconf::config{'key_file'};
    $SSL::CertFile = $SSLconf::config{'cert_file'};
  }
  return "";
}

sub VerifyKeyFile($) {
    my ($file)=@_;
    system ("$SSL::SSLKeyCmd rsa -in $file >/dev/null 2>/dev/null");
    if ($? != 0) {
        $cerr += $E_INCORRECTKEYFILE and $cerr += {file => $file};
        return "Incorrect key file $file .";
    } else {
        return "";
    }
}

sub ReadKeyModulus() {
    my($file)=@_;
    open (KEY,"$SSL::SSLKeyCmd rsa -in $file -text 2>/dev/null|") or
      $cerr += $E_KEYFILEPARSEPROBLEM and $cerr += {file => $file} and
      return "Problems parsing key file $file .\n";
    my($line,$found)=("","");
    $SSL::KeyModulus = "";
    while($line=<KEY>){
        if (not $found) {
            next if ($line !~ /^\s*modulus/i);
            $found = 1;
        } elsif ($line =~ /^\s*([0-9a-f:]+)$/i) {
            $SSL::KeyModulus .= $1;
        } else {
            last;
        }
    }
    close (KEY) or 
      return "Problems with key file $file .\n";
    if (not $found) {
        $cerr += $E_NOMODULUS and $cerr += { file => $file } ;
        return "Could not find modulus in key file $file .\n";}
    if (not $SSL::KeyModulus) {
        $cerr += $E_EMPTYMODULUS ;
        return "Modulus empty.\n";}
    return "";
}

sub ReadCertModulus() {
    my($file)=@_;
    open (CRT,"$SSL::SSLCmd x509 -text < $file 2>/dev/null|") or
      $cerr += $E_CERTFILEPARSEPROBLEM and $cerr += { file => $file } and
      return "Problems parsing cerificate file $file .\n";
    my($line,$found)=("","");
    $SSL::CertModulus = "";
    while($line=<CRT>){
        if (not $found) {
            next if ($line !~ /^\s*modulus/i);
            $found = 1;
        } elsif ($line =~ /^\s*([0-9a-f:]+)$/i) {
            $SSL::CertModulus .= $1;
        } else {
            last;
        }
    }
    close (CRT) or 
      $cerr += $E_CERTFILEPROBLEM and $cerr += { file => $file } and
      return "Problems with certificate file $file .\n";
    if (not $found) {
        $cerr += $E_NOMODULUSINCERT and $cerr += { file => $file };
        return "No modulus in certificate file $file .\n";}
    if (not $SSL::CertModulus) {
        $cerr += $E_EMPTYMODULUS ;
        return "Modulus empty.\n";}
    return "";
}

sub VerifyCertFile($) {
    my ($file)=@_;
    system ("$SSL::SSLCmd x509 < $file >/dev/null 2>/dev/null");
    if ($? != 0) {
        $cerr += $E_INCORRECTCERT and $cerr += { file => $file };
        return "Incorrect Certificate file $file .";
    } else {
        return "";
    }
}

sub VerifyMatchKeyCert($$) {
    my ($key,$cert) = @_;
    my ($error_message);
    if ($error_message=&SSL::ReadKeyModulus($key)) {return $error_message}
    if ($error_message=&SSL::ReadCertModulus($cert)) {return $error_message}
    if ($SSL::KeyModulus ne $SSL::CertModulus) {
        $cerr += $E_KEYCERTNOMATCH and $cerr += { key => $key, cert => $cert };
        return "The key $key and the certificate $cert do not match.\n";
    }
    return "";
}

# retrieves a specific certificate or request or key
# makes the call: open_ssl x509 -text -noout and takes input from a cert file
sub getCertificate() {
    my($info) = @_;

    if ($info eq "key") {
	    my(@key) = ();
	    if(open(CERT,"$SSL::KeyFile")){
	        @key = <CERT>;
	        close(CERT);
	    }
	    return \@key;
    } elsif ($info eq "request") {
	    my(@request) = ();
	    if (open(CERT,"$SSL::RequestFile")) {
	        @request = <CERT>;
	        close(CERT);
	    }
	    return \@request;
    } elsif ($info eq "certificate") {
	    my(@certificate) = ();
	    if(open(CERT,"$SSL::CertFile")){
	        @certificate = <CERT>;
	        close(CERT);
	    }
	    return \@certificate;
    } elsif (($info eq "subject") || ($info eq "issuer")) {
            my($temp) = {} ;
	    my($i,$left,$right);

	    unless ( -r $SSL::CertFile ) {
	        return {"Country" => "",
		        "State" => "",
		        "City" => "",
		        "Company" => "",
		        "Organization" => "",
		        "Common" => "",
		        "Email" => "",
		};
	    }

	    open(CERT,"$SSL::SSLCmd x509 -subject -noout -nameopt sep_multiline < $SSL::CertFile |") or 
           $cerr += $E_CANNOTANALYZECERT and $cerr << { cert => $SSL::CertFile, cmd => $SSL::SSLCmd } and 
	       return "Cannot analyze $SSL::CertFile for subject with $SSL::SSLCmd .\n";
	    my(@data) = <CERT>;
	    close(CERT) or $cerr += $E_CERTANALYZEERROR and $cerr << { cert => $SSL::CertFile } and return "Errors while analyzing $SSL::CertFile .\n";
	    foreach $i (@data) {
		($left,$right) = split(/=/,$i) ;

		# Trim spaces
                $left =~ s/^\s+// ;  # from the front
		$left =~ s/\s+$// ;  # and from the back
		$right =~ s/^\s+// ;
		$right =~ s/\s+$// ;

		if ($left eq "C")	{ $temp->{Country} = $right ; }
		if ($left eq "ST")	{ $temp->{State} = $right ; }
		if ($left eq "L")	{ $temp->{City} = $right ; }
		if ($left eq "O")	{ $temp->{Company} = $right ; }
		if ($left eq "OU")	{ $temp->{Organization} = $right ; }
		if ($left eq "CN")	{ $temp->{Common} = $right ; }
		if ($left eq "Email")	{ $temp->{Email} = $right ; }
	    }
	    return $temp ;
    } else {
	    my($issuer) = {};
	    my($subject) = {};
	    unless ( -r $SSL::CertFile ) {
	        return {"Country" => "",
		    "State" => "",
		    "City" => "",
		    "Company" => "",
		    "Organization" => "",
		    "Common" => "",
		    "Email" => "",
		   };
	}
	my($i,$j,@toks,@ltoks,$item);
	open(CERT,"$SSL::SSLCmd x509 -text -noout < $SSL::CertFile |") or 
	   return "Cannot analyze $SSL::CertFile with $SSL::SSLCmd .\n";
	my(@data) = <CERT>;
	close(CERT) or $cerr += $E_CERTANALYZEERROR and $cerr += { cert => $SSL::CertFile } and return "Errors while analyzing $SSL::CertFile .\n";
	foreach $i (@data) {
	    @toks = split(' ',$i);
	    $item = shift @toks;
	    my($commonName);
	    if ($item eq "Issuer:") {
		@ltoks = split(', ',join(' ',@toks));
		(undef,$issuer->{Country}) = split('=',$ltoks[0]);
		(undef,$issuer->{State}) = split('=',$ltoks[1]);
		(undef,$issuer->{City}) = split('=',$ltoks[2]);
		(undef,$issuer->{Company}) = split('=',$ltoks[3]);
		(undef,$issuer->{Organization}) = split('=',$ltoks[4]);
		($commonName,undef) = split('/',$ltoks[5]);
		(undef,$issuer->{Common}) = split('=',$commonName);
		(undef,undef,$issuer->{Email}) = split('=',$ltoks[5]);
	    }
	    if ($item eq "Subject:") {
		@ltoks = split(', ',join(' ',@toks));
		(undef,$subject->{Country}) = split('=',$ltoks[0]);
		(undef,$subject->{State}) = split('=',$ltoks[1]);
		(undef,$subject->{City}) = split('=',$ltoks[2]);
		(undef,$subject->{Company}) = split('=',$ltoks[3]);
		(undef,$subject->{Organization}) = split('=',$ltoks[4]);
		($commonName,undef) = split('/',$ltoks[5]);
		(undef,$subject->{Common}) = split('=',$commonName);
		(undef,undef,$subject->{Email}) = split('=',$ltoks[5]);
	    }
	}
	if ($info eq "subject") {
	    return $subject;
	} else {
	    return $issuer;
	}
    }
}

# saves a certificate for a specified host and makes a backup of the old
# certificate if it exists
sub saveCertificate() {
    my ($certificate) = @_;
    my($key,$cert,$i,$error_message);
    #split the key from the certificate if it is in there
    my(@toks) = split(/\n/,$certificate);
    my($inkey) = 0;
    my($incert) = 0;
    foreach $i (@toks) {
        if ($i eq "-----END RSA PRIVATE KEY-----") {
            $key .= "$i\n";
            $inkey = 0;
            $incert= 0;
        }
        elsif ($i eq "-----END CERTIFICATE-----") {
            $cert.= "$i\n";
            $inkey = 0;
            $incert= 0;
        }
        elsif ($i eq "-----BEGIN RSA PRIVATE KEY-----" || $inkey) {
            $key .= "$i\n";
            $inkey = 1;
            $incert= 0;
        } elsif ($i eq "-----BEGIN CERTIFICATE-----" || $incert) {
            $cert .= "$i\n";
            $incert= 1;
            $inkey = 0;
        }
    }
    $cerr += $E_NOCERTORKEY and return "No key or certificate supplied.\n" if (not $key and not $cert);
    if ($key) {
        # save the key into a temporary file
        open (KEY,">$SSL::KeyFile.new") or
            $cerr += $E_UNABLESAVEKEY and 
          return "Unable to save key.";
        print KEY $key;
        close (KEY);
        $error_message = &SSL::VerifyKeyFile("$SSL::KeyFile.new");
        if ($error_message) {return $error_message;}
    }
    if ($cert) {
	# save the certificate into a remporary file
	open (CERTIFICATE,">$SSL::CertFile.new") or
        $cerr += $E_UNABLESAVECERT and 
	  return "Unable to save certificate.";
	print CERTIFICATE $cert;
	close (CERTIFICATE); 
        $error_message = &SSL::VerifyCertFile ("$SSL::CertFile.new");
        if ($error_message) {return $error_message;}
    }
    #see if they match
    my ($keyfile,$certfile);
    $keyfile=($key?"$SSL::KeyFile.new":$SSL::KeyFile);
    $certfile=($cert?"$SSL::CertFile.new":$SSL::CertFile);
    $error_message = &SSL::VerifyMatchKeyCert ($keyfile,$certfile);
    if ($error_message) {return $error_message;}
    #now that we are sure the new stuff is good, we copy it over
    if ($key) {
        unlink ($SSL::KeyFile);
        rename ("$SSL::KeyFile.new",$SSL::KeyFile);
    }
    if ($cert) {
        unlink ($SSL::CertFile);
        rename ("$SSL::CertFile.new",$SSL::CertFile);
    }
    return "";
}


sub removeCertificate() {
}

sub createCertificate() {
    my($identC,$identST,$identL,$identO,$identOU,$identCN,$identEmail,$days) = @_;
    # Check the email address
    if ( $identEmail !~ /^[a-zA-Z0-9-\.-\_]+\@[a-zA-Z0-9-\.]+\.[a-zA-Z]+$/ ) {
        $cerr += $E_INVALIDEMAIL and $cerr += { email => $identEmail };
        return "Invalid email address $identEmail .\n";
    }
    # generate the key file
    system("$SSL::SSLKeyCmd genrsa -out $SSL::KeyFile.new 1024 >/dev/null 2>/dev/null");
    if ($? != 0) {
        $cerr += $E_CANNOTGENERATEKEY ;
        return "Cannot generate key.\n";
    }
    # generate the request file
    open (REQ, "|$SSL::SSLCmd req -new -config $SSL::ConfFile -key $SSL::KeyFile.new -days $days -out $SSL::RequestFile.new > /dev/null 2> /dev/null")  or
        $cerr += $E_CANNOTGENERATECERTREQ and 
      return "Cannot generate certificate request. \n";
    print REQ <<CERTINFO;
$identC
$identST
$identL
$identO
$identOU
$identCN
$identEmail
.
.
CERTINFO
;
    close (REQ) or 
        $cerr += $E_ERRORGENERATEREQ and
      return "Errors generating certificate request. \n";
    # generate the certificate file
    if( system("$SSL::SSLCmd x509 -days $days -req -signkey $SSL::KeyFile.new "
	       ."-in $SSL::RequestFile.new -out $SSL::CertFile.new > /dev/null 2> /dev/null") ) {
        $cerr += $E_UNABLEGENERATECERT ;
        return "Unable to generate the certificate." 
    }
    unlink ($SSL::KeyFile);
    rename ("$SSL::KeyFile.new",$SSL::KeyFile);
    unlink ($SSL::RequestFile);
    rename ("$SSL::RequestFile.new",$SSL::RequestFile);
    unlink ($SSL::CertFile);
    rename ("$SSL::CertFile.new",$SSL::CertFile);
    return "";
}


return 1;
