# --------------------------------------------------------------------------
###
#
# Copyright (c) Ensim Corporation 2000, 2001   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#
# $Id: BindOptions.pm,v 1.3 2002/04/12 15:25:57 adityap Exp $
# $Name:  $
# --------------------------------------------------------------------------

###
# This module provides an interface to manipulate BIND options statement
###


package BindOptions;

use strict;

require Exporter;
our @ISA = qw (Exporter);
our @EXPORT_OK = qw (addListenOnIPAddr removeListenOnIPAddr);

use lib ($ENV{'OCW_SVCPATH'} || "/usr/lib/opcenter")."/cmdline_common";

use LogConfFile;

use constant BIND_OPTIONS_FILE_DIR => "/etc/bind";
use constant BIND_OPTIONS_FILE_NAME => "options.conf.wp";
use constant BIND_OPTIONS_FILE => BIND_OPTIONS_FILE_DIR . "/" . BIND_OPTIONS_FILE_NAME;

use lib ($ENV{OCW_SVCPATH} or "/usr/lib/opcenter") . "/bind";
use lib ($ENV{OCW_SVCPATH} or "/usr/lib/opcenter") . "/cmdline_common";

use ERRORS;
use Carper;


###
# add an IP Address to listen-on clause. Assume default port (53)
###
sub addListenOnIPAddr ($) {
    my $ipAddr = shift;
    
    return _handleListenOnCommon (sub { ($_[0] =~ /\Q$ipAddr\E\s*;/)? 
					    $_[0] : ($_[0] . " $ipAddr; ");});
}


###
# remove an IP Address from listen-on clause. Assume default port (53)
###
sub removeListenOnIPAddr ($) {
    my $ipAddr = shift;

    return _handleListenOnCommon (sub { 
	my $addrList = shift; 
	$addrList =~ s/\s*\Q$ipAddr\E\s*;//;
	chomp $addrList;
	return $addrList;
    });
}


############################### PRIVATE SUBS ##################################

sub _handleListenOnCommon (&) {
    my $getAddrListSubRef = shift;

    my $inputHandle  = new IO::File (BIND_OPTIONS_FILE, "r");
    if (! defined $inputHandle) {
        $cerr += $E_UNABLEREADFILE and $cerr += { file => BIND_OPTIONS_FILE } ;
	return "Unable to open for read: " . BIND_OPTIONS_FILE;
    }
					       
    my $buf;

    my @lines = ();
    # find the first line
    my $statementHead = -1;
    my $listenOnPos = -1;
    my $addrList;
    while ($buf = <$inputHandle>) 
    {
    	if ($listenOnPos == -1) 
        {
	        if ($buf =~ /^\s*listen\-on\s+(port\s+53\s+)?{(.*)};$/i) 
            {
                $listenOnPos = scalar @lines;
                $addrList = $2;
    	    }
	    }
        if ($statementHead == -1) 
        {
	        if ($buf =~ /^\s*options\s+\{/i) 
            {
	    	    $statementHead = scalar @lines;
	        }
	    }
	    push @lines, $buf;
    }

    if ($listenOnPos == -1) {
	my $listenOnLine = sprintf ("\tlisten-on { %s };\n", &$getAddrListSubRef (""));
	splice @lines, $statementHead+1, 0, $listenOnLine;
    } else {
	my $newAddrList = &$getAddrListSubRef ($addrList);
	
	###
	# empty string -> no IP Address, need to delete the line
	###
	if ($newAddrList) {
	    if ($newAddrList eq $addrList) {
		# nothing to be done
		return "";
	    }
	    $lines[$listenOnPos] =~ s/{\Q$addrList\E};$/{$newAddrList};/;
	} else {
	    $lines[$listenOnPos] = "\n";
	}
    }

    #output phase
    my $outputHandle = LogConfFile::open_conf (BIND_OPTIONS_FILE_DIR,
					       "/var/log/conf",
					       BIND_OPTIONS_FILE_NAME, "w");
    if (!$outputHandle) {
        $cerr += $E_UNABLEWRITEFILE and $cerr += { file => BIND_OPTIONS_FILE };
	return "Unable to write to file: " . BIND_OPTIONS_FILE;
    }
					       
    foreach my $line (@lines) {
	print $outputHandle $line;
    }

    if (! LogConfFile::close_conf ($outputHandle, BIND_OPTIONS_FILE_DIR,
				   "/var/log/conf", BIND_OPTIONS_FILE_NAME)) {
        $cerr += $E_UNABLEUPDATEFILE and $cerr += { file => BIND_OPTIONS_FILE };
        return "Unable to update " . BIND_OPTIONS_FILE;
    }
}
