# Copyright (c) Ensim Corporation 2000, 2001, 2000   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#
# This module is used to set/get default SOA values

package DnsSoaDefaults;

use strict;

require Exporter;
our @ISA = qw (Exporter);
our @EXPORT_OK = qw(getSoaValues setSoaValues);

use IO::File;
use lib ($ENV{'OCW_SVCPATH'} || "/usr/lib/opcenter") . "/cmdline_common";
use Locking;

use lib ($ENV{OCW_SVCPATH} or "/usr/lib/opcenter") . "/bind";
use lib ($ENV{OCW_SVCPATH} or "/usr/lib/opcenter") . "/cmdline_common";

use ERRORS;
use Carper;

# args pass in as hash
# possible keys:
#  - refresh
#  - retry
#  - expire
#  - ttl
#  - email
sub getValues (;$) {
    my $key = shift;

    # parse the values
    my $defaultsRef = _getHash();

    if (defined $key) {
        return $defaultsRef->{$key};
    } else {
        return $defaultsRef;
    }
}

# args pass in as hash
# possible keys:
#  - refresh
#  - retry
#  - expire
#  - ttl
#  - email
sub setValues (%) {
    my %newVals = @_;
    
    return if (!scalar %newVals);

    my $filename = _getFileName();
    my $err;

    # there may be a race condition if the file
    # does not exists. But the case should be 
    # very rare since the default file should be
    # installed by the installer/upgrade script

    # make sure the files exists
    if (! -r $filename) {
        `touch $filename`;
    }

    # lock the file
    if ($err = Locking::lock ($filename)) {
        return $err;
    }

    # write back to file
    $err = _writeHash (\%newVals);

    # unlock
  Locking::unlock (_getFileName());

    return $err;
}

sub _getHash () {
    my $buf;
    my $handle_r = new IO::File _getFileName(), "r";

    # get default values
    my $retHashRef = _defaultVals();

    if (defined $handle_r) {
        while ($buf = <$handle_r>) {
            chomp $buf;
            my ($key, $val) = _unpackLine ($buf);
            
            if (defined $retHashRef->{$key} && $val) {
                # over defaults from file
                $retHashRef->{$key} = $val;
            }
        }
        
        undef $handle_r;
    }

    return $retHashRef;
}

sub _writeHash (\%) {
    my $hashRef = shift;
    my $handle_w = new IO::File _getFileName(), "w";

    if (!defined $handle_w) {
        $cerr += $E_UNABLEWRITEFILE and $cerr += { file => _getFileName() } ;
        return "failed to write to " . _getFileName();
    }

    # now, we need to load in the existing values
    my $currentRef = _getHash();

    # override current values and write back to file

    my $buf;
    my $val;
    foreach my $key (keys %{$currentRef}) {
        $val = ($hashRef->{$key} or $currentRef->{$key});
        $buf = _packLine($key, $val);
        print $handle_w $buf;;
        print $handle_w "\n";
    }
    
    undef $handle_w;
}

sub _getFileName () { return "/etc/bind/soa_defaults" }

sub _unpackLine ($) {
    my $line = shift;

    my ($key, $val) = split /=/, $line;

    if (defined $key && $key &&
        defined $val) {
        
        $key =~ s/^\s*(\S+)\s*$/$1/g;
        $val =~ s/^\s*(\S+)\s*$/$1/g;

        return ($key, $val);
    } else {
        return ("", "");
    }
}

sub _packLine ($$) {
    return "$_[0]=$_[1]";
}

sub _defaultVals() {
    return {'refresh' => 3600,
            'retry'   => 600,
            'expire'  => 86400,
            'ttl'     => 3600,
            'email'   => ''};
}

