# --------------------------------------------------------------------------
###
#
# Copyright (c) Ensim Corporation 2000, 2001   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#
# $Id: MasterName.pm,v 1.17 2003/03/14 06:37:05 naris Exp $
# $Name:  $
# --------------------------------------------------------------------------

package MasterName;

use strict;

use lib ($ENV{'OCW_SVCPATH'} || "/usr/lib/opcenter")."/bind";

require Exporter;
our @ISA = qw (Exporter);
our @EXPORT_OK = qw (getMasterHostname setMasterHostname);

use Sys::Hostname;
use Net::DNS;
use Socket;

use dns;
use DnsUpdate;
use DnsCommon;

use constant MASTER_FILE => "/var/named/mastername";

use lib ($ENV{OCW_SVCPATH} or "/usr/lib/opcenter") . "/bind";
use lib ($ENV{OCW_SVCPATH} or "/usr/lib/opcenter") . "/cmdline_common";

use ERRORS;
use Carper;


### 
# This method would return either the set via the GUI
# or DNS domain name of this host (ie. hostname(1) -d)
###
sub getMasterHostname () {
  my $fromDB = 1;
  my $master = _getName();
  
  # use node name if _getName returns empty string
  if (! $master) {
      $fromDB = 0;
      $master = _getHostname();
  }
  
  return wantarray ? ($master, $fromDB) : $master;
}



### 
# This method would set the name returned by
# getMasterHostname method instead of the hostname
# of the machine. If this method is called with an empty 
# string, getMasterHostname will return the hostname of 
# this host instead. The return value of this method is the 
# same as the value returned by getMasterHostname.
#
# Normally, the return value of this
# method is the same as the first argument, except if 
# the first argument is an empty string. In that case,
# hostname of this machine is returned instead.
#
# Exception will be thrown if error enocuntered.
###
sub setMasterHostname($) {
    my $newname = shift;
    
    my $error;
    
    # strip out whitespace
    $newname =~ s/\s*(.*)\s*$/$1/g;  
    
    # if newname is empty, remove the file,
    # use hostname instead
    if ($newname eq '') {
	unlink MASTER_FILE;
    } else {

	#verify hostname
        #turned off check.  PR 22491
	#!($error = DnsCommon::verifyHostname ($newname)) or die $error . "\n";
	
	# update database
	!($error = _updateName ($newname))     or die $error . "\n";
    }

    return getMasterHostname();
}

###
# This method would update SOA and NS records set with the 
# old hostname to the new one
### 
sub updateRR ($$) {
    my $oldMaster = shift;
    my $newMaster = shift;

    my $zoneHashRef = dns::getZones();

    # since the hostname is always a FQDN,
    # we will add a '.' if the last char is not a '.'
    substr ($newMaster, -1) =~ s/([^\.])/$1./g;

    my %errZones = ();
    my $resolver = DnsCommon::getResolver();
    my $ret = 0;

    foreach my $zone (keys %{$zoneHashRef}) {
	next if ($zoneHashRef->{$zone}->{type} ne "master");
	
	# skip locals
	next if ($zone eq 'localhost');
	next if ($zone eq '0.0.127.in-addr.arpa');
	
	# update only those zone with primary master set
	# to oldMaster
	my $soaResult = $resolver->query ($zone, 'SOA');

	# in case query failed
	next if (!$soaResult);
	
	my $zoneSOA = ($soaResult->answer)[0];
	
	if ($zoneSOA->mname ne $oldMaster) {
	    next;
	}
	
	# update SOA first
	my $updater = _updateSOA ($zone, $newMaster);
	
	# we don't need to do anything more since SOA
	# is not updated
	if ($updater) {
	    my $hasError = _handleSend($zone, $updater);
	    
	    if ($hasError) {
		$ret = 1;
		next;
	    }
	} else {
	    next;
	}
	
	# update NS record as well
	my $nsResult = $resolver->query ($zone, 'NS');
	
	next if (!$nsResult);
	
	my $nsRR;
	foreach my $rr ($nsResult->answer) {
	    next if ($rr->type ne 'NS');
	    
	    if ($rr->nsdname eq $oldMaster) {
		if (_handleSend($zone, _updateNS ($zone, 
						  $oldMaster, 
						  $newMaster,
						  $rr->ttl))) {
		    
		    $ret = 1;
		}
	    }
	    
	}
    }

    return $ret;
}

########################## HELPER METHODS ####################################

sub _updateSOA ($$) {
    my $zone = shift;
    my $master = shift;

    my $updater = new DnsUpdate ($zone);
    my $doUpdate;
    eval {$doUpdate = DnsUpdate->updateSOA ($updater, $zone, {mname=>$master}, 1)};
    # catch DNS lookup exception
    if ($@) {
	my $err = $@;
	chomp $err;
	return $err;
    }

    if (!$doUpdate) {
	return;
    } else {
	return $updater;
    }
}

sub _updateNS ($$$$) {
    my $zone = shift;
    my $curMaster = shift;
    my $newMaster = shift;
    my $rrTTL = shift;

    substr ($curMaster, -1) =~ s/([^\.])/$1./g;

    my $updater = new DnsUpdate($zone);
    
    $updater->pushPrereq ('yxrrset', '@ NS');
    $updater->pushUpdate ('add', '@', $rrTTL, 'NS', $newMaster);
    $updater->pushUpdate ('delete', '@', 'NS', $curMaster);
    
    return $updater;
}

sub _handleSend ($$) {
    my $zone = shift;
    my $updater = shift;
    
    my $err;
    my $reply = $updater->send();
    
    if (!$reply) {
	$err = $updater->errorstring;
    }
    
    if (ref $reply) {
	$err = $reply->status;
    } else {
	$err = $reply;
    }

    # ok
    if ($err eq 'NOERROR') {
	return;
    } 

    # handle error
    my $errCode = DnsCommon::getErrorCodeByStr($err);
    
    if (defined ($errCode)) {
	my $errStr = DnsCommon::getLongErrorByStr($err);
	print STDERR "$zone $errStr \n";
    } else {
	print STDERR "$zone $err \n";
    }
    
    return 1;
}

sub _updateName ($) {
  my $newname = shift;

  open (MASTERNAME,">" . MASTER_FILE) or
    $cerr += $E_UNABLEWRITEFILE and $cerr += { file => MASTER_FILE } and
    return "Could not write " . MASTER_FILE;
  print MASTERNAME $newname;
  close(MASTERNAME);

  return;
}

                 
sub _getName () {
  my $master = "";

  if (-r MASTER_FILE ) {
    open (HOSTNAME, MASTER_FILE) or
        $cerr += $E_UNABLEREADFILE and $cerr += { file => MASTER_FILE } and
      die "Could not read " . MASTER_FILE . ".\n";
    $master = <HOSTNAME>;
    close(HOSTNAME);
    #throwing away extra spaces just in case
    $master =~ s/^\s*(\S*)\s*$/$1/; 
  }  

  return $master;
}

sub _getHostname() {
    my $hostname = hostname;

    my $res = new Net::DNS::Resolver;
    my $query = $res->search ($hostname);
    
    if ($query) {
	return ($query->answer)[0]->name;
    } else {
	return $hostname;
    }
}

1;
