# --------------------------------------------------------------------------
###
#
# Copyright (c) Ensim Corporation 2000, 2001   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#
# $Id: SlaveServers.pm,v 1.8 2003/03/14 06:37:05 naris Exp $
# $Name:  $
# --------------------------------------------------------------------------

###
# This module provides an interface for Virutal DNS settings
###

package SlaveServers;

use strict;

use lib ($ENV{'OCW_SVCPATH'} || "/usr/lib/opcenter")."/bind";
use lib  ($ENV{'OCW_SVCPATH'} || "/usr/lib/opcenter")."/cmdline_common";

require Exporter;
our @ISA = qw (Exporter);
our @EXPORT_OK = qw (addSlave deleteSlave getSlaves);

use File::Basename;
use IO::File;
use Net::DNS;

use DnsCommon;

use LogConfFile;

use constant SLAVE_FILE => "/var/named/slaveServers";
use constant CONF_LOG_DIR => "/var/log/conf";

use ERRORS;
use Carper;

sub addSlave ($) {
  my $slaveName = shift;

  if (! DnsCommon::validateHostname ($slaveName)) {
    $cerr += $E_MALFORMEDHOSTNAME and $cerr += { hostname => $slaveName } ;
    return "Malformed hostname: $slaveName";
  }

  #    turned off check.  PR 22491
  # check if the server has an A record
  #if (!_checkDNSRecord($slaveName)) {
  #  $cerr += $E_NOARECORD and $cerr += { host => $slaveName } ;
  #  return "$slaveName does not have an A record";
  #}

  # check if the server exists already
  if (_serverExists($slaveName)) {
    $cerr += $E_SERVEREXISTS and $cerr += { server => $slaveName } ;
    return "Server $slaveName exists already";
  }

  
  # add to file
  my $fileDir  = dirname SLAVE_FILE;
  my $filename = basename SLAVE_FILE;

  # create the file if it does not exists
  if (! -e SLAVE_FILE) {
      open  TOUCH_FILE, ">" . SLAVE_FILE;
      close TOUCH_FILE;
  }

  my $slaveDataFH = LogConfFile::open_conf ($fileDir, CONF_LOG_DIR,
                                            $filename, "a");
  if (!$slaveDataFH) {
    # it is not necessary to delete the zone files
        $cerr += $E_UNABLEOPENFILE and $cerr += { file => SLAVE_FILE } ;
      return "unable to open file:" . SLAVE_FILE;
  }
  
  print $slaveDataFH "$slaveName\n";
  
  if (!LogConfFile::close_conf ($slaveDataFH, $fileDir, 
                                CONF_LOG_DIR, $filename)) {
    $cerr += $E_UNABLEUPDATE and $cerr += { file => SLAVE_FILE } ;
    return "Unable to update " . SLAVE_FILE;
  }

  return ;
}

sub deleteSlave ($) {
  my $slaveName = shift;
  my $inputHandle;
  my $outputHandle;
  
  my $fileDir  = dirname SLAVE_FILE;
  my $filename = basename SLAVE_FILE;

  if (!($inputHandle = new IO::File SLAVE_FILE, "r")) {
        $cerr += $E_UNABLEOPENFILE and $cerr += { file => SLAVE_FILE } ;
    return "Unable to open " . SLAVE_FILE
  }
  
  if (!($outputHandle = LogConfFile::open_conf ($fileDir, CONF_LOG_DIR,
                                                $filename, "w"))) {
    $cerr += $E_UNABLEUPDATE and $cerr += { file => SLAVE_FILE } ;
    return "Unable to update " . SLAVE_FILE;
  }
  
  my $pattern = _getPattern ($slaveName);
  
  # we just want to remove the line from /etc/named.conf
  while ($_ = _skipToLine ($inputHandle, $pattern, $outputHandle)) {
  }
  
  $inputHandle->close();
  if (! LogConfFile::close_conf ($outputHandle, $fileDir, 
                                 CONF_LOG_DIR,  $filename)) {
    $cerr += $E_UNABLEUPDATE and $cerr += { file => SLAVE_FILE } ;
    return "Unable to update " . SLAVE_FILE;
  }

  return;
}

                 
sub getSlaves () {
  my @slaves = ();

  if (-r SLAVE_FILE) {
    my $buf;

    my $slaveData = new IO::File SLAVE_FILE, "r";
    while ($buf = <$slaveData>) {
      chomp $buf;
      
      push @slaves, $buf;
    }
    $slaveData->close;
  }
  return \@slaves
}

########################## HELPERS ###########################################

sub _checkDNSRecord ($) {
  my $slaveName = shift;

  my $res = new Net::DNS::Resolver;
  $res->defnames(0);
  my $query = $res->query ($slaveName);

  my $found = 0;
  if ($query) {
    foreach my $rr ($query->answer) {
      if ($rr->type eq "A") {
        $found++;
        last;
      }
    }
  } 

  return $found;
}

sub _getPattern ($) {
  my $slaveName = shift;
  my $pattern = qr/^$slaveName$/;

  return $pattern;  
}


sub _serverExists ($) {
  my $slaveName = shift;
  
  my $inputHandle;
  if (!($inputHandle = new IO::File SLAVE_FILE, "r")) {
    return 0;
  }
  
  my $pattern = _getPattern ($slaveName);

  return _skipToLine ($inputHandle, $pattern);
}

sub _skipToLine ($$;$) {
    my $inputHandle = shift;
    my $pattern = shift;
    my $outputHandle = shift;
    
    my $buf;
    while ($buf = <$inputHandle>) {
        chomp $buf;
        if ($buf =~ m/$pattern/) {
            return $buf;
        }
        
        if ($outputHandle) {
            print $outputHandle $buf;
            print $outputHandle "\n";
        }
    }
}


1;
