# --------------------------------------------------------------------------
###
#
# Copyright (c) Ensim Corporation 2000, 2001   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#
# $Id: ZoneParser.pm,v 1.5 2001/10/05 01:58:58 andy Exp $
# $Name:  $
# --------------------------------------------------------------------------

package ZoneParser;

use strict;

use BIND::Conf_Parser;
our @ISA = qw(BIND::Conf_Parser);

use constant DEFAULT_CONTROLS_PORT => 953;

sub new {
    my $class = shift;
    my $self = { };
    bless $self, $class;
    
    $self->{zones} = {};
    $self;
}

sub handle_zone {
    my ($self, $name, $class, $type, $options) = @_;
    
    my %zoneInfo = ();
    $self->{zones}->{$name} = $options;
}

sub handle_control {
    my ($self, $proto, $control) = @_;
}

sub zones {
    my $self = shift;
    return $self->{zones};
}

####
# Overrides the default implementation to support BIND 9.x 
# style controls syntax.
####
sub parse_controls($) {
    my $self = shift;

    $self->expect('{', "following `controls'");
    while(1) {
	$self->expect([ [ qw(inet) ], ';' ], "reading `controls'");
	last if $self->{_token} eq ';';
        my($addr, $port, $addrMatchList, $keyList);
        $self->expect([ BIND::Conf_Parser::IPADDR, '*' ], "following `inet'");
        $addr = $self->{_token} eq '*' ? 0 : $self->{_data};
        $self->expect([ [ 'port', 'allow' ] ], "following 'IPADDR'");
        # port number is optional
        if ($self->{_data} ne 'allow') {
	    $self->expect([ BIND::Conf_Parser::NUMBER ], "following 'port'");
            $port = 0 + $self->{_data};
            $self->expect([["allow"]], "following port number");
        } else {
            $port = DEFAULT_CONTROLS_PORT;
        }
        $addrMatchList = $self->parse_addrmatchlist("following `allow'");
        
	$self->expect([['keys']], "expecting `keys'");
	$keyList = $self->parse_keyList("following `keys'");

        $self->handle_control("inet", [ $addr, $port, $addrMatchList,
                                        $keyList]);
    }
    $self->expect('}', "closing `controls'");
    $self->expect(';', "finishing `controls'");
}

sub parse_keyList ($$) {
    my ($self, $mess) = @_;

    $self->expect('{', $mess);
    my(@keys);
    while (1) {
        $self->expect([ BIND::Conf_Parser::WORD, 
                      BIND::Conf_Parser::STRING, '}' ],
                      $mess);
        last if $self->{_token} eq '}';
        push @keys, $self->{_data};
	$self->expect(';', "reading key list");
    }

    return \@keys;
}

sub parse_addrmatchlist($$;$) {
    my($self, $mess, $nolex) = @_;
    $self->expect('{', $mess, $nolex);
    my(@items, $negated, $data);
    while(1) {
	$negated = 0;
	$self->expect([ BIND::Conf_Parser::IPADDR, BIND::Conf_Parser::NUMBER, 
		      BIND::Conf_Parser::WORD, BIND::Conf_Parser::STRING, 
			'!', '{', '}' ], $mess);
	last if $self->{_token} eq '}';
	if ($self->{_token} eq '!') {
	    $negated = 1;
	    $self->expect([ BIND::Conf_Parser::IPADDR, 
			  BIND::Conf_Parser::NUMBER, BIND::Conf_Parser::WORD, 
			  BIND::Conf_Parser::STRING, '{' ],
			  "following `!'");
	}
	if ($self->{_token} eq '{') {
	    push @items, [ $negated, $self->parse_addrmatchlist($mess, 1) ];
	    next
	}
	if ($self->{_token} eq BIND::Conf_Parser::WORD) {
	    if ($self->{_data} eq "key") {
		###
		# this is key key_id construct, we need to find the key_id
		###
		$self->expect([ BIND::Conf_Parser::STRING, 
			      BIND::Conf_Parser::WORD ], $mess);
		push @items, [ $negated, "key", $self->{_data} ];
	    } else {
		push @items, [ $negated, "acl", $self->{_data} ];
	    }
	    next
	}	
	if ($self->{_token} eq BIND::Conf_Parser::STRING) {
	    push @items, [ $negated, "acl", $self->{_data} ];
	    next
	}
	$data = $self->{_data};
	$self->expect( $self->{_token} eq BIND::Conf_Parser::NUMBER ? '/' : [ '/', ';' ], $mess);
	if ($self->{_token} eq ';') {
	    push @items, [ $negated, $data ];
	    redo	# we already slurped the ';'
	}
	$self->expect(BIND::Conf_Parser::NUMBER, "following `/'");
	push @items, [ $negated, $data, $self->{_data} ];
    } continue {
	$self->expect(';', $mess);
    }
    return \@items
}

1;
