# --------------------------------------------------------------------------
###
#
# Copyright (c) Ensim Corporation 2000, 2001   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#
# --------------------------------------------------------------------------
# $Id: Locking.pm,v 1.7 2001/03/20 19:30:12 uday Exp $
# $Name:  $
# --------------------------------------------------------------------------
package Locking;
#flocks files

use strict;
use Fcntl ':flock'; # import LOCK_* constants
use Symbol;

use vars qw ( %locks @lockstack );

%locks = ();
@lockstack = ();

#Prints error message if and exits there's a problem, blocks if the
#file is locked by another process.

sub lock ($) {
  my ($filename) = @_;
  if (defined $locks{$filename}){
    #I assume that it's a bug if we want to lock again a file we
    #already locked.
    return "Cannot lock file $filename twice.\n";
  }
  $locks{$filename} = gensym;
  open ($locks{$filename},$filename) or return "Cannot open file $filename .\n";
  flock ($locks{$filename},LOCK_EX) or return "Cannot lock the file $filename .\n";
  return "";
}

sub unlock ($) {
  my ($filename) = @_;
  unless (defined $locks{$filename}){
    die "Cannot unlock file I didn't lock: $filename .\n";
  }
  flock ($locks{$filename},LOCK_UN);
  close ($locks{$filename});
  undef $locks{$filename};
}

#adds a lock to the lock stack
sub PushLock ($) {
  my ($lock) =@_;
  push @lockstack, $lock;
  return lock ($lock);
}

#releases all locks on the lock stack
sub ReleasePushedLocks(){
  my ($lock);
  while ($lock = pop @lockstack) {
    unlock($lock);
  }
}

sub Check($) {
    my ($filename) = @_;
    unless(defined $locks{$filename}) {
	die "$0: VHConfiguration lock not held .\n";
    }
}
return 1;
