#!/usr/bin/eperl
###
#
# Copyright (c) Ensim Corporation 2000, 2001   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#
# --------------------------------------------------------------------------
# $Id: LogConfFile.pm,v 1.41 2003/03/04 23:39:50 amit Exp $
# $Name:  $
# --------------------------------------------------------------------------
package LogConfFile;
use File::Copy;
use File::Compare;
use File::Basename;
use Locking;
use strict;

my $LOG_DIR = "/var/log/conf";

#maximum number of conf files in the directory
my $count = 20;
my $wrap_val = 100000;

sub get_log_file_numbers($$) {
    my ($LOG_DIR,$CONF_FILE) = @_;
    
    my $logfile = "$LOG_DIR/$CONF_FILE";
    my @files = ();
    
    if(!opendir(LOGFDIR, $LOG_DIR)) {
	return undef;
    }
   
    # Strip out the existing extensions for this log file
    for (readdir(LOGFDIR)) {
	if (/^$CONF_FILE\.(\d+)$/) {
	    push @files, $1;
	}
    }
    close LOGFDIR;
    #@files = sort { (-C "$logfile.$b") <=> (-C "$logfile.$a") } @files;
    @files = sort { $a <=> $b } @files;
   return @files;

}
    
# copy the conf file to the log directory
sub copy_to_log($$$){
    my($CONF_DIR,$LOG_DIR,$CONF_FILE)=@_;
    
    my $logfilename = "$LOG_DIR/$CONF_FILE";
    
    my @existing_files = get_log_file_numbers $LOG_DIR,$CONF_FILE;
    
    # Unlink any excess historic files
    while(@existing_files >= $count) {
	unlink "$logfilename." . (shift @existing_files);
    }

    # New file should be one greater than latest file
    my($next_suffix);
    my($index);
    if (scalar @existing_files &&
        $existing_files[$#existing_files]  >= $wrap_val){
	my $counterIndex = 0;
	foreach $index (@existing_files){
		`mv -f $logfilename.$index $logfilename.$counterIndex`;
		$counterIndex = $counterIndex + 1;
	}
	$next_suffix = $counterIndex;
    } else {
	$next_suffix = pop @existing_files;
        if (! defined $next_suffix) {
            $next_suffix = 0;
        }
        
        $next_suffix++;
    }
    #my $next_suffix =  ((pop @existing_files) + 1) % $wrap_val;

    $logfilename .= ".$next_suffix";
    
    if(!copy "$CONF_DIR/$CONF_FILE", $logfilename) {
	unlink $logfilename;
	return undef;
    }
    return $logfilename;
}

# returns the latest conf file from the log directory
sub get_latest_conf_file($$$){
    my($CONF_DIR,$LOG_DIR,$CONF_FILE) = @_;
    
    my $logfilename = "$LOG_DIR/$CONF_FILE";
    
    my @existing_files = get_log_file_numbers $LOG_DIR,$CONF_FILE;
    if(@existing_files) {
	return "$logfilename." . (pop @existing_files);
    } else {
	return undef;
    }

}

sub internal_check($$$){
    my($CONF_DIR, $LOG_DIR, $CONF_FILE) = @_;
    my $conffile = "$CONF_DIR/$CONF_FILE";

    my ($latest_stack);
    #retrieve the latest version of the conf file from the log directory
    $latest_stack  = get_latest_conf_file ($CONF_DIR, $LOG_DIR, $CONF_FILE);

    # check for any differences
    if($latest_stack && -f $latest_stack && -f $conffile) {

	if((compare $latest_stack, $conffile) == 1) {
	    # failed comparison
	    
	    # copy the tampered version to the log directory
	    $latest_stack = copy_to_log($CONF_DIR,$LOG_DIR,$CONF_FILE);

	    #send a warning message to STDERROR
	    print STDERR "CONF FILE $CONF_DIR/$CONF_FILE TAMPERED WITH\n";
	    if(defined $latest_stack) {
		`/sbin/initlog --facility=local1 --string="LogConfFile: CONF FILE TAMPERED WITH; SAVING IN THE LOG DIRECTORY: $latest_stack"`;
	    } else {
		`/sbin/initlog --facility=local1 --string="LogConfFile: Config file $CONF_DIR/$CONF_FILE Tampered with; unable to save in log directory"`;
	    }
	}

    }
}    
# This function returns the File Handle to the conf file
# it take the following parameter:- 
# directory for the conf file, log directory, conf file name, open mode
# for eg open_conf("/etc/httpd/conf","/var/log/conf","httpd_vwh.conf","w")

sub open_conf($$$$){
    my($CONF_DIR, $LOG_DIR, $CONF_FILE, $OPEN_MODE) = @_;
    internal_check($CONF_DIR,$LOG_DIR,$CONF_FILE);
    my $conffile = "$CONF_DIR/$CONF_FILE";
  
    # after doing all the comparisons now open the tmp conf file
    # this operation will fail only if there is no tmp conf file
    # to avoid this failure we always copy the conf file to the 
    # tmp conf file location just before locking it
    if ($OPEN_MODE eq "a"){
	# open in append mode
	if(!copy $conffile, "$conffile.tmp") {
	    return "";
	}
	open(CONFHANDLE,">>$conffile.tmp") or return("");
    }else{
	# open in overwrite mode
	open(CONFHANDLE,">$conffile.tmp") or return("");
    }
    return(\*CONFHANDLE);

}
sub check_conf($$$){
    my($CONF_DIR, $LOG_DIR, $CONF_FILE) = @_;
    #lock the conf file 
    Locking::lock("$CONF_DIR/$CONF_FILE");
    
    # do the internal check
    internal_check($CONF_DIR,$LOG_DIR,$CONF_FILE);

    # unlock the file
    Locking::unlock("$CONF_DIR/$CONF_FILE");
}

# copy the conf file to the stack directory after closing it
# takes the following parameters:-
# file handle, conf file location directory, log directory, conf file name
# for eg close_conf($FILEHANDLE,"/etc/virtualhosting","/var/log/conf","virthosts.conf")
sub close_conf($$$$){

    my ($CONFHANDLE,$CONF_DIR,$LOG_DIR,$CONF_FILE)=@_;
    my $conffile = "$CONF_DIR/$CONF_FILE";
    my $linkname;
    
    # close the temp file, will return 1 if failed because of disk full
    close($CONFHANDLE) or return undef;

    $linkname = readlink $conffile;
    # PSAdmin unfortunately has its links the other way round
    # (i.e. httpd_app.conf is a symlink to httpd.conf); make
    # sure not to break the symlink (and yes, I realize this
    # only protects us through one level of indirection).
    if (defined($linkname)) {
        rename "$conffile.tmp", $linkname or return undef;
    } else {    
        rename "$conffile.tmp", $conffile or return undef;
    }

    # copy the tmp file to the log directory
    copy_to_log($CONF_DIR,$LOG_DIR,$CONF_FILE);
    
    return 1;
}


return 1;
