# Copyright (c) Ensim Corporation 2000, 2001   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#
# This module handles the configuration files of apache and ftp, as
# used by virtual hosting.
#

package VHAppConf;

use lib ($ENV{'OCW_SVCPATH'} || "/usr/lib/opcenter")."/cmdline_common";
use lib ($ENV{'OCW_SVCPATH'} || "/usr/lib/opcenter")."/mivamerchant";

use lib ($ENV{OCW_SVCPATH} or "/usr/lib/opcenter") . "/virtualhosting";

use ERRORS;
use Carper;

use strict;
use VHConst;
use Errno;
use LogConfFile;
use SymLock;


use vars qw($APACHEDIR $APACHECONF $IP $APACHEHANDLE
	    $VIRTHOSTDIR $VIRTHOSTTMP $APPLIANCECONF $MAILCONF $TELNETCONF);

$APACHEDIR="/etc/httpd/conf";     #apache conf directory
$APACHECONF="$APACHEDIR/httpd_app.conf"; #apache conf file
$VIRTHOSTDIR="$APACHEDIR/virtual";       #virtual hosts
$VIRTHOSTTMP="$APACHEDIR/virtual.tmp"; # Temporary directory for virtual hosts
$APPLIANCECONF="$APACHEDIR/webppliance.conf";
$MAILCONF="/etc/virtualhosting/sendmail";     #to be read by sendmail
$TELNETCONF="/etc/virtualhosting/telnet";     #to be read by telnet

# This function returns a site's virtual hosting entry - for apache conf file 
sub SiteApacheConfEntry($){
  
  my ($record) = @_;
  my ($content) = "";

  unless (open (IPADDR, "/bin/hostname -i | cut -d ' ' -f 1 |")) {
    return "";
  }
  chomp($IP = <IPADDR>); 
  close (IPADDR);  

  $content = $content . "<VirtualHost $record->{IP}:80>\n".
    "\tServerName $record->{WebServerName}\n" ;
     
  if ($record->{DomainType} eq "NameBased") {
    $content = $content . "\tServerAlias $record->{Domain} *.$record->{Domain}\n";
  }

   $content = $content . "\tServerAdmin $record->{Email}\n".
     "\tDocumentRoot $VHConst::VIRTUALFS/$record->{User}/home/httpd/html\n".
     "\tUser $record->{User}\n".
     "\tGroup $record->{User}\n".
     "\tRedirect /admin https://$IP:19638/webhost/services/virtualhosting/siteadmin\n".
     "\tRedirect /user https://$IP:19638/webhost/services/virtualhosting/useradmin\n".
     "\tUserDir $VHConst::VIRTUALFS/$record->{User}/home/*/public_html\n".
     "\tAliasMatch ^/users/([^/]+)/?(.*) /home/virtual/$record->{User}/home/\$1/public_html/\$2\n".
     &CGIApacheConfEntry($record).
     &MivaCGIApacheConfEntry($record).
     &MivaActionEntry($record).
     &WebLogsApacheConfEntry($record).
     "<Directory $VHConst::VIRTUALFS/$record->{User}/home/httpd/html/>\n".
     "\tAllow from all\n".
     "\tAllowOverride All\n".
     "\tOrder allow,deny\n".
     &SSIApacheConfEntry($record).
     "</Directory>\n".
     "</VirtualHost>\n".
     &SSLApacheConfEntry($record);

  return $content;

}


#This function adds a virtual hosting entry to the apache conf file
sub AddToApacheConf($) {
    my ($record) = @_;
  
    SymLock::Check($record->{Domain});
    my $tmpfile = "$VIRTHOSTTMP/$record->{Domain}";

    my $virtfile= "$VIRTHOSTDIR/$record->{Domain}";
    
    DeleteFromApacheConf($record);
    
    unless (open (VIRTCONF,">$tmpfile")) {
    $cerr += $E_UNABLEOPENVCONF and $cerr += { perror => $! } ;
	return "Cannot open virtual host conf file: $!.\n";
    }

    local $APACHEHANDLE = \*VIRTCONF;
    
    if (-e "/etc/virtualhosting/customvirtapache.pm") {
    
	push @INC, "/etc/virtualhosting/";
	require "/etc/virtualhosting/customvirtapache.pm";
	
    } else {
	print $APACHEHANDLE SiteApacheConfEntry($record)
	    or $cerr += $E_WRITEERR and $cerr += { file => $tmpfile, perror => $! }
        and return "Error writing to $tmpfile: $!\n";
	
    }

    close (VIRTCONF) 
	or $cerr += $E_CLOSEERR and $cerr += { file => $tmpfile, perror => $! } 
    and return "Error closing $tmpfile: $!\n";
    
    rename $tmpfile, $virtfile
	or $cerr += $E_VHOSTMOVE and $cerr += { perror => $! } 
    and return "Error moving temporary virtual host file into configuration directory: $!\n";
	    
    return "";
}


sub DeleteFromApacheConf($) {
    my ($record)=@_;

    # do a tamper check on the apache_conf file every time add a domain
    LogConfFile::check_conf("/etc/httpd/conf","/var/log/conf","httpd_app.conf");
    
    my $virtfile= "$VIRTHOSTDIR/$record->{Domain}";
    
    if(!unlink $virtfile) {

	# Ignore ENOENT, since we don't want to fail if the file
	# wasn't there 

	if(!$!{ENOENT}) {
        $cerr += $E_RM and $cerr += { file => $virtfile, perror => $! } ;
	    return "Error removing $virtfile: $!\n";
	}
	
    }
    return "";
}
	    
sub RemoveApacheVirtEntries() {
    
    opendir(VIRTDIR, "$VHAppConf::VIRTHOSTDIR") 
	or $cerr += $E_UNABLEACCESS and
    $cerr += { file => "$VHAppConf::VIRTHOSTDIR" , perror => $! } 
    and return "Cannot access $VHAppConf::VIRTHOSTDIR: $!\n";

    my $file;
    
    my $errormessage = "";

    for $file (readdir(VIRTDIR)) {
	
	next if -d $file;
	
	unlink $file or
        $cerr += $E_RM and $cerr += { file => $file, perror => $! } and
	    $errormessage .= "Cannot remove $file:$!";
    }

    return $errormessage;
}

sub AddApacheVirtEntries() {
    my ($error_message);

    open(WPLOG,">>/var/log/WEBppliance");
    print WPLOG "\n";
    print WPLOG "AddApacheVirtEntries: reconstructing virt containers...\n";

    if ($error_message=VHConfiguration::ReadConfFile()) {
        return $error_message;
    }

    my ($record);
    foreach $record (@VHConfiguration::TheConfFile) {
        next if (not ref ($record));
        if ($record->{Web} and $record->{Enabled}) {
	    SymLock::lock($record->{Domain});
            print WPLOG "AppendAllVirtEntries: adding $record->{WebServerName}\n";
            $error_message .= VHAppConf::AddToApacheConf($record);
	    SymLock::unlock($record->{Domain});
        }
    }

    print WPLOG "AppendAllVirtEntries: done\n";
    close(WPLOG);
    return $error_message;
}

# Update the index.html for a given site with appropriate domainname
#   does a search for a keyword and replaces it with the domainname
sub UpdateIndexHTML() {
    my ($owner,$domainname) = @_;
    my ($error_message) = "";
    my ($keyword) = "ENSIM_DOMAINNAME";
    my ($indexhtml) = "$VHConst::VIRTUALFS/$owner/home/httpd/html/index.html";
    my (@lines, $line);
    if ($error_message = &Locking::PushLock($indexhtml)){
	return $error_message;
    }
    open (HTML,"$indexhtml")
    or $cerr += $E_UNABLEREAD and $cerr += { file => $indexhtml } 
    and $error_message = "Could not read $indexhtml.\n";
    if ($error_message) { &Locking::ReleasePushedLocks; return $error_message; }
    @lines = <HTML>;
    close (HTML);
    foreach $line (@lines) {
        $line =~ s/$keyword/$domainname/g;
    }
    open (HTML,">$indexhtml")
    or $error_message = "Could not write to $indexhtml.\n";
    if ($error_message) { &Locking::ReleasePushedLocks; return $error_message; }
    print HTML @lines;
    close (HTML);
    &Locking::ReleasePushedLocks;
    return $error_message;
}

# Change the ftp server name in the chrooted ftp conf file
sub ChangeFtpServerName() {
    my ($ftpservername) = @_;
    my ($prefix) = "$VHConst::VIRTUALFS/$::user";
    my ($error_message) = "";
    my (@lines, $line);
    open (CONF,"$prefix/etc/proftpd.conf")
    or $cerr += $E_UNABLEREAD and $cerr += {file => "$prefix/etc/proftpd.conf"}
    and return "Could not read $prefix/etc/proftpd.conf .\n";
    @lines = <CONF>;
    close (CONF);
    my($found) = 0;
    foreach $line (@lines) {
        if (not $found) {
            next if ($line !~ /^\s*ServerName/i);
            $found = 1;
            $line = "ServerName			\"$ftpservername\"\n";
        }
    }
    if (not $found) {
        $cerr += $E_SERVERNAMENOTFOUND ;
        $error_message = "ServerName entry not found in ftp conf file.\n";
    } else {
        open (CONF,">$prefix/etc/proftpd.conf")
        or $cerr += $E_UNABLEWRITE 
        and $cerr += { file => "$prefix/etc/proftpd.conf" } 
        and return "Could not write $prefix/etc/proftpd.conf .\n";
        print CONF @lines;
        close (CONF);
    }
    return $error_message;
}

# The anonymous ftp entry in the chrooted ftp conf file 
sub AnonymousFtpConfEntry() {
  return "<Anonymous /home/ftp>\n".
    "\t<Limit WRITE>\n".
    "\t\tDenyAll\n".
    "\t</Limit>\n".
    "\tRequireValidShell on\n".
    "\tDisplayFirstChdir .message\n".
    "\tDisplayLogin welcome.msg\n".
    "\tUserAlias anonymous ftp\n".
    "\tUser ftp\n".
    "\tGroup ftp\n".
    "\tMaxClients 10\n".
    "</Anonymous>\n";
}

# The SSI entry in the apache conf file *in the section of a virt domain *
sub SSIApacheConfEntry() {
  my ($record) = @_;
  if ($record->{SSI}) {
      return "\tOptions Includes\n";
  } else {
    return "";
  }
}

sub SSLApacheConfEntry() {
  my ($record) = @_;
  if ($record->{SSL} and $record->{DomainType} eq "IPBased") {
    return "<IfDefine SSL.$record->{IP}>\n".
           "  <VirtualHost $record->{IP}:443>\n".
	   "    ServerName $record->{WebServerName}\n".
	   "    ServerAdmin $record->{Email}\n".
	   "    DocumentRoot $VHConst::VIRTUALFS/$record->{User}/home/httpd/html\n".
	   "    User $record->{User}\n".
	   "    Group $record->{User}\n".
	   "    SSLEngine on\n".
	   "    SSLCertificateFile  $VHConst::VIRTUALFS/$record->{User}/etc/httpd/conf/ssl.crt/server.crt\n".
	   "    SSLCertificateKeyFile  $VHConst::VIRTUALFS/$record->{User}/etc/httpd/conf/ssl.key/server.key\n".
	   "    SetEnvIf User-Agent \".*MSIE.*\" nokeepalive ssl-unclean-shutdown\n".
	   &CGIApacheConfEntry($record).
           &MivaCGIApacheConfEntry($record).
           &MivaActionEntry($record).
	   &WebLogsApacheConfEntry($record).
	   "    <Directory $VHConst::VIRTUALFS/$record->{User}/home/httpd/html/>\n".
           "      Allow from all\n".
           "      AllowOverride All\n".
	   "      Order allow,deny\n".
	   &SSIApacheConfEntry($record).
	   "    </Directory>\n".
	   "  </VirtualHost>\n".
	   "</IfDefine>\n";

  } else {
    return "";
  }
}

# The CGI directory entry in the apache conf file *in the section of a virt
# domain*
sub CGIApacheConfEntry() {
  my ($record) = @_;
  if ($record->{CGI}) {
    return "\tScriptAlias /cgi-bin/ $VHConst::VIRTUALFS/$record->{User}/home/httpd/$record->{ScriptAlias}/\n";
  } else {
    return "";
  }
}

# The Miva CGI directory entry in the apache conf file *in the section of a virt
# domain*
sub MivaCGIApacheConfEntry() {
  my ($record) = @_;
  if ($record->{MivaMerchant}) {
    return "\tScriptAlias /cgi-miva/ $VHConst::VIRTUALFS/$record->{User}/home/httpd/cgi-miva/\n";
  } else {
    return "";
  }
}

# The Miva Action entry in the apache conf file *in the section of a virt
# domain*
sub MivaActionEntry() {
  my ($record) = @_;
  if ($record->{MivaMerchant}) {
    return "\tAddType application/x-miva .mv\n\tAction application/x-miva /cgi-miva/miva\n";
  } else {
    return "";
  }
}

# The log entries in a virtualhost section of the apache conf file
sub WebLogsApacheConfEntry() {
  my ($record) = @_;
  if ($record->{WebLogs}) {
    return "\tCustomLog $VHConst::VIRTUALFS/$record->{User}/var/log/httpd/access_log combined\n".
           "\tErrorLog $VHConst::VIRTUALFS/$record->{User}/var/log/httpd/error_log\n";
  } else {
    return "";
  }
}

# Adds the information related to the new domain to the local DNS server
sub AddToDNS() {
  my ($record,$DNS) = @_;
#  
#  push @INC, ($ENV{'OCW_SVCPATH'} || "/usr/lib/opcenter") . "/bind";
#
#  require "dns.pm";
#
#  #Silently return if we don't have DNS
#  unless (@{&dns::get_config()}){return ""};
#
#  if ($record->{DomainType} eq "IPBased"){
#    #create the reverse mapping if we are primary for the reverse mapping.
#    my($arpaaddr) = &dns::ip_to_arpa($record->{IP});
#    my($domain,$domain_file);
#    $domain = &dns::split_hostname($arpaaddr);
#    $domain_file = &dns::get_domain_file($domain);
#    if ($domain_file){
#      # Check if address is unique
#      my(@records);
#      @records = &dns::read_zone_file($domain_file,$domain);
#      # Only interested in 'PTR' records
#      @records = grep { $_->{'type'} eq 'PTR' and $_->{'name'} eq "$arpaaddr"} @records;
#      if ($#records == 0 and grep {$_ eq $record->{Domain} or $_ eq "$record->{Domain}."} @{$records[0]->{values}}){
#	  #nothing to do, we happen to have the right data
#      } elsif ($#records >= 0) {
#	  return "IP address $record->{IP} already allocated in DNS to others.\nPlease delete those entries before using this IP address.\n";
#      } else {
#	  &dns::create_record($domain_file,"$arpaaddr",undef,'IN','PTR',"$record->{Domain}.");
#      }
#    }
#  }
#  if ($DNS) {
#    #create the forward mapping
#    my(@names)=($record->{Domain}); #names to add to the DNS
#    if ($record->{WebServerName} ne $record->{Domain}) { 
#        unless (grep {$_ eq $record->{WebServerName}} (@names,"not.enabled")) {
#            unless ($record->{WebServerName} =~ /^(.*)\.?$record->{Domain}$/) {
#                return "Web server $record->{WebServerName} is not part of the domain $record->{Domain} .\n";
#            }
#            push @names,$record->{WebServerName}
#        }
#    }
#    unless (grep {$_ eq $record->{FtpServerName}} (@names,"not.supported","not.enabled")) {
#      unless ($record->{FtpServerName} =~ /^(.*)$record->{Domain}$/) {
#	return "Ftp server $record->{FtpServerName} is not part of the domain $record->{Domain} .\n";
#      }
#      push @names,$record->{FtpServerName}
#    }
#    #adding trailing dots to names
#    @names = map {($_ =~/\.$/?$_:$_.".")} @names;
#    #Create zone if it doesn't already exist
#    my($config) = &dns::get_config();
#    my(@zones) = grep { $_->{'name'} eq 'zone' } @{$config};
#    unless (grep { $_->{'value'} eq $record->{Domain} } @zones) {
#	my($params) = &dns::create_master_default_params($record->{Domain});
#	&dns::create_master($params);
#    }
#    #add the names to DNS
#    @dns::get_config_cache=(); #Sorry I have to manually delete the
#                               #cached configuration from the dns
#                               #module. 
#    my $domain_file = &dns::get_domain_file($record->{Domain});
#    map {&dns::create_record($domain_file,"$_",undef,'IN','A',$record->{IP})} @names;
#  }
  return "";
}

# Removes the information related to a domain from the local DNS server
sub DeleteFromDNS () {
  my ($record,$DNS) = @_;
#  
#  push @INC, ($ENV{'OCW_SVCPATH'} || "/usr/lib/opcenter") . "/bind";
#
#  require "dns.pm";
#
#  #Silently return if we don't have DNS
#  unless (@{&dns::get_config()}){return ""};
#
#  if ($record->{DomainType} eq "IPBased"){
#    #remove the reverse mapping if we are primary for the reverse mapping.
#    my($revconf,$revfile,$revrec) = &dns::find_reverse($record->{IP});
#    if (defined $revrec) {
#      my( @zone ) = &dns::read_zone_file($revfile,$revconf->{'value'});
#      # Only look at PTR records
#      @zone = grep { $_->{'type'} eq 'PTR' } @zone;
#      # Only look at the records we are removing
#      @zone = grep { lc($_->{'name'}) eq lc(&dns::ip_to_arpa($record->{IP})) } @zone;
#      if (@zone) {
#	# Remove the line
#	my($line);
#	$line = $zone[0];
#	&dns::delete_record($line->{'file'},$line);
#      }
#    }
#  }
#  if ($DNS) {
#    #remove the domain from the DNS conf file
#    my(@conf);
#    @conf = @{&dns::get_config()};
#
#    # Only look at zones
#    @conf = grep { $_->{'name'} eq 'zone' } @conf;
#    @conf = grep { $_->{'value'} eq $record->{Domain} } @conf;
#
#    if ($#conf >= 0) {
#      # When removing zones we do not delete any files but only remove
#      # the zone record from the master conf file.
#      &dns::delete_record( $conf[0]->{'file'}, $conf[0] );
#    }
#  }
  return "";
}

my $fpext_install = "/usr/lib/opcenter/frontpage/install_fp";
my $fpext_uninstall = "/usr/lib/opcenter/frontpage/uninstall_fp";

sub addFrontPageExt() {
    my($user,$group,$passwd,$host,$ssl,$FPExt) = @_; 
    # return if FPExt is not supposed to be enabled
    if ($FPExt == 0) {
        return "" ;
    }
    if (-e $fpext_install) {
        my $errcode = 
	    system ("$fpext_install $user $group $passwd $host >/dev/null 2>/dev/null");
	if ($ssl == 1){
	    system (" cp -f /usr/local/frontpage/$host:80.cnf /usr/local/frontpage/$host:443.cnf");
	}
        return "" ;
    }
    $cerr += $E_UNABLEFINDFPEXT ;
    return "Could not find Front Page Extensions!\n" ;
}

sub removeFrontPageExt() {
    my($host,$user) = @_; 
    if (-e $fpext_uninstall) {
        my $errcode = system ("$fpext_uninstall $host $user >/dev/null 2>/dev/null");
        system("rm -f /usr/local/frontpage/$host:443.cnf");
        return "" ;
    }
    $cerr += $E_UNABLEFINDFPEXT ;
    return "Could not find Front Page Extensions!\n" ;
}


# This function sets the frontpage password of the user owning the
# virtual domain to the password that user has in the /etc/passwd of
# the domain.
sub resetFrontPagePassword() {
    my($user) = @_;
    unless (open(PASSWD,"$VHConst::VIRTUALFS/$user/etc/shadow")) {
    $cerr += $E_UNABLEREADPWDFILE and 
    $cerr += {vroot => "$VHConst::VIRTUALFS/$user" } ;
	return "Cannot read password file $VHConst::VIRTUALFS/$user/etc/passwd\n";
    }
    my ($line);
    while ($line=<PASSWD>) {
	my (@fields)=split(/:/,$line);
	if ($fields[0] eq $user ) {
	    my($password)=$fields[1];
	    unless (open(FPPSWD,"$VHConst::VIRTUALFS/$user/home/httpd/html/_vti_pvt/service.pwd")) {
        $cerr += $E_UNABLEREADFPPWD and
        $cerr += { pwd => "$VHConst::VIRTUALFS/$user/home/httpd/html/_vti_pvt/service.pwd" };
		return "Cannot read frontpage password file $VHConst::VIRTUALFS/$user/home/httpd/html/_vti_pvt/service.pwd\n";
	    }
	    my(@fppasswd)=<FPPSWD>;
	    close(FPPSWD);
	    foreach (@fppasswd) {
		if ( $_ =~ /^$user:/ ) {
		    $_ = "$user:$password\n";
		}
	    }
	    unless (open(FPPSWD,">$VHConst::VIRTUALFS/$user/home/httpd/html/_vti_pvt/service.pwd")) {
        $cerr += $E_UNABLEWRITEFPPWD and 
        $cerr += { pwd => "$VHConst::VIRTUALFS/$user/home/httpd/html/_vti_pvt/service.pwd" } ;
		return "Cannot write frontpage password file $VHConst::VIRTUALFS/$user/home/httpd/html/_vti_pvt/service.pwd\n";
	    }
	    print FPPSWD @fppasswd;
	    close(FPPSWD);
	    return "";
	}
    }
    $cerr += $E_NODOMAINOWNERINPWD and
    $cerr += { owner => $user, pwd => "$VHConst::VIRTUALFS/$user/etc/passwd" };
    return "Cannot find domain owner $user in password file $VHConst::VIRTUALFS/$user/etc/passwd\n";
}

# These two functions never need locking because the sendmail/telnet file is
# changed only when the main virtual hosting file is locked.
sub AddToSvcConf() {
    my($record, $svc)=@_;
    my $CONF;

    if ( ($svc eq "sendmail") and (not $record->{Sendmail}) ) {return""}
    if ( ($svc eq "telnet") and (not $record->{Telnet}) ) {return""}

    $CONF = get_svc_conf($svc);

    unless (open(CF,">>$CONF")) {
    $cerr += $E_UNABLEOPENSVCCONF and $cerr += { conf => $CONF } ;
	return "Cannot open service conf file $CONF .\n";
    }
    print CF "$record->{Domain} $record->{User}\n";
    close(CF);
    return "";
}

sub DeleteFromSvcConf() {
    my($record, $svc)=@_;
    my $CONF;

    $CONF = get_svc_conf($svc);

    unless (open(CF,$CONF)) {
    $cerr += $E_UNABLEREADSVCCONF and $cerr += { conf => $CONF } ;
	return "Cannot read service conf file $CONF .\n";
    }
    my (@lines) = <CF>;
    close (CF);
    my (@oldentries) = grep {$_ =~ /\s$record->{User}\n/} @lines;
    my ($error_message)="";
    if ($#oldentries == -1) {
	#no entry for our domain
	return "";
    }
    @lines = grep {$_ !~ /\s$record->{User}\n/} @lines;
    unless (open(CF,">$CONF")) {
    $cerr += $E_UNABLEWRITESVCCONF and $cerr += { conf => $CONF } ;
	return "Cannot write to service conf file $CONF .\n";
    }
    print CF @lines;
    close (CF);
    return "";
}

sub get_svc_conf($) {

    my ($svcname) = @_;

    if ($svcname eq "sendmail") { return $MAILCONF }
    if ($svcname eq "telnet") { return $TELNETCONF }
}

#This function does a DNS lookup to see if the DNS got updated
#properly after a new domain was added

sub CheckDNS() {
    my ($record)=@_;
    my ($ip4bytes);
    $ip4bytes = gethostbyname ($record->{Domain});
    if (defined $ip4bytes) {
	my ($ipdotted) = join(".",unpack('C4',$ip4bytes));
	if ($ipdotted ne $record->{IP}) {
        $cerr += $E_DNSMATCH and 
        $cerr += {name => "$record->{Domain}",ip1 => $ipdotted,ip2 => "$record->{IP}" } ;
	    print STDERR "Looking up the name $record->{Domain} I got $ipdotted instead of $record->{IP} .\n";
	}
    } else {
        $cerr += $E_UNABLELOOKUP and $cerr += { what => "$record->{Domain}" } ;
	print STDERR "Cannot look up $record->{Domain} . Make sure that the DNS was updated. You might have to restart your DNS server.\n";
    }
}

# This function uses the MivaMerchant modules to perform
# the actual task adding entries to miva*'s conf fileS
sub addMivaMerchant($) {
    my $record = shift;

    # Set Miva Values
    $record->{mivawebadmin} = $record->{Email} ;
    $record->{mivaroot}     = "$VHConst::VIRTUALFS/$record->{User}/home/httpd/html" ;
    $record->{mivadata}     = "$VHConst::VIRTUALFS/$record->{User}/home/httpd/mivadata" ;


    eval "require mivamerchant";
    if ($@) {
        $cerr += $E_UNABLEIMPORT and $cerr += { why => $@ };
        return "failed to import required modules... $@";
    }
    
    return "mivamerchant"->addVirtDomain ($record->{Domain},
                                          $record);
}

sub delMivaMerchant($) {
    my $record = shift;
    
    # Set Miva Values
    $record->{mivawebadmin} = $record->{Email} ;
    $record->{mivaroot}     = "$VHConst::VIRTUALFS/$record->{User}/home/httpd/html" ;
    $record->{mivadata}     = "$VHConst::VIRTUALFS/$record->{User}/home/httpd/mivadata" ;

    eval "require mivamerchant";
    if ($@) {
        $cerr += $E_UNABLEIMPORT and $cerr += { why => $@ };
        return "failed to import required modules... $@";
    }
    
    return "mivamerchant"->delVirtDomain ($record->{Domain},
                                          $record);
}

return 1;
