# Copyright (c) Ensim Corporation 2000, 2001   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#
# This module handles the virtual hosting configuration file
#
use lib ($ENV{'OCW_SVCPATH'} || "/usr/lib/opcenter")."/cmdline_common";

package VHConfiguration;
use Sys::Hostname;
use Locking;

use lib ($ENV{OCW_SVCPATH} or "/usr/lib/opcenter") . "/virtualhosting";
use lib ($ENV{OCW_SVCPATH} or "/usr/lib/opcenter") . "/cmdline_common";

use ERRORS;
use Carper;

BEGIN {
$TEMPLATEDIR="/etc/virtualhosting/virtualhosting_svc_templates";  #the templates dir
$VIRTHOSTCONF="/etc/virtualhosting/virthosts.conf";  #the conf file
$IPRANGECONF="/etc/virtualhosting/ipranges";   #the file storing the
                                               #valid IP ranges 
$UIDCONF="/etc/virtualhosting/uidranges.conf"; #uid ranges for domains
@TheTemplateFile = (); # The data from a template file
@TheConfFile = (); # The data from the conf file
@UIDConfFile = (); # The data from the uid ranges
$MinVirtGid = 0; #Minimum uid for virtual users (to be read from
$MaxVirtGid = 499;      #the uid ranges file)
$MinVirtUid = 100000; #Minimum uid for virtual users (to be read from
$MaxVirtUid = 0;      #the uid ranges file)
$ServerAddress = "0.0.0.0";
}

END {}

require LogConfFile;
use Config::IniFiles;

# Check if a domain name is valid
sub CheckDomain() {
    if ($_[0] =~ /^[a-zA-Z0-9-\.]+\.[a-zA-Z]+$/) {
	return "";
    } else {
    $cerr += $E_INVALIDDOMAINNAME and $cerr += { domain => $_[0] };
	return "Invalid domain name $_[0] .\n";
    }
}

# Check if a user name is valid
sub CheckUser() {
    if (length $_[0] > 31) {
        $cerr += $E_INVALIDUSER and $cerr += { user => $_[0] };
        $cerr += $E_MAXLEN and $cerr += { len => 32 } ;
        return "Invalid user name $_[0]. Must be less than 32 bytes in length .\n";
    } elsif ($_[0] =~ /^[a-zA-Z][a-zA-Z0-9_-]*(\.[a-zA-Z][a-zA-Z0-9_-]*)?$/) {
	return "";
    } else {
    $cerr += $E_INVALIDUSER and $cerr += { user => $_[0] };
	return "Invalid user name $_[0] .\n";
    }
}

# Check if a site user name is valid
sub CheckSiteUser() {
    my ($mysqlret) ; 

    if (length $_[0] > 16) {
        $cerr += $E_INVALIDUSER and $cerr += { user => $_[0] };
        $cerr += $E_MAXLENEQ and $cerr += { len => 16 } ;
        return "Invalid user name $_[0]. Must be 16 or less bytes in length .\n";
    } elsif ($_[0] =~ /^[a-zA-Z][a-zA-Z0-9_-]*(\.[a-zA-Z][a-zA-Z0-9_-]*)?$/) {
        $mysqlret = CheckMySQLUser($_[0]) ;
        if ( $mysqlret eq "" ) {
            $mysqlret = CheckMySQLDatabase($_[0]) ;
        }
	return $mysqlret ;
    } else {
    $cerr += $E_INVALIDSITEUSERNAME and $cerr += { user => $_[0] };
	return "Invalid site user name $_[0] .\n";
    }
}

# Check if a user name is mysql valid
sub CheckMySQLUser() {
    if (length $_[0] > 16) {
        $cerr += $E_INVALIDUSER and $cerr += { user => $_[0] } ;
        $cerr += $E_MAXLENEQ and $cerr += { len => 16 };
        return "Invalid user name $_[0]. Must be 16 or less bytes in length .\n";
    } elsif ($_[0] =~ /^[a-zA-Z][a-zA-Z0-9_-]*(\.[a-zA-Z][a-zA-Z0-9_-]*)?$/) {
	return "";
    } else {
    $cerr += $E_INVALIDMYSQLUSERNAME and $cerr += { user => $_[0] } ;
	return "Invalid mysql user name $_[0] .\n";
    }        
}

# Check if a database name is mysql valid
sub CheckMySQLDatabase() {
    if (length $_[0] > 64) {
        $cerr += $E_INVALIDDBNAME and $cerr += { db => $_[0] };
        $cerr += $E_MAXLENEQ and $cerr += { len => 64 };
        return "Invalid database name $_[0]. Must be 64 or less bytes in length .\n";
    } elsif ($_[0] =~ /^[a-zA-Z][a-zA-Z0-9_-]*(\.[a-zA-Z][a-zA-Z0-9_-]*)?$/) {
        if ( $_[0] eq "mysql" ) {
            $cerr += $E_INVALIDMYSQLDBNAME and $cerr += { db => $_[0] } ;
            return "Invalid mysql database name $_[0] .\n" ;
        }
	return "";
    } else {
    $cerr += $E_INVALIDMYSQLDBNAME and $cerr += { db => $_[0] } ;
	return "Invalid mysql database name $_[0] .\n";
    }        
}


# Check if a full user name is valid
sub CheckFullName() {
    my(@chars)=split(//,$_[0]);
    foreach (@chars) {
	if ($_ !~ /^[a-zA-Z0-9 \.\'_-]$/ and ord($_) <128) {
        $cerr += $E_INVALIDFULLNAME and $cerr += { name => $_[0] } ;
	    return "Invalid full name $_[0] .\n";
	}
    }
    return "";
}

# Check if ScriptAlias is valid
sub CheckScriptAlias() {
    if ($_[0] eq "html") {
        $cerr += $E_INVALIDSCRIPTALIAS and $cerr += { what => $_[0] } ;
        return "Invalid ScriptAlias $_[0] .\n";
    } elsif ($_[0] =~ /^[a-zA-Z0-9_-]*(\.[a-zA-Z][a-zA-Z0-9_-]*)?$/) {
	return "";
    } else {
    $cerr += $E_INVALIDSCRIPTALIAS and $cerr += { what => $_[0] } ;
	return "Invalid ScriptAlias $_[0] .\n";
    }
}

# Check if a web server name is valid
sub CheckWeb() {
    if ($_[0] =~ /^[a-zA-Z0-9-\.]+\.[a-zA-Z]+$/) {
	return "";
    } else {
    $cerr += $E_INVALIDWEBSERVERNAME and $cerr += { what => $_[0] } ;
	return "Invalid web server name $_[0] .\n";
    }
}

# Check if an ftp server name is valid
sub CheckFtp() {
    if ($_[0] =~ /^[a-zA-Z0-9-\.]+\.[a-zA-Z]+$/) {
	return "";
    } else {
    $cerr += $E_INVALIDFTPSERVERNAME and $cerr += { what => $_[0] };
	return "Invalid ftp server name $_[0] .\n";
    }
}
# Check if an email is valid
sub CheckEmail() {
    if ($_[0] =~ /^[a-zA-Z0-9-\.-\_]+\@[a-zA-Z0-9-\.]+\.[a-zA-Z]+$/) {
	return "";
    } else {
    $cerr += $E_INVALIDEMAIL and $cerr += { what => $_[0] };
	return "Invalid email address $_[0] .\n";
    }
}
# Check if a MaxUser parameter is valid
sub CheckMaxUser() {
    if ($_[0] =~ /^[0-9]+$/ and $_[0] >= 0) {
	return "";
    } else {
    $cerr += $E_INVALIDMAXUSER and $cerr += { what => $_[0] };
	return "Invalid MaxUser parameter $_[0] .\n";
    }
}
# Check if a Quota parameter is valid
sub CheckQuota() {
    if ($_[0] =~ /^[0-9]+(\.[0-9]*)?$/ and $_[0] >= 0) {
	return "";
    } else {
    $cerr += $E_INVALIDQUOTA and $cerr += { what => $_[0] } ;
	return "Invalid Quota parameter $_[0] .\n";
    }
}
# Check if an IP address is valid
sub CheckIP() {
  my ($IP)=@_;
  my ($byte1,$byte2,$byte3,$byte4);
  if ( $IP =~ /^([0-9]+)\.([0-9]+)\.([0-9]+)\.([0-9]+)$/) {
    $byte1=$1;
    $byte2=$2;
    $byte3=$3;
    $byte4=$4;
  } else {
    $cerr += $E_INVALIDIP and $cerr += { what => $IP } ;
    return "Invalid IP address $IP .\n";
  }
  if ($byte1 > 255 or $byte2 > 255 or $byte3 > 255 or $byte4 > 255) {
    $cerr += $E_INVALIDIP and $cerr += { what => $IP } ;
      return "Invalid IP address $IP .\n";
  }
  $byte1 =~ s/^(0*)([0-9]+)/$2/;
  $byte2 =~ s/^(0*)([0-9]+)/$2/;
  $byte3 =~ s/^(0*)([0-9]+)/$2/;
  $byte4 =~ s/^(0*)([0-9]+)/$2/;
  my ($hostaddr)=&VHConfiguration::GetServerAddress;
  if ($hostaddr eq "$byte1.$byte2.$byte3.$byte4") {
  $cerr += $E_UNABLEUSESRVADDR and $cerr += { addr => $hostaddr };
      return "Cannot use the address of the server $hostaddr .\n";
  }
  unless (open(I,"$IPRANGECONF")) {
  $cerr += $E_UNABLEOPENIPRANGE and $cerr += { what => $IPRANGECONF };
    return "Cannot open IP range file $IPRANGECONF .\n";
  }
  my ($entries)=0;
  while (<I>) {
    next if ($_ =~ /^\s*\#/);
    $entries++;
    if ($_ =~ /^([0-9]+)\.([0-9]+)\.([0-9]+)\.([0-9-]+)/) {
      if ($byte1 eq $1 and $byte2 eq $2 and $byte3 eq $3) {
	if ($byte4 eq $4) {
	  return "";
	} elsif($4 =~ /^([0-9]+)-([0-9]+)$/) {
	  if ($byte4 >= $1 and $byte4 <= $2) {
	    return "";
	  }
	}
      }
    } 
  }
  close (I);
  if ($entries) {
    $cerr += $E_IPOUTOFRANGE and $cerr += { ip => $IP };
    return "IP address $IP outside allowed ranges\n";
  }
  return "";
}


#
# Reading the template file
#
sub ReadTemplateFile {
  my ($templateName)=@_;
  if (@TheTemplateFile) {return;} #return if we already have the file
  unless (open (TEMPLATEFILE,"$templateName")) {
  $cerr += $E_UNABLEOPENTEMPLATEFILE and $cerr += { file => $templateName };
    return "Cannot open template file $templateName .\n";
  }
  @TheTemplateFile = <TEMPLATEFILE>;
  close (TEMPLATEFILE);
  my ($i);
  my ($error_message)="";
  #Parse the file
  for ($i=0; $i<=$#TheTemplateFile;$i++) {
    next if ($TheTemplateFile[$i] =~ /^\s*\#/);
    my (@fields) = split (/\s+/,$TheTemplateFile[$i]);
    if ($#fields != 19) 
    {
    $cerr += $E_TMPLCORRUPTEDHTML ;
      $error_message = "<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;The service plans template file is probably corrupted.\n";
      $cerr += $E_INVALIDLINEHTML and $cerr += { line => $i+1, file => $templateName } ;
      $error_message .= "<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Line ".($i+1)." invalid in template file " . $templateName . ".\n";
      return $error_message;
    }
    my $record = {};
    $record->{WebLogs} = $fields[0];
    $record->{CGI} = $fields[1];
    $record->{SSI} = $fields[2];
    $record->{FPExt} = $fields[3];
    $record->{SSL} = $fields[4];
    $record->{AnonFtp} = $fields[5];
    $record->{MySQL} = $fields[6];
    $record->{Telnet} = $fields[7];
    $record->{Sendmail} = $fields[8];
    $record->{FTP} = $fields[9];
    $record->{Majordomo} = $fields[10];
    $record->{IMAP} = $fields[11];
    $record->{Vacation} = $fields[12];
    $record->{Web} = $fields[13];
    $record->{Analog} = $fields[14];
    $record->{Backup} = $fields[15];
    $record->{FileMgr} = $fields[16];
    $record->{maxusers} = $fields[17];
    $record->{MivaMerchant} = $fields[18];

    # Note: quota must be last field

    $record->{quota} = $fields[19];
    $TheTemplateFile[$i] = $record;
  }
  return $error_message;
}

sub PushLock() {
    return Locking::PushLock($VIRTHOSTCONF);
}

sub Lock() {
    return Locking::lock($VIRTHOSTCONF);
}

sub Unlock() {
    Locking::unlock($VIRTHOSTCONF);
}

sub CheckLock() {
    Locking::Check($VIRTHOSTCONF);
}


#
# Reading the conf file
#
sub ReadConfFile {

  if (@TheConfFile) {return;} #return if we already have the file

    my($i) = 0;
    my($error_message)="";

  opendir(SITEDIR, "/home/virtual") or 
    $cerr += $E_UNABLEREADFILE and $cerr += { file => "/home/virtual" } and
    return "Could not open /home/virtual\n";
  @allsites = grep { /site[0-9]+/ } readdir SITEDIR;
  closedir SITEDIR;  

  foreach $site (@allsites) {
      my $record = {};
      opendir(SITEDIR, join("", "/home/virtual/", $allsites[$i], "/info"));
      @allconfs = grep { not /^(new|old|\.{1,2})$/ } readdir SITEDIR;
      closedir SITEDIR;
      foreach (@allconfs) {
          my %ini;
          tie (%ini,'Config::IniFiles', 
               (-file => join("", "/home/virtual/", $allsites[$i], "/info/", $_)));
        SWITCH: {
            /^siteinfo/ && 
                do { $record->{Domain} = $ini{DEFAULT}{domain};
                     $record->{User} = $ini{DEFAULT}{admin};
                     $record->{Email} = $ini{DEFAULT}{email};
                     $record->{Enabled} = 1;
                     last SWITCH; };
            /^ipinfo/ && 
                do { 
                    if($ini{DEFAULT}{namebased}) {
                        $record->{DomainType} = "NameBased";
                        $ini{DEFAULT}{nbaddrs} =~ tr/[']//d;
                        $record->{IP} = (split(/,/,$ini{DEFAULT}{nbaddrs}))[0];
                    } else {
                        $record->{DomainType} = "IPBased";
                        $ini{DEFAULT}{ipaddrs} =~ tr/[']//d;
                        $record->{IP} = (split(/,/,$ini{DEFAULT}{ipaddrs}))[0];
                    }
                    last SWITCH; };
            /^diskquota/ &&
                do { $record->{Quota} = $ini{DEFAULT}{quota};
                     last SWITCH; };
            /^users/ &&
                do { $record->{MaxUsers} = $ini{DEFAULT}{maxusers};
                     last SWITCH; };
            /^backup/ &&
                do { $record->{Backup} = $ini{DEFAULT}{enabled};
                     last SWITCH; };
            /^files/ &&
                do { $record->{FileMgr} = $ini{DEFAULT}{enabled};
                     last SWITCH; };
            /^bind/ && do { last SWITCH; };
            /^mysql/ &&
                do { $record->{MySQL} = $ini{DEFAULT}{enabled};
                     last SWITCH; };
            /^telnet/ &&
                do { $record->{Telnet} = $ini{DEFAULT}{enabled};
                     last SWITCH; };
            /^apache/ && 
                do { $record->{WebServerName} = $ini{DEFAULT}{webserver};
                     $record->{Web} = $ini{DEFAULT}{enabled};
                     last SWITCH; };
            /^sendmail/ &&
                do { $record->{Sendmail} = $ini{DEFAULT}{enabled};
                     last SWITCH; };
            /^proftpd/ &&
                do { $record->{FtpServerName} = $ini{DEFAULT}{ftpserver};
                     $record->{FTP} = $ini{DEFAULT}{enabled};
                     last SWITCH; };
            /^weblogs/ && 
                do { $record->{WebLogs} = $ini{DEFAULT}{enabled};
                     last SWITCH; };
            /^cgi/ &&
                do { $record->{CGI} = $ini{DEFAULT}{enabled};
                     $record->{ScriptAlias} = $ini{DEFAULT}{scriptalias};
                     last SWITCH; };
            /^ssi/ &&
                do { $record->{SSI} = $ini{DEFAULT}{enabled};
                     last SWITCH; };
            /^openssl/ &&
                do { $record->{SSL} = $ini{DEFAULT}{enabled};
                     last SWITCH; };
            /^analog/ &&
                do { $record->{Analog} = $ini{DEFAULT}{enabled};
                     last SWITCH; };
            /^mivamerchant/ &&
                do { $record->{MivaMerchant} = $ini{DEFAULT}{enabled};
                     last SWITCH; };
            /^vacation/ &&
                do { $record->{Vacation} = $ini{DEFAULT}{enabled};
                     last SWITCH; };
            /^imap/ &&
                do { $record->{IMAP} = $ini{DEFAULT}{enabled};
                     last SWITCH; };
            /^squirrelmail/ && do { last SWITCH; };
            /^majordomo/ &&
                do { $record->{Majordomo} = $ini{DEFAULT}{enabled};
                     last SWITCH; };
            /^anonftp/ &&                
                do { $record->{AnonFtp} = $ini{DEFAULT}{enabled};
                     last SWITCH; };
            /^frontpage/ &&
                do { $record->{FPExt} = $ini{DEFAULT}{enabled};
                     last SWITCH; };
        }
          undef %ini;
      }
      $TheConfFile[$i] = $record;
      $i++;
  }

  return $error_message;
}

#
# Writing out the template file
#
sub WriteTemplateFile {
  my ($templfile)=@_;
  unless (open (TEMPLATEFILE,">$templfile")) {
    $cerr += $E_UNABLEWRITEFILE and $cerr += { file => $templfile };
    return "Cannot open template file $templfile for writing.\n";
  }
  my $line;
  foreach $line (@TheTemplateFile) {
    if (ref $line) {
      print TEMPLATEFILE &FormatTemplateFileLine($line);
    } else { #It's a comment
      print TEMPLATEFILE $line;
    }
  }
  close (TEMPLATEFILE);
  return "";
}

#
# Writing out the conf file
#
sub WriteConfFile {
  my $VIRTCONFHANDLE = 
      LogConfFile::open_conf("/etc/virtualhosting",
			     "/var/log/conf","virthosts.conf","w");

  if ($VIRTCONFHANDLE eq "") {
  $cerr += $E_UNABLEWRITEVCONF and $cerr += { file => $VIRTHOSTCONF };
      return "Cannot open virtual hosting file $VIRTHOSTCONF for writing.\n";
  }
  
  CheckLock();

#
# TODO need lock
#

#  my $line;
#  foreach $line (@TheConfFile) {
#      foreach svc ('siteinfo', 'ipinfo', 'diskquota', 'users', 'backup', 'files', 
#                   'mysql', 'telnet', 'apache', 'sendmail', 'proftpd', 'weblogs', 
#                     'cgi', 'ssi', 'openssl', 'analog', 'mivamerchant', 'vacation',
#                     'imap', 'majordomo', 'anonftp', 'frontpage') {
#            site = split();
#            $cfg = Config::IniFiles->new(-default => 'default');
#          SWITCH: {
            
#          }
#        }             
#    }

  my $line;
  foreach $line (@TheConfFile) {
      if (ref $line) {
	  print $VIRTCONFHANDLE &FormatConfFileLine($line);
      } else { 
	  #It's a comment
	  print $VIRTCONFHANDLE $line;
      }
  }

  if (!(LogConfFile::close_conf($VIRTCONFHANDLE,
			       "/etc/virtualhosting","/var/log/conf",
				"virthosts.conf"))) {
      $cerr += $E_UNABLECOMMITVCONF ;
      return "Cannot commit virtual hosting file. \n";
  }
  return "";
}

#formats a record into a line for printing into the conf file
sub FormatTemplateFileLine () {
  my ($record) = @_;
  $excessspaces=0;
  return &field("$record->{WebLogs} $record->{CGI} $record->{SSI} $record->{FPExt} $record->{SSL}",2).
     &field("$record->{AnonFtp}",1).
     &field("$record->{MySQL}",1).
     &field("$record->{Telnet}",1).
     &field("$record->{Sendmail}",1).
     &field("$record->{FTP}",1).
     &field("$record->{Majordomo}",1).
     &field("$record->{IMAP}",1).
     &field("$record->{Vacation}",1).
     &field("$record->{Web}",1).
     &field("$record->{Analog}",1).
     &field("$record->{Backup}",1).
     &field("$record->{FileMgr}",1).
     &field("$record->{maxusers}",1).
     &field("$record->{MivaMerchant}",1).

     # Note: quota must be last field
     "$record->{quota}\n";
}

#formats a record into a line for printing into the conf file
sub FormatConfFileLine () {
  my ($record) = @_;
  $excessspaces=0;
  return &field($record->{Domain},3). 
     &field($record->{User},1).
     &field($record->{Enabled},1).
     &field((($record->{DomainType} eq "IPBased")?$record->{IP}:"0.0.0.0"),2).
     &field($record->{WebServerName},3).
     &field("$record->{WebLogs} $record->{CGI} $record->{SSI} $record->{FPExt} $record->{SSL}",2).
     &field($record->{FtpServerName},3).
     &field("$record->{AnonFtp}",1).
     &field("$record->{MySQL}",1).
     &field("$record->{MaxUsers}",1).
     &field("$record->{Telnet}",1).
     &field("$record->{Sendmail}",1).
     &field("$record->{Quota}",1).
     &field("$record->{Email}",3).
     &field("$record->{FTP}",1).
     &field("$record->{Majordomo}",1).
     &field("$record->{IMAP}",1).
     &field("$record->{Vacation}",1).
     &field("$record->{Web}",1).
     &field("$record->{Analog}",1).
     &field("$record->{Backup}",1).
     &field("$record->{FileMgr}",1).
     &field("$record->{MivaMerchant}",1).
     "$record->{ScriptAlias}\n";

}

#pads a field in the conf file with spaces
sub field() {
  my ($data,$tabs)=@_;
  #my $excessspaces=0;
  my $spaceleft=8*$tabs-(length($data)+$excessspaces);
  if ($spaceleft>0) {
    while ($spaceleft>0) {
      $data .= "\t";
      $spaceleft -= 8;
    }
    $excessspaces=0;
  } else {
    $data .= " ";
    $excessspaces = 1 - $spaceleft;
  }
  return $data;
}

#
# Reading the record related to a template into the list
# provided by the caller.
#
sub GetTemplate() {
  my ($templateName,$precord)=@_;
  my ($error_message);

  unless (open(TEMPLATEFILE,"$TEMPLATEDIR/$templateName")) {
    $cerr += $E_UNABLEOPENTEMPLATEFILE and $cerr += { file => "$TEMPLATEDIR/$templateName" };
       return "Could not open template file $TEMPLATEDIR/$templateName for reading.\n";
    }
  $templfile = "$TEMPLATEDIR/$templateName";
  close(TEMPLATEFILE);
  if ($error_message = &ReadTemplateFile($templfile)) {return $error_message}
  # Go through the Template file
  my ($record);
  foreach $record (@TheTemplateFile) {
    next if (! ref($record));#This is a comment
    $$precord=$record;
    return "";
  }
  $cerr += $E_NODOMAININCONF and $cerr += { domain => $domain } ;
  return "Domain $domain not found in configuration file.\n";
}

#
# Reading the record related to a virtual domain into the list
# provided by the caller.
#
sub GetDomainConf() {
  my ($domain,$precord)=@_;
  my ($error_message);
  if ($error_message = &ReadConfFile()) {return $error_message}
  # Go through the conf file
  my ($record);
  foreach $record (@TheConfFile) {
    next if (! ref($record));#This is a comment
    if ($record->{Domain} eq $domain) {
      $$precord=$record;
      return "";
    }
  }
  $cerr += $E_NODOMAININCONF and $cerr += { domain => $domain } ;
  return "Domain $domain not found in configuration file.\n";
}

#
# It writes out changes made to the line of a domain back into the
# virtual hosting conf file.
#
sub SetDomainConf() {
  my ($record)=@_;
  my ($error_message);
  if ($error_message = &ReadConfFile()) {return $error_message}
  # Go through the conf file
  my ($crt);
  foreach $crt (@TheConfFile) {
    next if (! ref($crt));#This is a comment
    if ($crt->{Domain} eq $record->{Domain}) {
      $crt=$record;
      return &WriteConfFile();
    }
  }
  $cerr += $E_NODOMAININCONF and $cerr += { domain => $domain } ;
  return "Domain $domain not found in configuration file.\n";
}

# Formats for printing the data for a template
sub FormatTemplateData() {
  my ($record)=@_;
  return "$record->{WebLogs} ".
	 "$record->{CGI} ".
	 "$record->{SSI} ".
	 "$record->{FPExt} ".
	 "$record->{SSL} ".
	 "$record->{AnonFtp} ".
	 "$record->{MySQL} ".
	 "$record->{Telnet} ".
	 "$record->{Sendmail} ".
	 "$record->{FTP} ".
	 "$record->{Majordomo} ".
	 "$record->{IMAP} ".
	 "$record->{Vacation} ".
	 "$record->{Web} ".
	 "$record->{Analog} ".
	 "$record->{Backup} ".
	 "$record->{FileMgr} ".
	 "$record->{maxusers} ".
         "$record->{MivaMerchant} ".
	 "$record->{quota}\n";
}

# Formats for printing the data about a virtual domain
sub FormatVirtHostData() {
  my ($record)=@_;
  return "$record->{Domain} ".
         "$record->{DomainType} ".
         "$record->{User} ".
	 "$record->{Enabled} ".
	 (($record->{DomainType} eq "IPBased")?"$record->{IP} ":"0.0.0.0 ").
         "$record->{WebServerName} ".
	 "$record->{WebLogs} ".
	 "$record->{CGI} ".
	 "$record->{SSI} ".
	 "$record->{FPExt} ".
	 "$record->{SSL} ".
         "$record->{FtpServerName} ".
	 "$record->{AnonFtp} ".
	 "$record->{MySQL} ".
	 "$record->{MaxUsers} ".
	 "$record->{Telnet} ".
	 "$record->{Sendmail} ".
	 "$record->{Quota} ".
	 "$record->{Email} ".
	 "$record->{FTP} ".
	 "$record->{Majordomo} ".
	 "$record->{IMAP} ".
	 "$record->{Vacation} ".
	 "$record->{Web} ".
	 "$record->{Analog} ".
	 "$record->{Backup} ".
	 "$record->{FileMgr} ".
         "$record->{MivaMerchant} ".
         "$record->{ScriptAlias}\n";
}

#
# Add a new template file 
#
sub AddToTemplateFile() {
  my ($templfile,$newrecord)=@_;
  my ($error_message);

  #if ($error_message = &ReadTemplateFile()) {return $error_message}
  #Check the conf file for duplicates
  push @TheTemplateFile,$newrecord;
  if ($error_message = &WriteTemplateFile($templfile)) {return $error_message}
  return "";
}

#
# Add a new entry to the configuration file (and to the uid ranges file)
#
sub AddToConfFile() {
  my ($newrecord)=@_;
  my ($error_message);

  if ($error_message = &ReadConfFile()) {return $error_message}
  if ($error_message = &ReadUIDConfFile()) {return $error_message}
  #Check the conf file for duplicates
  my ($record,$count,$namecount);
  $count=$namecount=0;
  foreach $record (@TheConfFile) {
    next if (! ref ($record)); #comment line
    if ($record->{Domain} eq $newrecord->{Domain}) {
    $cerr += $E_DOMAINALREADYINCONF and $cerr += { domain => $newrecord->{Domain} };
      return "Domain $newrecord->{Domain} already exists in the configuration file.\n";
    }
    if ($record->{User} eq $newrecord->{User}) {
    $cerr += $E_DOMAINALREADYINCONF and $cerr += { domain => $newrecord->{User} };
      return "User $newrecord->{User} already exists in the configuration file.\n";
    }
    if ($newrecord->{DomainType} eq "IPBased" and
	$record->{DomainType} eq "IPBased" and
	$record->{IP} eq $newrecord->{IP}) {
    $cerr += $E_DOMAINALREADYINCONF and $cerr += { domain => $newrecord->{IP} };
      return "IP address $newrecord->{IP} already exists in the configuration file.\n";
    }
    $count++;
    $namecount++ if ($record->{DomainType} eq "NameBased");
  }
  my ($maxcount,$maxnamecount);
  &ReadLimits(\$maxcount,\$maxnamecount);
  if ($count >= $maxcount ) {
    $cerr += $E_NOFURTHERSITES ;
      return "You are not allowed to create further site.\n";
  }
  if ($newrecord->{DomainType} eq "NameBased" and $namecount >= $maxnamecount){
  $cerr += $E_NOFURTHERNBD ;
      return "You are not allowed to create further name based domains.\n";
  }
  push @TheConfFile,$newrecord;
  if ($error_message = &WriteConfFile()) {return $error_message}
  my ($uidstoallocate) = $newrecord->{MaxUsers};
  $newrecord->{Ranges} = [];
  push @UIDConfFile,$newrecord;
  my ($range);
  while ($uidstoallocate) {
    $range=&GetUIDRange($uidstoallocate);
    if (ref ($range)) {
      push @{$newrecord->{Ranges}},({"min"=>$range->[0],"max"=>$range->[1]});
      $uidstoallocate -= $range->[1]-$range->[0]+1;
    } else {
      # we got an error message
      if ($range eq "Could not find any free UID ranges" and $#{$newrecord->{Ranges}}>=0) {
      $cerr += $E_UIDALLOCERR and $cerr += { howmany => ($newrecord->{MaxUsers}-$uidstoallocate) };
	return "Could only allocate ".($newrecord->{MaxUsers}-$uidstoallocate)." uids, aborting.\n";
      } else {
	return $range;
      }
    }
  }
  if ($error_message = &WriteUIDConfFile()) {return $error_message}
  return "";
}

#
# Remove an entry from the configuration file
#
sub DeleteFromConfFile() {
  my ($Domain)=@_;
  my ($error_message);
  if ($error_message = &ReadConfFile()) {return $error_message}
  &ReadUIDConfFile();
  my ($i);
  for ($i=0;;$i++) {
    if (ref($TheConfFile[$i]) and $TheConfFile[$i]->{Domain} eq $Domain) {
      splice (@TheConfFile,$i,1);
      $error_message = &WriteConfFile();
      last;
    }
    if ($i+1>$#TheConfFile) {
        $cerr += $E_NODOMAININCONF and $cerr += { domain => $Domain };
      $error_message .= "Domain $Domain not found in the conf file.\n";
      last;
    }
  }
  for ($i=0;;$i++) {
    if (ref($UIDConfFile[$i]) and $UIDConfFile[$i]->{Domain} eq $Domain) {
      splice (@UIDConfFile,$i,1);
      $error_message .= &WriteUIDConfFile();
      last;
    }
    if ($i+1>$#UIDConfFile) {
        $cerr += $E_NODOMAININUID and $cerr += { domain => $Domain } ;
      $error_message .= "Domain $Domain not found in the uid ranges file.\n";
      last;
    }
  }
  return $error_message;
}

sub ReadLimits() {
    my ($pmax,$pmaxname) =@_;
    $$pmax=$$pmaxname=100000;
    open (MAX,"$IPRANGECONF") or return;
    while (<MAX>) {
	if (/^\#\s*MaxDomains\s+(\d+)/) {
	    $$pmax=$1;
	} elsif (/^\#\s*MaxDomains\s+\*/) {
	    $$pmax=100000;
	} elsif (/^\#\s*MaxNameDomains\s+(\d+)/) {
	    $$pmaxname=$1;
	} elsif (/^\#\s*MaxNameDomains\s+\*/) {
	    $$pmaxname=100000;
	}
    }
}


#
# Retirns group associated with the owner of a virtual domain
#
sub getGroup() {
    my($record) = @_;
    my(@pwent) = getpwnam($record->{User});
    unless (@pwent) {
    $cerr += $E_INVALIDUSER and $cerr += { user => $User };
      return "Invalid user name $User .\n";
    }
    my(@grent) = getgrgid($pwent[3]);
    unless (@grent) {
        $cerr += $E_INVALIDGRPID and $cerr += { user => $User } ;
        return "Invalid group if for $User .\n";
    }
    $Group = $grent[0];
    return $Group;
}

#
# Reading the uid ranges configuration file
#
sub ReadUIDConfFile {

  if (@UIDConfFile) {return;} #return if we already have the file
  unless (open (UIDFILE,"$UIDCONF")) {
    $cerr += $E_UNABLEOPENUIDRANGES and $cerr += { file => $UIDCONF };
    return "Cannot open uid ranges file $UIDCONF .\n";
  }
  @UIDConfFile = <UIDFILE>;
  close (UIDFILE);
  unless (@TheConfFile) {
    $cerr += $E_UIDBEFORECONF ;
    return "Internal error, the uid ranges file read before the conf file.\n";
  }
  my ($i,$n); #We use $n to walk through the conf file
  my ($error_message)="";
  #Parse the file
  for ($i=$n=0; $i<=$#UIDConfFile;$i++) {
    next if ($UIDConfFile[$i] =~ /^\s*\#/);
    my (@fields) = split (/\s+/,$UIDConfFile[$i]);
    if ($MinVirtUid > $MaxVirtUid) {
      if ($fields[0] ne "VirtualUIDRange") {
      $cerr += $E_VIRTUIDRANGEERR ;
	return "The uid ranges file doesn't start with the VirtualUIDRange line.\n";
      } elsif ($#fields != 1) {
      $cerr += $E_VIRTUIDSYNTAXERR ;
	return "Syntax error in the VirtualUIDRange line.\n";
      } else {
	($MinVirtUid,$MaxVirtUid) = split(/-/,$fields[1]);
	if ($MinVirtUid>$MaxVirtUid or $MinVirtUid<=0 or $MaxVirtUid>=65536) {
    $cerr += $E_VIRTUIDINVALIDVALS and 
    $cerr += { val1 => $MinVirtUid, val2 => $MaxVirtUid };
	  return "Invalid values \"$MinVirtUid\" and \"$MaxVirtUid\" in the VirtualUIDRange line.\n";
	} else {
	  next;
	}
      }
    }
    #OK, we gotta have a valid domain line here
    my ($domain) = shift @fields;
    while (not ref($TheConfFile[$n])){
      $n += 1;
      if ($n > $#TheConfFile) {
      $cerr += $E_UIDRANGESINVALIDLINE and
      $cerr += { line => ($i+1) };
	return $error_message."Line ".($i+1)." invalid in uid ranges file.\n";
      }
    }
    my ($record) = $TheConfFile[$n];
    $n += 1;
    if ($domain ne $record->{Domain}) {
    $cerr += $E_UIDRANGESINVALIDDOMAIN and 
    $cerr += { domain1 => $domain, domain2 => $record->{Domain} };
      $error_message .= "Invalid or misplaced domain $domain ($record->{Domain} expected) in uid ranges file.\n";
      next;
    }
    #domain OK, let's see the ranges
    my ($min,$max);
    $record->{Ranges} = [];
    $UIDConfFile[$i] = $record;
    my ($range);
    while ($range = shift(@fields)) {
      ($min,$max) = ("","");
      ($min,$max) = split(/-/,$range);
      if ($min>$max or $min<$MinVirtUid or $max>$MaxVirtUid) {
      $cerr += $E_UIDRANGESINVALIDUIDRANGE and 
      $cerr += { uid => $min-$max, domain => $domain } ;
	$error_message .= "Invalid uid range $min-$max for $domain in uid ranges file.\n";
	next;
      }
      push @{$record->{Ranges}},({"min"=>"$min","max"=>"$max"});
    }
  }
  return $error_message;
}

#
# Writing out the uid range conf file
#
sub WriteUIDConfFile {
  unless (open (UIDCONFFILE,">$UIDCONF")) {
  $cerr += $E_UNABLEWRITEUIDRANGES and $cerr += { file => $UIDCONF };
    return "Cannot open uid ranges file $UIDCONF for writing.\n";
  }
  my $line;
  foreach $line (@UIDConfFile) {
    if (ref $line) {
      print UIDCONFFILE &FormatUIDFileLine($line);
    } else { #It's a comment (or the VirtualUIDRange line)
      print UIDCONFFILE $line;
    }
  }
  close (UIDCONFFILE);
  return "";
}

sub FormatUIDFileLine() {
  my($record)=@_;
  my(@ranges)= (map {"$_->{min}-$_->{max}"} @{$record->{Ranges}});
  return "$record->{Domain} ".(join (" ",@ranges))."\n";
}

#
# Gets a range of available uids for virtual domains. Returns the
# range on success and an error message on failure.
#
sub GetUIDRange() {
  my ($size)=@_;
  if ($size<1) {
    $cerr += $E_INVALIDSIZE and $cerr += { size => $size } ;
    return "Invalid size $size for a UID range.\n";}
  $size-=1;
  my ($error_message) = &ReadConfFile();
  if ($error_message) {return $error_message;}
  if ($error_message=&ReadUIDConfFile()) {return $error_message;}
  my @freeuidranges = ([$MinVirtUid,$MaxVirtUid]);
  # The uid ranges file is almost sorted, so constructing the sorted
  # list of free uid ranges doesn't take as long as it seems by
  # looking at the algorithm.
  my ($line);
  foreach $line (@UIDConfFile) {
    next if (not ref ($line));
    my ($range);
    foreach $range (@{$line->{Ranges}}) {
      my ($i);
      for ($i=0;$i<=$#freeuidranges;$i++) {
	next if ($range->{min} > $freeuidranges[$i]->[1]);
	if ($range->{min} < $freeuidranges[$i]->[0]) {
    $cerr += $E_UIDOVERLAP and
    $cerr += { domain => $line->{Domain}, min => $range->{min}, 
        max => (&MIN($freeuidranges[$i]->[0],$range->{max})) };
	  return "UIDs overlap for $line->{Domain} from $range->{min} to ".
	    (&MIN($freeuidranges[$i]->[0],$range->{max}))." .\n";
	}
	if ($range->{max} > $freeuidranges[$i]->[1]) {
    $cerr += $E_UIDOVERLAP and
    $cerr += { domain => $line->{Domain}, 
        min => ($freeuidranges[$i]->[1]+1), 
        max => $range->{max} };
	  return "UIDs overlap for $line->{Domain} from ".
	    ($freeuidranges[$i]->[1]+1)." to $range->{max}.\n";
	}
	#OK, we have a match
	if ($range->{min} == $freeuidranges[$i]->[0]) {
	  if ($range->{max} == $freeuidranges[$i]->[1]) {
	    splice (@freeuidranges,$i,1);
	  } else {
	    $freeuidranges[$i]->[0] = $range->{max}+1;
	  }
	} else {
	  if ($range->{max} == $freeuidranges[$i]->[1]) {
	    $freeuidranges[$i]->[1] = $range->{min}-1;
	  } else {
	    splice (@freeuidranges,$i,1,
		    ([$freeuidranges[$i]->[0],$range->{min}-1],
		     [$range->{max}+1,$freeuidranges[$i]->[1]]));
	  }
	}
	last; # once we have the match, we are not looking at
	      # @freeuidranges anymore
      }
    }
  }
  my ($result,$crtsize,$i);
  $crtsize=-1;
  for ($i=$#freeuidranges; $i>=0; $i--){
    next if ($freeuidranges[$i]->[1] - $freeuidranges[$i]->[0] <= $crtsize);
    if ($freeuidranges[$i]->[1] - $freeuidranges[$i]->[0] >= $size) {
      return [$freeuidranges[$i]->[1]-$size,$freeuidranges[$i]->[1]];
    } else {
      $result=$freeuidranges[$i];
      $crtsize=$freeuidranges[$i]->[1] - $freeuidranges[$i]->[0];
    }
  }
  if ($crtsize>=0) {
    return $result;
  } else {
  $cerr += $E_UNABLEFINDFREEUIDRANGE ;
    return "Could not find any free UID ranges";
  }
}

#
# This function reads the /etc/passwd file of a virtual domain and
# extends the uid Ranges structure with information found therein.
#
sub ParseDomainPasswd {
  my ($record)=@_;
  unless (open(PASSWD,"$VHConst::VIRTUALFS/$record->{User}/etc/passwd")) {
    $cerr += $E_UNABLEREADSITEPWD and 
    $cerr += { file => "$VHConst::VIRTUALFS/$record->{User}/etc/passwd" };
    return "Could not open site password file $VHConst::VIRTUALFS/$record->{User}/etc/passwd for reading.\n";
  }
  map {$_->{used}=0;$_->{usedmin}=$_->{max}+1;$_->{usedmax}=$_->{min}-1}
      @{$record->{Ranges}};
  while (<PASSWD>){
    my (@fields)=split(/:/);
    my ($uid)=$fields[2];
    next if ($uid > $MaxVirtUid or $uid < $MinVirtUid);
    my ($range);
    foreach $range (@{$record->{Ranges}}) {
      next if ($uid > $range->{max} or $uid < $range->{min});
      $range->{used}++;
      if ($uid > $range->{usedmax}) {$range->{usedmax}=$uid;}
      if ($uid < $range->{usedmin}) {$range->{usedmin}=$uid;}
      last;
    }
  }
  close(PASSWD);
  return "";
}

sub ChangeMaxUsers() {
  my($newnumber,$record)=@_;
  my($vroot)="$VHConst::VIRTUALFS/$record->{User}";
  my($error_message);
  my $total=0;
  map {$total+=$_->{max}-$_->{min}+1} @{$record->{Ranges}};
  if ($total != $record->{MaxUsers}) {
    #This should never happen unless someone sticks his fingers in the
    #conf files. Let's ignore it.
    $record->{MaxUsers} = $total;
  }
  if ($newnumber>$total) {
    #So we want to add new ranges
    while ($newnumber>$total) {
      my ($range)=&GetUIDRange($newnumber-$total);
      if (ref ($range)) {
	push @{$record->{Ranges}},({"min"=>$range->[0],"max"=>$range->[1]});
	$total += $range->[1]-$range->[0]+1;
      } else {
	# we got an error message
	if ($range eq "Could not find any free UID ranges") {
	  last;
	} else {
	  return $range; #It's an error message
	}
      }
    }
    if ($total>$record->{MaxUsers}){
      $record->{MaxUsers}=$total;
      #Combine adjacent ranges
      my($i,$j);
      for($i=0;$i<$#{$record->{Ranges}};$i++) {
	for ($j=$i+1;$j<=$#{$record->{Ranges}};){
	  if ($record->{Ranges}->[$i]->{min}-$record->{Ranges}->[$j]->{max}==1) {
	    $record->{Ranges}->[$i]->{min}=$record->{Ranges}->[$j]->{min};
	  } elsif ($record->{Ranges}->[$j]->{min}-$record->{Ranges}->[$i]->{max}==1) {
	    $record->{Ranges}->[$i]->{max}=$record->{Ranges}->[$j]->{max};
	  } else {
	    $j++;
	    next;
	  }
	  splice (@{$record->{Ranges}},$j,1);
	}
      }
      #OK, I know that this algrithm misses some cases, the output is
      #still valid, just that it's not optimal.
      if($error_message=&WriteConfFile()) {
	return $error_message;
      } elsif($error_message=&WriteUIDConfFile()) {
	return $error_message;
      }
    } else { #($total == $record->{MaxUsers})
    $cerr += $E_UNABLEALLOCUIDRANGE and $cerr += {domain => $record->{Domain}};
      return "Couldn't allocate any new UID ranges for $record->{Domain} .\n";
    }
  } elsif ($newnumber<$total) {
    #We need to give back some uids. We get rid of the ranges with
    #most free space first
    if ($error_message=&ParseDomainPasswd($record)) {
      return $error_message;
    }
    map {$_->{endspace}=$_->{max}-($_->{used}?$_->{usedmax}:$_->{min}-1);
	 $_->{frontspace}=($_->{used}?$_->{usedmin}-$_->{min}:0);
	 $_->{freespace}=$_->{frontspace}+$_->{endspace};
       } @{$record->{Ranges}};
    @{$record->{Ranges}} = sort {$b->{freespace} <=> $a->{freespace}}
                                @{$record->{Ranges}};
    my $i=0;
    while ($newnumber<$total) {
      last if ($i > $#{$record->{Ranges}});
      my ($decr)=&MIN($total-$newnumber,$record->{Ranges}->[$i]->{endspace});
      $record->{Ranges}->[$i]->{max}-=$decr;
      $record->{Ranges}->[$i]->{used}-=$decr;
      $total-=$decr;
      if ($record->{Ranges}->[$i]->{max}<$record->{Ranges}->[$i]->{min}){
	splice (@{$record->{Ranges}},$i,1);
	next;
      }
      $decr=&MIN($total-$newnumber,$record->{Ranges}->[$i]->{frontspace});
      $record->{Ranges}->[$i]->{min}+=$decr;
      $record->{Ranges}->[$i]->{used}-=$decr;
      $total-=$decr;
      $i++;
    }
    if ($total<$record->{MaxUsers}){
      $record->{MaxUsers}=$total;
      if($error_message=&WriteConfFile()) {
	return $error_message;
      }elsif($error_message=&WriteUIDConfFile()) {
	return $error_message;
      }elsif ($error_message=&SetLeastUsedRange($record)) {
	return $error_message;
      }
    }else { #$total == $record->{MaxUsers}
    $cerr += $E_UNABLEFREEUPUIDS and $cerr += { what => $record->{Domain} };
      return "Due to existing users, couldn't free up any uids for $record->{Domain} .\n";
    }
  } else {
    #no change in the number of allocated uids
  }
  #If asked to change the number of users we didd some work, but
  #possibly not all of it, so we give some warnings here
  if ($newnumber > $total) {
    $cerr += $E_UIDALLOCERR and $cerr += { howmany => $total } ;
      print STDERR "Could allocate only $total uids.\n";
  } elsif ($newnumber < $total) {
  $cerr += $E_REDUCEUSERCOUNT and $cerr += { howmany => $total };
      print STDERR "Due to existing users could only reduce the maximum user count to $total .\n";
  }
  return "";
}

#
# Sets the range to be used within a virtual domain to the onme with
# the most free uids. It assumes that $record has ->{Ranges}
#
sub SetLeastUsedRange() {
  my($record)=@_;
  @{$record->{Ranges}} = sort 
    {($b->{max}-$b->{min}-$b->{used})<=>($a->{max}-$a->{min}-$a->{used})}
  @{$record->{Ranges}};
  my ($leastused)=$record->{Ranges}->[0];
  return &SetUIDRange("$VHConst::VIRTUALFS/$record->{User}",$leastused->{min},$leastused->{max});
}

#
# Sets the uid range to be used within a virtual domain
#
sub SetUIDRange() {
  my ($vroot,$min,$max)=@_;
  my ($error_message);
  $min+=0;
  $max+=0;
  push @INC, ($ENV{'OCW_SVCPATH'} || "/usr/lib/opcenter")."/cmdline_common";
  &Locking::lock("$vroot/etc/login.defs");
  unless (open(DEFS,"$vroot/etc/login.defs")) {
    &Locking::unlock("$vroot/etc/login.defs");
    $cerr += $E_UNABLEREADFILE and $cerr += { file => "$vroot/etc/login.defs"};
    return "Cannot open $vroot/etc/login.defs .\n";
  }
  my (@defs)=<DEFS>;
  close(DEFS);
  map {$_ =~ s/UID_MIN\s+\d+/UID_MIN\t\t$min/} @defs;
  map {$_ =~ s/UID_MAX\s+\d+/UID_MAX\t\t$max/} @defs;
  unless (open(DEFS,">$vroot/etc/login.defs")) {
    &Locking::unlock("$vroot/etc/login.defs");
    $cerr += $E_UNABLEWRITEFILE and $cerr += {file => "$vroot/etc/login.defs"};
    return "Cannot write to $vroot/etc/login.defs .\n";
  }
  print DEFS @defs;
  close(DEFS);
  &Locking::unlock("$vroot/etc/login.defs");
  return "";
}

sub MAX() {return ($_[0]>$_[1]?$_[0]:$_[1]);}
sub MIN() {return ($_[0]<$_[1]?$_[0]:$_[1]);}

#This function returns the dotted representation of the main address
#of the server (the one used by name based domains)
sub GetServerAddress {
    if ($VHConfiguration::ServerAddress ne "0.0.0.0") {
        return $VHConfiguration::ServerAddress;
    }
    my ($hostname) = Sys::Hostname::hostname();
    my ($name,$aliases,$addrtype,$length,@addrs) = gethostbyname ($hostname);
    ($name,$aliases,$addrtype,$length)=();
    unless ($#addrs >=0) {
    $cerr += $E_UNABLEGETIP;
	print STDERR "FATAL ERROR: Couldn't get the IP of the server.\n";
        exit 1;
    }
    $VHConfiguration::ServerAddress = join (".",(unpack('C4',$addrs[0])));
    if ( $VHConfiguration::ServerAddress eq "127.0.0.1") {
        $cerr += $E_SERVERMISCONF ;
	print STDERR "FATAL ERROR: Server misconfigured, has IP 127.0.0.1 .\n";
        exit 1;
    }
    return $VHConfiguration::ServerAddress;
}


return 1;
