#!/usr/bin/eperl
# 
#   Copyright (c) Ensim Corporation 2000, 2001   All Rights Reserved.
# 
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#
# Exit codes (on failure error message goes to stderr):
#  0 - success
#  1 - failure
# 
# Boolean arguments on the command line are given as 0 or 1. All the
# command line arguments are encoded to avoid problems with escapes.
#
# All functions defined here either return an error message if an
# error occured and "" if everything went well or allways return a
# valid value, but exit (with code 1) printing an error message if an
# error occurs. This second type of functions have _e appended to
# their name. Functions are allowed to print results onto stdout, but
# errors are printed only in the main program.
#

package VHCreateFS;

require VHConst;
require VHOptions;

use lib ($ENV{OCW_SVCPATH} or "/usr/lib/opcenter") . "/virtualhosting";
use lib ($ENV{OCW_SVCPATH} or "/usr/lib/opcenter") . "/cmdline_common";

use ERRORS;
use Carper;

#
# This function returns the list of files and directories that need to
# be added to a virtual domain for a specific service (or the base).
#
sub GetFileList() {
    my($service)=@_;

    if (defined $VHConst::OPTS{$service}) {
	$service = $VHConst::OPTS{$service};
    }

   if($service eq "base") {
	#Directories to be created 
	# Note: It is important that all directories on the path to binaries
	# or libraries that are run as root in the chrooted environent are
	# not writable by anyone but root (of course the same holds for
	# the binaries / libraries too).
	my(@directories) =
	  (["","root","root",0755], #the / of the virtual
	                            #filesystem
	   ["/bin","root","root",0755],
	   ["/dev","root","root",0755],
	   ["/etc","root","$::group",0750],
	   ["/etc/cron.daily","root","root",0755],
	   ["/etc/cron.monthly","root","root",0755],
	   ["/etc/cron.weekly","root","root",0755],
	   ["/etc/default","root","root",0755],
	   ["/etc/logrotate.d","root","root",0755],
	   ["/etc/mail","root","root",0755],
	   ["/etc/pam.d","root","root",0755],
	   ["/etc/skel","$::user","$::group",0755],
	   ["/etc/skel/public_html","$::user","$::group",0711],
	   ["/home","$::user","$::group",0751],
	   ["/home/$::user","$::user","$::group",0700],
	   ["/lib","root","root",0755],
	   ["/lib/security","root","root",0755],
	   ["/sbin","root","root",0755],
	   ["/tmp","$::user","$::group",01777],
	   ["/usr","root","root",0755],
	   ["/usr/bin","root","root",0755],
	   ["/usr/lib","root","root",0755],
	   ["/usr/lib/opcenter","root","root",0755],
	   ["/usr/lib/opcenter/backup","root","root",0755],
	   ["/usr/lib/opcenter/cmdline_common","root","root",0755],
	   ["/usr/lib/perl5","root","root",0755],
	   ["/usr/lib/python2.1","root","root",0755],
	   ["/usr/local","root","root",0755],
	   ["/usr/man","root","root",0755],
	   ["/usr/man/man1","root","root",0755],
	   ["/usr/sbin","root","root",0755],
	   ["/usr/share","root","root",0755],
           ["/usr/share/ssl","root","root",0755],
	   ["/usr/share/man","root","root",0755],
	   ["/usr/share/man/man1","root","root",0755],
	   ["/usr/share/groff","root","root",0755],
	   ["/usr/share/groff/font","root","root",0755],
	   ["/usr/share/groff/font/devascii","root","root",0755],
	   ["/usr/share/groff/font/devlatin1","root","root",0755],
	   ["/usr/share/groff/tmac","root","root",0755],
	   ["/usr/share/groff/tmac/mdoc","root","root",0755],
	   ["/usr/lib/locale","root","root",0755],
	   ["/usr/lib/locale/en_US","root","root",0755],
	   ["/usr/lib/locale/en_US/LC_MESSAGES","root","root",0755],
	   ["/usr/share/opcenter","root","root",0755],
	   ["/usr/share/opcenter/responders","root","root",0755],
	   ["/usr/share/terminfo","root","root",0755],
	   ["/usr/share/zoneinfo","root","root",0755],
	   ["/var","$::user","$::group",0755],
	   ["/var/lib","root","root",0755],
	   ["/var/log","$::user","$::group",0755],
	   ["/var/spool","$::user","$::group",0755],
	   ["/var/spool/mail","$::user","$::group",0770],
	   ["/var/spool/mqueue","root","$::group",0700],
	   ["/var/run","$::user","$::group",0755],
	   ["/var/tmp","$::user","$::group",01777],
	  );
	#Files to be added (hardlinked)
	my(@version_glibc,@version_pam,@version_pwdb,@version_libtermcap,@version_cracklib,@version_glib,@version_ncurses,@version_libstdc,@version_openssl);

	if ($::user eq "root") {
		# This will retrieve the version of glibc installed on the system
		@glibc=split /-/, `rpm -q glibc`;
		$version_glibc=$glibc[1];
		# This will retrieve the version of pam installed on the system
		@pam=split /-/, `rpm -q pam`;
		$version_pam=$pam[1];

		# retrieve pwdb version
		@pwdb=split /-/, `rpm -q pwdb`;
		$version_pwdb=$pwdb[1];

		# retrieve libtermcap version
		@libtermcap=split /-/, `rpm -q libtermcap`;
		$version_libtermcap=$libtermcap[1];

		#retrieve cracklib version
		@cracklib=split /-/, `rpm -q cracklib`;
		$version_cracklib=$cracklib[1];

		#retrieve openssl version
		@openssl=split /-/, `rpm -q openssl`;
		$version_openssl=$openssl[1];
	}else{
		# This will retrieve the version of glibc installed on the system
		@glibc=split /([-.])/, `find /home/virtual/FILESYSTEMTEMPLATE/lib -maxdepth 1 |grep "libc-"`;
		$version_glibc="$glibc[2].$glibc[4].$glibc[6]";

		# This will retrieve the version of pam installed on the system
		@pam=split /([-.])/, `find /home/virtual/FILESYSTEMTEMPLATE/lib -maxdepth 1 |grep "libpam.so."`;
		$version_pam="$pam[4].$pam[6]";
		chomp($version_pam);
		# retrieve pwdb version
		@pwdb=split /([.])/, `find /home/virtual/FILESYSTEMTEMPLATE/lib -maxdepth 1 |grep "libpwdb.so."`;
		$version_pwdb="$pwdb[4].$pwdb[6]";
		chomp($version_pwdb);
		# retrieve libtermcap version
		@libtermcap=split /([.])/, `find /home/virtual/FILESYSTEMTEMPLATE/lib -maxdepth 1 |grep "libtermcap.so."`;
		$version_libtermcap="$libtermcap[4].$libtermcap[6].$libtermcap[8]";
		chomp($version_libtermcap);
		#retrieve libstdc++ version
		@cracklib=split /([.])/, `find /home/virtual/FILESYSTEMTEMPLATE/usr/lib -maxdepth 1 |grep "libcrack.so."`;
		$version_cracklib="$cracklib[4].$cracklib[6]";
		chomp($version_cracklib);
		#retrieve openssl version
		@openssl=split /([.])/, `find /home/virtual/FILESYSTEMTEMPLATE/usr/lib -maxdepth 1 |grep "libcrypto.so.0"`;
		$version_openssl="$openssl[4].$openssl[6].$openssl[8]";
		chomp($version_openssl);
	}
	#replced all the hardcoding of the version numbers with 
	#the versions retrieved above

	my(%hardlinks) = 
	  ("/bin" => ["awk","bash","cat","chmod","chown","cp","date","egrep","false","ftponly","gawk","grep","gzip","gunzip","login","ln","ls","mkdir","more","mv","ping","rm","sed","sh","su","tar","touch","true","vi","zcat"],
	   "/dev" => ["log","null","ptmx",],
	   "/etc" => ["issue.net","localtime","man.config","ocwinodes","profile","protocols","resolv.conf","services"],
	   "/etc/pam.d" => ["login","other","su","system-auth"],
	   "/lib" => ["ld-$version_glibc.so","libc-$version_glibc.so","libcrypt-$version_glibc.so","libdl-$version_glibc.so","libm-$version_glibc.so","libNoVersion-$version_glibc.so","libnsl-$version_glibc.so","libnss_dns-$version_glibc.so","libnss_files-$version_glibc.so","libpam.so.$version_pam","libpam_misc.so.$version_pam","libpthread-0.9.so","libpwdb.so.0.61.1","libresolv-$version_glibc.so","libtermcap.so.$version_libtermcap","libutil-$version_glibc.so",],
	   "/lib/security" => ["pam_console.so","pam_cracklib.so","pam_deny.so","pam_listfile.so","pam_nologin.so","pam_pwdb.so","pam_securetty.so","pam_unix_acct.so","pam_unix_auth.so","pam_unix_passwd.so","pam_unix_session.so", "pam_stack.so", "pam_limits.so", "pam_unix.so", "pam_env.so"],
	   "/sbin" => ["ldconfig"],
	   "/usr/bin" => ["ftp","ncftp","ncftpget","ncftpput", "compress","exclude_templates","find","groff","grotty","gtbl","gzip","head","id","less","man","passwd","perl","python2.1","run-parts","tail","tbl","troff","uncompress","whois","fwhois","md5sum", "split", "wc"],
	   "/usr/lib" => ["libdb1.so.2","libdb2.so.3","cracklib_dict.hwm","cracklib_dict.pwd","cracklib_dict.pwi","libcrack.so.$version_cracklib","libgdbm.so.2.0.0","libglib-1.2.so.0.0.9","libncurses.so.5.2","libreadline.so.4.1","libstdc++-3-libc6.2-2-2.10.0.so","virtualname.so","libcrypto.so.$version_openssl","libssl.so.$version_openssl", "libpopt.so.0.0.0"],
	   "/usr/lib/opcenter/backup" => ["backup","restore","schedule_backup"],
	   "/usr/lib/opcenter/cmdline_common" => ["CmdLineCoder.pm","Locking.pm"],
	   "/usr/lib/perl5" => [], #everything is linked over
	   "/usr/lib/python2.1" => [],#everything is linked over
           "/usr/share/ssl" => ["openssl.cnf"],
	   "/usr/share/man/man1" => ["bash.1.gz","cat.1.gz","chmod.1.gz","compress.1.gz","cp.1.gz","date.1.gz","find.1.gz","gawk.1.gz","grep.1.gz","gunzip.1.gz","gzip.1.gz","head.1.gz","id.1.gz","less.1.gz","ln.1.gz","ls.1.gz","man.1.gz","mkdir.1.gz","more.1.gz","passwd.1.gz","perl.1.gz","rm.1.gz","sed.1.gz","vi.1.gz","vim.1.gz","tar.1.gz","tail.1.gz","uncompress.1.gz","zcat.1.gz",],
	   "/usr/sbin" => ["chpasswd","useradd","userdel","traceroute"],
	   "/usr/lib/locale/en_US" => ["LC_COLLATE","LC_CTYPE","LC_MONETARY","LC_NUMERIC","LC_TIME","LC_ADDRESS","LC_IDENTIFICATION","LC_MEASUREMENT","LC_NAME","LC_PAPER","LC_TELEPHONE"],
	   "/usr/lib/locale/en_US/LC_MESSAGES" => ["SYS_LC_MESSAGES"],
	   "/usr/share/terminfo" => [], #everything is linked over
	   "/usr/share/zoneinfo" => [], #everything is linked over
	  );

	if ($::user ne "root") {
	  $hardlinks{"/etc"} =  ["issue.net","localtime","man.config","ocwinodes","profile","protocols","resolv.conf","services", ".wpversion", ".wpkey"],
	}

	my(@version_kernel);
	@version_kernel=split /-/, `uname -r`;
	if ($version_kernel[0] eq "2.2.19" or $version_kernel[0] eq "2.4.2") {
	     # $hardlinks{"/usr/man/man1"} = [],
	     $hardlinks{"/usr/share/man/man1"} = ["bash.1.gz","cat.1.gz","chmod.1.gz","compress.1.gz","cp.1.gz","date.1.gz","find.1.gz","gawk.1.gz","grep.1.gz","gunzip.1.gz","gzip.1.gz","head.1.gz","id.1.gz","less.1.gz","ln.1.gz","ls.1.gz","man.1.gz","mkdir.1.gz","more.1.gz","passwd.1.gz","perl.1.gz","rm.1.gz","sed.1.gz","vi.1.gz","vim.1.gz","tar.1.gz","tail.1.gz","uncompress.1.gz","zcat.1.gz",],
	     $hardlinks{"/usr/share/groff/font/devascii"} = ["B","BI","DESC","I","R"],
	     $hardlinks{"/usr/share/groff/font/devlatin1"} = ["B","BI","DESC","I","R"],
	     $hardlinks{"/usr/share/groff/tmac"} = ["hyphen.us","man.local","tmac.an","tmac.andoc","tmac.doc","tmac.safer","tmac.tty","troffrc",],
	     $hardlinks{"/usr/share/groff/tmac/mdoc"} = ["doc-common","doc-ditroff","doc-nroff","doc-syms",],
	
	}
	if ($::user eq "root"){
	  $hardlinks{"/bin"} = ["awk","bash","cat","chmod","chown","cp","date","egrep","false","ftponly","gawk","grep","gzip","gunzip","login","ln","ls","mkdir","more","mv","ping","rm","sed","sh","su","tar","touch","true","vi","zcat"],
	}
	#Files to be created anew (their contents are in $VHConst::TEMPLATEDIR)
	my(@newfiles) = 
	  (["/bin/dnsdomainname","root","root",0755,"dnsdomainname"],
	   ["/bin/hostname","root","root",0755,"hostname"],
	   ["/etc/HOSTNAME","root","root",0644,""],
	   ["/etc/aliases","root","root",0644,"aliases"],
	   ["/etc/domainname","root","root",0644,""],
	   ["/etc/group","root","root",0644,""],
	   ["/etc/ld.so.conf","root","root",0644,""],
	   ["/etc/ld.so.preload","root","root",0644,"ld.so.preload"],
	   ["/etc/login.defs","root","root",0644,"login.defs"],
	   ["/etc/nsswitch.conf","root","root",0644,"nsswitch.conf"],
	   ["/etc/pam.d/passwd","root","root",0644,"passwd"],
	   ["/etc/passwd","root","root",0644,""],
	   ["/etc/pwdb.conf","root","root",0644,"pwdb.conf"],
	   ["/etc/shadow","root","root",0400,""],
	   ["/etc/shells","root","root",0644,"shells"],
	   ["/etc/skel/.bash_logout","$::user","$::group",0644,""],
           ["/etc/skel/.bash_profile","$::user","$::group",0644,""],
           ["/etc/skel/.bashrc","$::user","$::group",0644,""],
	   ["/etc/skel/public_html/index.html","$::user","$::group",0755,""],
	   ["/etc/default/useradd","root","root",0644,"useradd"],
	   ["/etc/mail/access","root","root", 0644,"access"],
	  );

	my(@nontmplfiles);
	if ($::user eq "root"){
	  @nontmplfiles = ();
	} else {
	  @nontmplfiles = 
	    (["/etc/hosts","root","root",0644,],);
	}
	return (\@directories,\%hardlinks,\@newfiles,\@nontmplfiles);
    }
 elsif ($service eq "Web") {
	push @INC, ($ENV{OCW_SVCPATH} || "/usr/lib/opcenter")."/apache";
	require VHCreateFSapache;
	return (&VHCreateFSapache::getFileList());
    } elsif ($service eq "Analog") {
	push @INC,($ENV{OCW_SVCPATH} || "/usr/lib/opcenter")."/analog";
	require VHCreateFSanalog;
	return (&VHCreateFSanalog::getFileList());
    } elsif ($service eq "SSL") {
	push @INC,($ENV{OCW_SVCPATH} || "/usr/lib/opcenter")."/openssl";
	require VHCreateFSopenssl;
	return (&VHCreateFSopenssl::getFileList());
    } elsif ($service eq "IMAP") {
	push @INC,($ENV{OCW_SVCPATH} || "/usr/lib/opcenter")."/imap";
	require VHCreateFSimap;
	return (&VHCreateFSimap::getFileList());
    } elsif ($service eq "Vacation") {
	push @INC,($ENV{OCW_SVCPATH} || "/usr/lib/opcenter")."/vacation";
	require VHCreateFSvacation;
	return (&VHCreateFSvacation::getFileList());
    } elsif ($service eq "Majordomo") {
	push @INC,($ENV{OCW_SVCPATH} || "/usr/lib/opcenter")."/majordomo";
	require VHCreateFSmajordomo;
	return (&VHCreateFSmajordomo::getFileList());
    } elsif ($service eq "Sendmail") {
	push @INC,($ENV{OCW_SVCPATH} || "/usr/lib/opcenter")."/sendmail";
	require VHCreateFSsendmail;
	return (&VHCreateFSsendmail::getFileList());
    } elsif ($service eq "Telnet") {
	push @INC,($ENV{OCW_SVCPATH} || "/usr/lib/opcenter")."/telnet";
	require VHCreateFStelnet;
	return (&VHCreateFStelnet::getFileList());
    } elsif ($service eq "FTP") {
	push @INC,($ENV{OCW_SVCPATH} || "/usr/lib/opcenter")."/proftpd";
	require VHCreateFSproftpd;
	return (&VHCreateFSproftpd::getFileList());
    } elsif ($service eq "AnonFtp") {
	push @INC,($ENV{OCW_SVCPATH} || "/usr/lib/opcenter")."/proftpd";
	require VHCreateFSanon;
	return (&VHCreateFSanon::getFileList());
     } elsif ($service eq "WebLogs") {
	my(@directories)=(["/var/log/httpd","$::user","$::group",0755],
			 );
	my(%hardlinks)=();
	my(@newfiles)=(["/etc/logrotate.d/apache","root","root",0644,""],
		      );
        my(@nontmplfiles)=();
	return (\@directories,\%hardlinks,\@newfiles,\@nontmplfiles);
     } elsif ($service eq "MivaMerchant") {
	push @INC,($ENV{OCW_SVCPATH} || "/usr/lib/opcenter")."/base/services/mivamerchant";
	eval "require VHCreateFSmiva";
        if ($@) {
            $cerr += $E_NOMIVABACKEND ;
            print STDERR "miva merchant backend not available\n";
            return "";
        }
	return (&VHCreateFSmiva::getFileList());
     } elsif ($service eq "Nslookup") {
	push @INC,($ENV{OCW_SVCPATH} || "/usr/lib/opcenter")."/bind";
	require VHCreateFSbind;
	return (&VHCreateFSbind::getFileList());
    }
    $cerr += $E_OPTION and $cerr += { option => $service } ;
    print STDERR "Internal error: unsupported option $service .\n";
    exit 1;
}

#
# This function creates the filesystem to be used by a user with
# virtual hosting. This filesystem is used by the chrooted
# applications (e.g. pop3d) and by the normal ones
# (e.g. apache). Returns an error message that is empty on success.
#
sub CreateFileSystem() {
  my ($record,$passwd) = @_;
  local($::user,$::group);
  my ($domain,$telnet,$webserver,$minuid,$maxuid);
  $::user = $record->{User};
  $domain = $record->{Domain};
  $telnet = $record->{Telnet};
  $webserver = $record->{WebServerName};
  $::group = $::user;
  if (@{$record->{Ranges}}) {
      $minuid = "${$record->{Ranges}}[0]->{min}";
      $maxuid = "${$record->{Ranges}}[0]->{max}";
  } else {
      $minuid = "0";
      $maxuid = "0";
  }
  # $record is not used below this line
  my ($error_message);
  #The common files
  if ($error_message = &VHCreateFS::ActualCreateFilesystem(&VHCreateFS::GetFileList("base"))) { 
      return $error_message;
  }

  my ($prefix) ="$VHConst::VIRTUALFS/$::user";
  #Updating some files with site specific information

  {
      open (PASSWD,">$prefix/etc/passwd")
	or return "Could not open $prefix/etc/passwd for writing.\n";
      print PASSWD "root:x:0:0:root:/root:/dev/null\n";
      open (FROM,"/etc/passwd")
	or $cerr += $E_UNABLEREADFILE and $cerr += { file => "/etc/passwd" } 
    and return "Could not open /etc/passwd for reading.\n";
      my (@passwd) = <FROM>;
      close (FROM);
      my ($passwdline) = grep {/^$::user:/} @passwd;
      my (@passwdfields) = split (/:/,$passwdline);


      if ( $passwdfields[6] =~ "/bin/virtshell*" ) { $passwdfields[6] = "/bin/bash\n"}
      print PASSWD join(":",@passwdfields);
      $passwdfields[0] = "mail";
      $passwdfields[1] = "x";
      $passwdfields[4] = "mail";
      $passwdfields[5] = "/var/spool/mail";
      $passwdfields[6] = "/dev/null\n";
      print PASSWD join(":",@passwdfields);
      close (PASSWD);

      my ($inbox)="$prefix/var/spool/mail/$::user";
      open (INBOX,">$inbox")
        or $cerr += $E_UNABLECREATEMBOX and $cerr += { mbox => $inbox } 
        and return "Cannot create mail box $inbox \n";
      close (INBOX);
      chown($passwdfields[2],$passwdfields[3], $inbox) or
      $cerr += $E_UNABLECHOWNMBOX and $cerr += { mbox => $inbox } and
	return "Cannot change ownership of mail box $inbox \n";
      unless (chmod(0600,$inbox)) {
      $cerr += $E_UNABLECHMODMBOX and $cerr += { mbox => $inbox } ;
	  return "Cannot modify permissions of mail box $inbox \n";
      }
  }

  {
      open (HOSTSFILE, ">>$prefix/etc/hosts")
        or $cerr += $E_UNABLEWRITEFILE and $cerr += {file=>"$prefix/etc/hosts"} 
        and return "Could not open $prefix/etc/hosts for writing.\n";
      print HOSTSFILE "$record->{IP} $record->{Domain} # webppliance\n";
      close (HOSTSFILE);
  }

  {
      open (SHADOW,">$prefix/etc/shadow")
	or $cerr += $E_UNABLEWRITEFILE and $cerr += {file => "$prefix/etc/shadow" }
    and return "Could not open $prefix/etc/shadow for writing.\n";
      open (FROM,"/etc/shadow")
	or $cerr += $E_UNABLEREADFILE and $cerr += {file => "/etc/shadow" }
    and return "Could not open /etc/shadow for reading.\n";
      my (@shadow) = <FROM>;
      close (FROM);
      my ($shadowline) = grep {/^$::user:/} @shadow;
#      my (@shadowfields) = split (/:/,$shadowline);
#      $shadowfields[0] = "root";
#      $shadowfields[1] = "*";
#      print SHADOW join(":",@shadowfields); #root
      print SHADOW $shadowline; #site owner
#      $shadowfields[0] = "mail"; #mail user
#      print SHADOW join(":",@shadowfields);
      close (SHADOW);
  }
  {
      open (GRP,">$prefix/etc/group")
	or $cerr += $E_UNABLEWRITEFILE and $cerr += { file => "$prefix/etc/group" }
    and return "Could not open $prefix/etc/group for writing.\n";
      open (FROM,"/etc/group")
	or $cerr += $E_UNABLEREADFILE and $cerr += { file => "/etc/group" }
    and return "Could not open /etc/group for reading.\n";
      while (<FROM>) {
	  if ($_ =~ /^root:/ or $_ =~ /^$::group:/ ) {
	      print GRP $_;
	      if (/^$::group:/) {
		  $_ =~ s/^$::group(.+)$/mail$1/;
		  print GRP $_;
	      }
	  }
      }
      close (FROM);
      close (GRP);
  } 
  {
      open (ALIAS,">>$prefix/etc/aliases")
	or $cerr += $E_UNABLEAPPENDFILE and $cerr += {file =>"$prefix/etc/aliases"}
    and return "Could not open $prefix/etc/aliases for appending.\n";
      print ALIAS "\nroot:\t$::user\n";
      print ALIAS "catch-all:\t$::user\n";
      close (ALIAS);
  }
  {   
      open (USRADD,">>$prefix/etc/default/useradd")
	or $cerr += $E_UNABLEAPPENDFILE 
    and $cerr += {file => "$prefix/etc/default/useradd"}
    and return "Could not open $prefix/etc/default/useradd for appending.\n";
      #add the GROUP line to the useradd default file
      my(@gwent) = getgrnam($::group);
      print USRADD "GROUP=$gwent[2]\n";
      close (USRADD);
  }
  {
      open (DOM,">$prefix/etc/domainname")
	or $cerr += $E_UNABLEWRITEFILE
    and $cerr += { file => "$prefix/etc/domainname" }
    and return "Could not open $prefix/etc/domainname for writing.\n";
      #append the domain
      print DOM "$domain\n";
      close (DOM);
  }
  {
      open (HOST,">$prefix/etc/HOSTNAME")
	or $cerr += $E_UNABLEWRITEFILE
    and $cerr += { file => "$prefix/etc/HOSTNAME" }
    and return "Could not open $prefix/etc/HOSTNAME for writing.\n";
      #append the domain
      print HOST "$domain\n";
      close (HOST);
  }
  {
      open (DEFS,">>$prefix/etc/login.defs")
	or $cerr += $E_UNABLEAPPENDFILE
    and $cerr += { file => "$prefix/etc/login.defs" }
    and return "Could not open $prefix/etc/login.defs for appending.\n";
      #append the UID ranges
      print DEFS "UID_MIN\t\t$minuid\nUID_MAX\t\t$maxuid\n";
      close (DEFS);
  }
  {
      open (USRINDEX,">>$prefix/etc/skel/public_html/index.html")
	or $cerr += $E_UNABLEAPPENDFILE
    and $cerr += { file => "$prefix/etc/skel/public_html/index.html" }
    and return "Could not open $prefix/etc/skel/public_html/index.html for appending.\n";
      print USRINDEX "<A HREF=http://$webserver>$webserver</A>";
      close (USRINDEX);
  }

  #setting up the dynamic libraries
  system ("cd $prefix; /usr/sbin/chroot . /sbin/ldconfig >/dev/null 2>/dev/null");

  #Symlinks from the user's home to the virtual fs (for convenience)
  my(%symlinks) = ("webdocs" => "/home/httpd/html",
		   "weblogs" => "/var/log/httpd",
		   "webscripts" => "/home/httpd/cgi-bin",
		   "homes" => "/home",
		   "mailspool" => "/var/spool/mail",
		   "etc" => "/etc",
                   "siteroot" => "",
		  );


  # Create the symlinks
  my(@pwent) = getpwnam($::user);
  unless (@pwent) {
    $cerr += $E_INVALIDUSER and $cerr += { user => $::user } and
    return "Invalid user name $::user .\n" ;}
  my($homedir) = $pwent[7];
  foreach $dir (keys %symlinks) {
    symlink("$prefix$symlinks{$dir}","$homedir/$dir");
    }

  #Installing the log rotation file
  open (LOGROT,">/etc/logrotate.d/0virtualhosting.$::user") or 
    $cerr += $E_UNABLECREATELOGROT and 
    $cerr += { log => "/etc/logrotate.d/0virtualhosting.$::user" } and
    return "Could not create log rotation file /etc/logrotate.d/0virtualhosting.$::user\n";
  print LOGROT <<"EOF";
include $VHConst::VIRTUALFS/$::user/etc/logrotate.d
EOF
  close (LOGROT);
  open (CRONJOB, ">/etc/cron.daily/0virtualhosting.$::user") or 
  $cerr += $E_UNABLECREATECRON and 
  $cerr += { cron => "/etc/cron.daily/0virtualhosting.$::user" } and
    return "Could not create cron file /etc/cron.daily/0virtualhosting.$::user\n";
  print CRONJOB <<"EOF";
#!/bin/sh
/usr/sbin/chroot $VHConst::VIRTUALFS/$::user /usr/bin/run-parts /etc/cron.daily
EOF
  close (CRONJOB);
  chmod (0755,"/etc/cron.daily/0virtualhosting.$::user") or
    return "Error with cron file /etc/cron.daily/0virtualhosting.$::user\n";
  open (CRONJOB, ">/etc/cron.weekly/0virtualhosting.$::user") or 
    return "Could not create cron file /etc/cron.weekly/0virtualhosting.$::user\n";
  print CRONJOB <<"EOF";
#!/bin/sh
/usr/sbin/chroot /home/virtual/$::user /usr/bin/run-parts /etc/cron.weekly
EOF
  close (CRONJOB);
  chmod (0755,"/etc/cron.weekly/0virtualhosting.$::user") or
    return "Error with cron file /etc/cron.weekly/0virtualhosting.$::user\n";
  open (CRONJOB, ">/etc/cron.monthly/0virtualhosting.$::user") or 
    return "Could not create cron file /etc/cron.monthly/0virtualhosting.$::user\n";
  print CRONJOB <<"EOF";
#!/bin/sh
/usr/sbin/chroot /home/virtual/$::user /usr/bin/run-parts /etc/cron.monthly
EOF
  close (CRONJOB);
  chmod (0755,"/etc/cron.monthly/0virtualhosting.$::user") or
    $cerr += $E_CRONFILEERR and 
    $cerr += { cron => "/etc/cron.monthly/0virtualhosting.$::user" } and
    return "Error with cron file /etc/cron.monthly/0virtualhosting.$::user\n";

  return "";
}

# Installs and runs private server owner's customizations
sub RunCustomizations() {
  my ($record,$passwd) = @_;
  local($::user);
  my ($domain);
  $::user = $record->{User};
  $domain = $record->{Domain};
  my ($prefix) ="$VHConst::VIRTUALFS/$::user";

  #Installing the customizations
  if (-f "/usr/local/customized/virtDomain.tar"){
      system ("cd $prefix; /bin/tar xf /usr/local/customized/virtDomain.tar >/dev/null 2>/dev/null");
      my($exit_value)  = $? >> 8;
      if ($exit_value ne 0) {
      $cerr += $E_CUSTOMPROB ;
	  return "Problems occured while installing the customizations.\n";
      }
  }
  if (-x "/usr/local/customized/virtDomain.sh"){
      open (CUST,"|cd $VHConst::VIRTUALFS/$::user; /usr/local/customized/virtDomain.sh $domain $::user $record->{IP} >dev/null");
      print CUST "$passwd\n";
      close (CUST);
   }  
}

#
# All this function does is to create a file system (by copying if we
# work on the initial file system template and by linking most of the
# files if it is a virtual domain)
#
sub ActualCreateFilesystem() {

    #please ensure that you do error checking in appropriate places here. 

    my ($pdirs,$pfiles,$pnfiles,$pntfiles)=@_;
    my ($prefix)=($::user eq "root" ? "$VHConst::VIRTUALFS/FILESYSTEMTEMPLATE"
		                    : "$VHConst::VIRTUALFS/$::user");
    my ($srcprefix)="$VHConst::VIRTUALFS/FILESYSTEMTEMPLATE";
    my ($error_message);

    #Handle the directories
    my ($dir);
    foreach $dir (@{$pdirs}) {
	unless ((-d "$prefix$dir->[0]") or (mkdir ("$prefix$dir->[0]",0755))) {
      $cerr += $E_MKDIR and 
      $cerr += { dir => "$prefix$dir->[0]", perror => $! } ;
	  return "Could not create directory $prefix$dir->[0]: $! .\n";
	  }
    }

    #Handle normal files
    foreach $dir (keys %{$pfiles}) {
	my ($files) = $pfiles->{$dir};
	my ($file);
	foreach $file (@$files) {
	    if ($::user eq "root") { #we are copying over the file
		if (-l "$dir/$file") {
		    my ($target) = readlink("$dir/$file");
		    my ($firstchar) = substr $target, 0, 1;
		    if ($firstchar eq "/") {
			# absolute, so just copy
			#"[abs] $target --> $prefix$dir/$file .\n";
			if (system("/bin/cp -p $target $prefix$dir/$file") && 0xff00) {
                $cerr += $E_CPFILES and
                $cerr += { file1 => $target, file2 => "$prefix$dir/$file" };
			    return "Could not copy file $target to $prefix$dir/$file .\n";
			}
		    } else {
			# relative
			#"[rel] $dir/$target --> $prefix$dir/$file .\n";
			if (system("/bin/cp -p $dir/$target $prefix$dir/$file") && 0xff00) {
                $cerr += $E_CPFILES and
                $cerr += {file1 =>"$dir/$target",file2 => "$prefix$dir/$file"};
			    return "Could not copy file $dir/$target to $prefix$dir/$file .\n";
			}
		    }
		} elsif (-f "$dir/$file") {
		  if (system ("/bin/cp -p $dir/$file $prefix$dir") && 0xff00){
                $cerr += $E_CPFILES and
                $cerr += {file1 =>"$dir/$file",file2 => "$prefix$dir"};
		    return "Could not copy file $dir/$file to $prefix$dir .\n";
		  }
		} elsif (-S "$dir/$file") {
		    #syslogd will take care of this for us (it's /dev/log)
		} elsif (-c "$dir/$file") {
		    my (@stat) = stat("$dir/$file");
		    my ($mode,$uid,$gid,$rdev) = ($stat[2],@stat[4..6]);
		    my ($minor) = $rdev%256;
		    my ($major) = $rdev>>8;
		    $mode = sprintf "%lo",$mode%512;
		    unless (-e "$prefix$dir/$file") {
		      if (system ("/bin/mknod -m 0$mode $prefix$dir/$file c $major $minor") && 0xff00){
              $cerr += $E_CREATFILE and $cerr += {file =>"$prefix$dir/$file"};
			return "Could not create file $prefix$dir/$file \n";
		      }		      
		    }
		    chown ($uid,$gid,"$prefix$dir/$file");
		} else {
            $cerr += $E_UNABLEHANDLEFILE and 
            $cerr += { file => "$dir/$file" };
		    print STDERR "I cannot handle $dir/$file !\n";
		}
	    } else { #we link the file over
		my ($src) = "$srcprefix$dir/$file";
		my ($dest) = "$prefix$dir/$file";
		if (-c "$src") {
		    # linking devices does not work for character devices on 
		    # nfs mounted stuff, so let's mknod
		    my (@stat) = stat("$src");
		    my ($mode,$uid,$gid,$rdev) = ($stat[2],@stat[4..6]);
		    my ($minor) = $rdev%256;
		    my ($major) = $rdev>>8;
		    $mode = sprintf "%lo",$mode%512;
		    unless ( -e "$dest") {
		      if (system ("/bin/mknod -m 0$mode $dest c $major $minor") && 0xff00) {
            $cerr += $E_CREATFILE and $cerr += { file => "$prefix$dir/$file"};
			return "Could not create file $prefix$dir/$file \n";
		      }
		    }
		    chown ($uid,$gid,"$dest");
		} elsif (-S "$dir/$file") {
                    link("$src","$dest"); #This might fail on NFS so
                                          #we don't fail if it does
                } else {
		    link("$src","$dest") or
                $cerr += $E_HARDLINK and
                $cerr += { file => "$dir/$file", perror => $! } and
		      return "Could not create hard link for $dir/$file: $! .\n";
		}
	    }
	}
	if ($#{$files} == -1 ) {
	    #We handle recursively each file in this directory
	    if ($::user eq "root" ) {
		# we want to follow links, as we want the TEMPLATEFS
		# to be self-sufficient...
	      if(system ("cd $dir; /bin/cp -pRu * $prefix$dir") && 0xff00){
          $cerr += $E_RCPFILES and 
          $cerr += { file1 => $dir, file2 => "$prefix$dir" };
		return "Could not do recursive copy from $dir to $prefix$dir \n";
	      }
	    } else {
		if($error_message=&RecursiveLink($dir,"$prefix","$srcprefix")){
		    return $error_message;
		}
	    }
	}
    }

    my($cpsrc);
    if ($::user eq "root") {
        $cpsrc = "";
        $cpdest = $srcprefix;
    } else {
        $cpsrc = $srcprefix;
        $cpdest = $prefix;
    }
    my($file);
    foreach $filestruct (@{$pntfiles}) {
        $file = $filestruct->[0];
        if (system ("/bin/cp -p $cpsrc$file $cpdest$file") && 0xff00) {
        $cerr += $E_CPFILES and 
        $cerr += { file1 => "$cpsrc$file", file2 => "$cpdest$file" } ;
	  return "Could not copy $cpsrc$file to $cpdest$file\n";
	}
    }

    if ($::user eq "root") { #we have no more work to do
	return "";
    }

    #Handle new files
    #first read files from templates directory
    my($filestruct);
    my($content);
    foreach $filestruct (@{$pnfiles}) {
	if ($filestruct->[4]) {
	    open (FILE,"$VHConst::TEMPLATEDIR/$filestruct->[4]") or 
        $cerr += $E_UNABLEREAD and 
        $cerr += { file => "$VHConst::TEMPLATEDIR/$filestruct->[4]" } and
	      return "Could not read $VHConst::TEMPLATEDIR/$filestruct->[4] .\n";
	    $content = "";
	    while (<FILE>) {$content .= $_}
	    close (FILE);
	    $filestruct->[4] = $content;
	}
    }
    #and actually create the files
    foreach $file (@{$pnfiles}) {
	open (NEWFILE,">$prefix$file->[0]") or
    $cerr += $E_UNABLECREATEFILE and 
    $cerr += { file => "$prefix$file->[0]", perror => $! } and
	  return "Could not create the file $prefix$file->[0]: $! .\n";
	print NEWFILE $file->[4];
	close (NEWFILE);
    }

    #Change ownership on directories and new files
    my (@dirsandfiles)=(@{$pdirs},@{$pnfiles},@{$pntfiles});
    #first fill up the uid/gid cache
    local (%::uids) = ();
    local (%::gids) = ();
    my (@userlist) = map {$_->[1]} (@dirsandfiles);
    my (@grouplist) =  map {$_->[2]} (@dirsandfiles);
    my ($crt);
    foreach $crt (@userlist) {
	unless (defined $::uids{$crt}) {
	    my (@pwent) = getpwnam($crt);
	    unless (@pwent) {
            $cerr += $E_INVALIDUSER and $cerr += { user => $crt } ;
            return "Invalid user name $crt .\n" ;}
	    $::uids{$crt} = $pwent[2];
	}
    }
    foreach $crt (@grouplist) {
	unless (defined $::gids{$crt}) {
	    my (@grent) = getgrnam($crt);
	    unless (@grent) {
            $cerr += $E_INVALIDGROUP and $cerr += { grp => $crt } and
            return "Invalid group name $crt .\n" ;}
	    $::gids{$crt} = $grent[2];
	}
    }
    #and do the work
    foreach $file (@dirsandfiles) {
	if ($error_message = &ChownChmod($file)) {return $error_message}
    }

    return "";
}


#
# This function chowns and chmods the file given as argument. It
# expects a list reference like the one used by CreateFileSystem. The
# translations from user and group names to ids have to be precomputed.
#
sub ChownChmod(){
  my ($file)=@_;
  chown ($::uids{$file->[1]},$::gids{$file->[2]},"$VHConst::VIRTUALFS/$::user$file->[0]") or
  $cerr += $E_UNABLECHOWNFILE and
  $cerr += { file => "$VHConst::VIRTUALFS/$::user$file->[0]", perror => $! } and
    return "Could not change ownership of $VHConst::VIRTUALFS/$::user$file->[0] to $file->[1] $file->[2]: $! .\n";
  chmod ($file->[3],"$VHConst::VIRTUALFS/$::user$file->[0]") or 
  $cerr += $E_UNABLECHMODFILE and 
  $cerr += { file => "$VHConst::VIRTUALFS/$::user$file->[0]", perror => $! } and
    return "Could not modify permissions of $VHConst::VIRTUALFS/$::user$file->[0]: $! .\n";
  return "";
}

#
# This function links all files from $src and all its subdirectories
# into $dest
#
sub RecursiveLink() {
    my($src,$dest,$prefix)=@_;
    if (not defined $prefix) {$prefix=""}
    if (opendir(SRC,"$prefix$src")) { 
	my(@filenames) = map {"$src/$_"} readdir(SRC);
	@filenames = grep {$_ ne "$src/." and $_ ne "$src/.."} @filenames;
	closedir(SRC);
	my(@directories) = grep {-d "$prefix$_"} @filenames;
	@filenames = grep {not -d "$prefix$_"} @filenames;
	my($file);
	foreach $file (@filenames) { #all the files
	  if (!link ("$prefix$file", "$dest$file")) {
	      if ( -e "$dest$file" ) {
		    #Probably a previous installation didn't delete it
            $cerr += $E_FILEEXISTS and $cerr += { file => "$prefix$file" };
		    print STDERR "File $prefix$file already exists.\n";
		} else {
            $cerr += $E_HARDLINK and 
            $cerr += { file => "$prefix$file", perror => $! } ;
		    return "Could not hard link $prefix$file: $! .\n";
		}
	  }
	}
	my($dir,$error_message);
	foreach $dir (@directories) {
	    unless (mkdir ("$dest$dir",0755)) {
		if ( -d "$dest$dir" ) { 
		    #Probably a previous installation didn't delete it
		    #because it had (user) files in it.
            $cerr += $E_DIREXISTS and $cerr += dir => "$dest$dir" } ;
		    print STDERR "Directory $dest$dir already exists.\n";
		} else {
            $cerr += $E_MKDIR and $cerr += { dir => "$dest$dir", perror => $!};
		    return "Could not create directory $dest$dir: $! .\n";
		}
	  }
	    if ($error_message=&RecursiveLink($dir,$dest,$prefix)) {
		return $error_message;
	    }
	}
    else {
    $cerr += $E_READDIR and $cerr += { dir => $src } ;
	return "Cannot read directory $src .\n";
    }
    return "";
}

#
# This function deletes from a site the files related to a service
#
sub ActualDeleteFilesystem() {
    my ($pdirs,$pfiles,$pnfiles,$pntfiles)=@_;
    my ($prefix)=($::user eq "root" ? "$VHConst::VIRTUALFS/FILESYSTEMTEMPLATE"
		                    : "$VHConst::VIRTUALFS/$::user");
    #Delete the "new files"
    my ($file);
    if ($::user ne "root") {
        foreach $file (@{$pnfiles}) {
            unlink ("$prefix$file->[0]") or
            $cerr += $E_USERDATANOTDEL and 
            $cerr += { data => "$prefix$file->[0]" } and
              print STDERR "User data has not been deleted: $prefix$file->[0] .\n";
        }
        foreach $file (@{$pntfiles}) {
            unlink ("$prefix$file->[0]") or 
            $cerr += $E_USERDATANOTDEL and 
            $cerr += { data => "$prefix$file->[0]" } and
              print STDERR "User data has not been deleted: $prefix$file->[0] .\n";
        }
    }

    #Handle "normal" files
    my ($dir);
    foreach $dir (keys %{$pfiles}) {
	my ($files) = $pfiles->{$dir};
	foreach $file (@$files) {
	    unlink ("$prefix$dir/$file") or
            $cerr += $E_USERDATANOTDEL and 
            $cerr += { data => "$prefix$dir" } and
	      print STDERR "User data has not been deleted: $prefix$dir/$file .\n";
	}
	if ($#{$files} == -1 ) { #we linked everything over
                                 #recursively 
	    if( opendir (DIR,"$prefix$dir")) {
		my (@allfiles) =  map {"$prefix$dir/$_"} readdir(DIR);
		closedir(DIR);
		@allfiles = grep {$_ ne "$prefix$dir/." and $_ ne "$prefix$dir/.."} @allfiles;
		system ("rm -rf ".join(" ",@allfiles));
	    } else {
        $cerr += $E_FILEPROB and $cerr += { file => "$prefix$dir" };
		print STDERR "Problems with $prefix$dir .\n";
	    }
	}
    }

    #This is slow, but we want to remove the links created by ldconfig for some services and
    #not fail to remove the directories
    system ("cd $prefix; /usr/sbin/chroot . /sbin/ldconfig >/dev/null 2>/dev/null"); 

    #Handle the directories
    foreach $dir (reverse (@{$pdirs})) {
	unless (rmdir "$prefix$dir->[0]") {
        $cerr += $E_NOTDELUSERDATA and $cerr += { dir => "$prefix$dir->[0]"} ;
	    print STDERR "Did not delete $prefix$dir->[0] . It probably contains user data.\n";
	}
    }
}
