import string
import sys
import socket
import copy
import os
import statvfs

import ensimapplpath
from vh3 import virthost
from vh3 import virtutil
from vh3.subdomain_utils import subdomain_factory
import vh3.modules
import requestutil

vh_diskquota = vh3.modules.diskquota
vh_users = vh3.modules.users
vh_reseller = vh3.modules.reseller
vh_apache = vh3.modules.apache

import vh3.db.bandwidth as bandwidthdb

import applutil
from serverman.current import networkmgr
from serverman import serviceman
import sessionman
import quota
import convertunits

import egv

def get_subdomain_count(site, siteconf):
    factory  = subdomain_factory(site,siteconf)
    return len(factory.items())

def get_html_root(conf):
    basic_path = vh_apache.html_dir()
    if conf['ssh']['jail'] == '1' or \
       conf['telnet']['jail'] == '1':
        return basic_path
    else:
        domain_path = virthost.domain_path(domain = conf['siteinfo']['domain'])
        return '%s%s' % (domain_path,basic_path)

def get_cgi_root(conf):
    basic_path = os.path.join(vh_apache.html_root(),
                              conf['cgi']['scriptalias'])
    if conf['ssh']['jail'] == '1' or \
       conf['telnet']['jail'] == '1':
        return basic_path
    else:
        domain_path = virthost.domain_path(domain = conf['siteinfo']['domain'])
        return '%s%s' % (domain_path,basic_path)
        

def get_uptime():
    """ returns uptime in seconds, rounded down """
    uptime_str = open('/proc/uptime').readline()
    i = string.split(uptime_str, ' ')[0]
    i = string.split(i, '.')[0]
    return string.atol(i)

def get_used_total(dir):
    devinfo = os.statvfs(dir)
    # used to use F_FRSIZE, but fedora vpses were giving large
    # negative numbers. somewhat safe to use F_BSIZE, since
    # ext3 seems to have them both equal.
    block_size = long(devinfo[statvfs.F_BSIZE])
    blocks_total = devinfo[statvfs.F_BLOCKS] * block_size
    blocks_used = blocks_total - (devinfo[statvfs.F_BFREE] * block_size)
    return (blocks_used, blocks_total,)

def get_home_virtual_disk_used():
    return get_used_total('/home/virtual')

def load_reseller_request(REQUEST, dict):
    load_system_conf(REQUEST, dict)
    dict['login_name'] = REQUEST['AUTHENTICATED_USER'].getUserName()

def get_reseller(username):
    reseller_info = vh_reseller.get_reseller(username)[0]
    reseller_id = reseller_info['reseller_id']
    reseller = vh_reseller.get_reseller_list(reseller_id = reseller_id)[0]
    return reseller
    
def get_reseller_conf(username):
    fqdn = applutil.get_fqdn()
    ip = networkmgr.NetworkManager().getIPAddress()
    reseller = get_reseller(username)
    retval = {'username': username,
              'uptime': get_uptime(),
              'reseller_id': reseller['reseller_id'],
              'reseller_fullname': reseller['fullname'],
              'reseller_email': reseller['email'],
              'hidden_services': virthost.get_hidden_service_list(),
              'server_name': fqdn,
              'server_ip': ip,
              }
    # bandwidth hack
    if (not egv.safe_is_pro()) and (not 'bandwidth' in retval['hidden_services']):
        retval['hidden_services'].append('bandwidth')
    return retval

def get_reseller_usage(username):
    reseller = get_reseller(username)
    newconf = {'diskquota_used': convertunits.convert_to_MB(reseller['alloc_quota'],
                                                            'B'),
               'diskquota_total': convertunits.convert_to_MB(reseller['total_quota'],
                                                             'B'),
               'bandwidth_used': convertunits.convert_to_MB(reseller['alloc_threshold'],
                                                            'B'),
               'bandwidth_total': convertunits.convert_to_MB(reseller['total_threshold'],
                                                             'B'),
               'users_used': reseller['alloc_maxusers'],
               'users_total': reseller['total_maxusers'],
               'ipinfo_nb_used': reseller['alloc_namebased'],
               'ipinfo_nb_total': reseller['total_namebased'],
               'ipinfo_ip_used': reseller['alloc_ipbased'],
               'ipinfo_ip_total': reseller['total_ipbased'],}
    return newconf
    
def get_user_conf(site, user):
    conf,errs = virthost.get_configs_from_site(site)
    domain = conf['siteinfo']['domain']
    retval = {'siteinfo_domain': domain,
              'username': user,}
    return retval
    
def get_user_usage(site, user):
    conf,errs = virthost.get_configs_from_site(site)
    uid = int(virthost.site_getpwnam(site, user, 'passwd')[2])
    used, errlist = vh_diskquota.get_site_user_used_quota(site, uid)
    quota, errlist = vh_diskquota.get_site_user_quota(site, uid)
    domain = conf['siteinfo']['domain']
    retval = {'siteinfo_domain': domain,
              'username': user,}
    if not used is None:
        retval['diskquota_used'] = convertunits.convert_to_MB(used, 'B')
    if not quota is None:
        retval['diskquota_total'] = convertunits.convert_to_MB(quota, 'B')
        
    return retval

def load_user_request(REQUEST,dict):
    load_system_conf(REQUEST, dict)
    dict['login_name'] = sessionman.getSession(REQUEST)['virtuser']
    
def load_appliance_request(REQUEST, dict):
    load_system_conf(REQUEST, dict)
    dict['login_name'] = REQUEST['AUTHENTICATED_USER'].getUserName()

def get_appliance_usage():
    disk_used = get_home_virtual_disk_used()
    return {'disk_used': convertunits.convert_to_MB(disk_used[0], 'B'),
            'disk_total': convertunits.convert_to_MB(disk_used[1], 'B')}
    
def get_appliance_conf():
    fqdn = applutil.get_fqdn()
    ip = networkmgr.NetworkManager().getIPAddress()
    retval = {'uptime': get_uptime(),
              'domain_count': len(virthost.get_domain_list()),
              'server_name': fqdn,
              'server_ip': ip,
              'hidden_services': virthost.get_hidden_service_list(),
              'services': serviceman.getSvcQueryList(),
              }
    # bandwidth hack
    if (not egv.safe_is_pro()) and (not 'bandwidth' in retval['hidden_services']):
        retval['hidden_services'].append('bandwidth')
    if not egv.safe_is_pro():
        try:
            retval['services'].remove('bandwidth')
        except ValueError,v:
            pass
    return retval


def load_system_conf(REQUEST, conf):
    fqdn = applutil.get_fqdn()
    ip = networkmgr.NetworkManager().getIPAddress()
    conf.update({'http_host': REQUEST['HTTP_HOST'],
                 'display_version': applutil.get_display_version(),
                 'product_name': applutil.get_product_name(),
                 'domain_limit': applutil.get_domain_limit()})

def load_site_request(REQUEST, dict):
    siteconf = sessionman.getSession(REQUEST)['siteconf']
    load_system_conf(REQUEST, dict)
    dict['login_name'] = siteconf['siteinfo']['admin_user']
    
def get_site_conf(site):
    conf,errs = virthost.get_configs_from_site(site)
    return conf

def get_site_usage(site):
    """ adds more pieces of data to site conf """
    conf,errs = virthost.get_configs_from_site(site)
    domain = conf['siteinfo']['domain']
    site = virthost.get_site_from_domain(domain)
    site_num = int(virthost.site_num(site))

    newconf = {}

    # disk quota, output: int (in MB), int (in MB)
    used, errlist = virthost.modules.diskquota.get_site_used_quota(site)
    diskquota_used, diskquota_total = None, None
    diskquota_dict = conf['diskquota']
    if not used == None:
        used = vh_diskquota.get_disp_quota(used, diskquota_dict['units'])
        if int(diskquota_dict['enabled']):
            diskquota_used = vh_diskquota.convert_to_MB(used,
                                                        diskquota_dict['units'])
            diskquota_total = vh_diskquota.convert_to_MB(diskquota_dict['quota'],
                                                         diskquota_dict['units'])
    if not diskquota_used is None:
        newconf['diskquota_used'] = diskquota_used
    if not diskquota_total is None:
        newconf['diskquota_total'] = diskquota_total

    # bw. output: long (XXX in bytes), long (XXX in bytes)
    # needs to be MB
    bw_used, bw_total, bw_rollover = bandwidthdb.get_site_totals(site_num,
                                                                 virtutil)
    if bw_used != None:
        newconf['bandwidth_used'] = convertunits.convert_to_MB(bw_used,'B')
    newconf['bandwidth_total'] =  convertunits.convert_to_MB(bw_total,'B')

    # users. output: int, int
    users_used = len(vh_users.get_site_uids(site_num))
    users_total = conf['users']['maxusers']

    # mysql
    # may not always be installed... so must take care in loading it
    # up.
    if hasattr(vh3.modules, 'mysql') and int(conf['mysql']['enabled']) == 1:
        mysql_mod = getattr(vh3.modules, 'mysql')
        newconf['mysql_total'] = int(conf['mysql']['dbasenum'])
        newconf['mysql_used'] = len(mysql_mod.ListDatabases(site))
    else:
        newconf['mysql_total'] = 0
        newconf['mysql_used'] = 0

    # subdomains
    if hasattr(vh3.modules, 'subdomain') and int(conf['subdomain']['enabled']) == 1:
        subdomain_mod = getattr(vh3.modules, 'subdomain')
        newconf['subdomain_total'] = int(conf['subdomain']['max'])
        newconf['subdomain_used'] = len(subdomain_mod.subdomain_factory(site,conf).listitems())
    else:
        newconf['subdomain_total'] = 0
        newconf['subdomain_used'] = 0
        
        
    newconf.update({'users_used': users_used,
                    'users_total': users_total,})
    return newconf
