import os
import ensimapplpath
import DynamicImport
import applutil
import string
import intl
import dashboard
import requestutil
from vh3 import virthost
import egv

SKINS_DIR = '/usr/lib/opcenter/skins'
CURRENT_SKIN_SYMLINK = os.path.join(SKINS_DIR, 'current')

DEFAULT_SKIN = 'mercury'

def get_skins():
    # looks in SKINS_DIR and lists out all
    # dirs in that directory, filtering out the 'current' symlink
    entries = filter(lambda x: x != 'current', os.listdir(SKINS_DIR))
    entries = filter(lambda x: os.path.isdir(os.path.join(SKINS_DIR, x)), entries)
    return entries

def get_current_skin():
    if os.path.exists(CURRENT_SKIN_SYMLINK):
        return os.path.basename(os.readlink(CURRENT_SKIN_SYMLINK))
    else:
        # XXX what should default skin be?
        return DEFAULT_SKIN

def set_current_skin(skin_name):
    try:
        os.unlink(CURRENT_SKIN_SYMLINK)
    except:
        pass
    if os.path.exists(os.path.join(SKINS_DIR,skin_name)):
        os.symlink(os.path.join(SKINS_DIR, skin_name),
                   CURRENT_SKIN_SYMLINK)

def load_skin(REQUEST):
    skin_name = get_current_skin()
    skin_mod = DynamicImport.loadModule(skin_name,
                                        path = \
                                        os.path.join(SKINS_DIR,
                                                     skin_name))
    skin_mod.Skin.skin_name = skin_name
    return skin_mod.Skin(REQUEST)

class UnhiddenServicesFilter:
    """ returns 0 if the service should be hidden from display or not.
    this queries the backend to see if the service has been
    'hide_service'-ed  or not.
    Usage:
    filterobj = HiddenServicesFilter()
    if filterobj.is_hidden('apache'):
        print 'service apache is hidden'
    else:
        print 'service apache is unhidden' """

    def __init__(self, REQUEST):
        self.conf = dashboard.get_appliance_conf()

    def is_hidden(self, service):
        if service in self.conf['hidden_services'] or service not in self.conf['services']:
            return 1
        else:
            return 0

class EnabledServicesFilter:
    
    """ returns 0 if the service should be hidden from display or not.
    this queries the backend to see if the service has been
    enabled for the site (pointed to by REQUEST) or not.
    Usage:
    filterobj = EnabledServicesFilter()
    if filterobj.is_hidden('apache'):
        print 'service apache is disabled'
    else:
        print 'service apache is enabled' """

    def __init__(self, REQUEST):
        site = requestutil.get_site_from_request(REQUEST)
        self.saconf = dashboard.get_site_conf(site)
        if self.saconf.has_key('users'):
            # users not hidden
            self.saconf['users']['enabled']='1'

    def is_hidden(self, service):
        # bandwidth hack
        if (not egv.safe_is_pro()) and service == 'bandwidth':
            return 1
        # users has nothing to do with not showing.
        if self.saconf.has_key(service):
            # PR 29686 - I am not sure what this check means. But I believe we
            # should not show spam option to the user if it is not configured
            # for the site. Remaining changes are in mercury.py
            #if self.saconf[service] == "spam_filter":
            #    return self.saconf[service]['enabled']
            #else:
            return not int(self.saconf[service]['enabled'])
        else:
            return 1
    
class SkinBase:

    legacy_customization_supported = 0
    
    def __init__(self, REQUEST):
        self.REQUEST = REQUEST
        self.skin_base_dir = os.path.join(SKINS_DIR,
                                          self.skin_name)
        if self.REQUEST:
            self.common_translations = {}
            self.prep_common_translations(self.common_translations)

    def get_response(self):
        return self.REQUEST.response

    def get_lang(self):
        return requestutil.lang_from_request(self.REQUEST)

    def get_display_version(self):
        return applutil.get_display_version()

    def get_product_name(self):
        return applutil.get_product_name()
    
    def get_domain_limit(self):
        return applutil.get_domain_limit()
    
    def write_header(self, field, val):
        r = self.REQUEST.response
        r.setHeader(field, val)

    def ftoa(self, conv):
        # convert float or string to localized display
        return intl.ftoa(requestutil.lang_from_request(self.REQUEST),
                         conv)

    def get_httpd_conf_path(self):
        return os.path.join(self.skin_base_dir, 'httpdconf')

    def batch_ftoa(self, dict, convitems):
        for i in convitems:
            dict[i] = self.ftoa(dict[i])
            
    def T_(self,in_string):
        """ gets the translated string from the .po file """
        return intl.T__(self.REQUEST, in_string)

    def T_duration(self, seconds):
        """ returns the i18n-ed display of time duration, e.g.
        6 days, 5 hours, 30 minutes """
        days = seconds / 86400L
        hours = ((seconds % 86400L) / 3600L)
        minutes = ((seconds % 86400L) % 3600L) / 60L
        return self.T_('%(days)d days, %(hours)d hours, %(minutes)d minutes') % \
               {'days': days,
                'hours': hours,
                'minutes': minutes}

    def load_common_translations(self, dict):
        dict.update(self.common_translations)
        
    def prep_common_translations(self, dict):
        for i in ['User Administrator',
                  'Site Administrator',
                  'Appliance Administrator',
                  'Reseller Administrator',
                  'Server',
                  'IP Address',
                  'Domain Count',
                  'Server Statistics',
                  'MB Used',
                  'Username',
                  'SYSTEM STATS',
                  'Disk Space',
                  'Bandwidth',
                  'Fullname',
                  'Email',
                  'Users',
                  'IP-Based Domains',
                  'Name-Based Domains',
                  'Subdomains',
                  'Databases',
                  'Web Document Path',
                  'CGI Document Path',
                  'Web Server Name',
                  'Mail Server Name',
                  'FTP Server Name',
                  'Administrator',
                  'Site Details',
                  'Domain Name',
                  'IP-Based Domains',
                  'Name-Based Domains',
                  'Reseller',
                  'Reseller Statistics']:
            newstr = string.replace(i, '-', '_')
            newstr = string.replace(newstr, ' ', '_')
            dict['T_%s' % newstr] = self.T_(i)
        dict['frames_warning'] = self.T_('Your browser does not support frames. Please upgrade your browser to a version that does.')
                 
if __name__=='__main__':
    print load_skin(None)
