errors = {}
options = {}
macro_ops = {}
stages = {}
svcid = 20

SUCCESS = 0
WARNING = 1
ERROR = 2

def determine_severity(msgs):
    severity = SUCCESS
    if msgs and len(msgs) > 0:
        for msg in msgs:
            severity = max(severity, msg[0])
    return severity

def determine_action_severity(list):
    severity = SUCCESS
    if list and len(list) > 0:
        for action in list:
            severity = max(severity, determine_severity(action[2]))
    return severity

def get_option_from_code(code, options_table):
    index = (code >> 48) & 0xff
    return options_table.get(index, None)

def get_errornum_from_code(code):
    return (code >> 56) & 0xff
options[0] = "namebased"; namebased = 0L
options[1] = "nbaddrs"; nbaddrs = 1L
options[2] = "ipaddrs"; ipaddrs = 2L
options[3] = "enabled"; enabled = 3L
options[4] = "total_namebased"; total_namebased = 4L
options[5] = "total_ipbased"; total_ipbased = 5L
options[6] = "enabled_namebased"; enabled_namebased = 6L
options[6] = "enabled_ipbased"; enabled_ipbased = 6L

NB_NO_ADDRS = (20L << 56) | (nbaddrs << 48) | 0; errors[(20L << 56) | (nbaddrs << 48) | 0] = 'Namebased sites must have at least one IP address selected'
NB_BAD_ADDR = (20L << 56) | (nbaddrs << 48) | 1; errors[(20L << 56) | (nbaddrs << 48) | 1] = "Bad IP address %(addr)s"
NB_DEREG_ADDR = (20L << 56) | (nbaddrs << 48) | 2; errors[(20L << 56) | (nbaddrs << 48) | 2] = "IP address %(addr)s is no longer in list of registered namebased addresses"
NB_UNREG_ADDR = (20L << 56) | (nbaddrs << 48) | 3; errors[(20L << 56) | (nbaddrs << 48) | 3] = "IP address %(addr)s is not in list registered namebased addresses"
IP_NO_ADDRS = (20L << 56) | (ipaddrs << 48) | 4; errors[(20L << 56) | (ipaddrs << 48) | 4] = 'IP-based sites must have at least one IP address specified'
IP_BAD_ADDR = (20L << 56) | (ipaddrs << 48) | 5; errors[(20L << 56) | (ipaddrs << 48) | 5] = "Bad IP address %(addr)s"
IP_ADDR_IS_NB = (20L << 56) | (ipaddrs << 48) | 6; errors[(20L << 56) | (ipaddrs << 48) | 6] = "IP address %(addr)s is in list of registered namebased addresses"
IP_ADDR_IN_USE = (20L << 56) | (ipaddrs << 48) | 7; errors[(20L << 56) | (ipaddrs << 48) | 7] = "IP address %(addr)s is already in use by existing site %(site)s"
INVALID_NAMEBASED_FLAG = (20L << 56) | (namebased << 48) | 8; errors[(20L << 56) | (namebased << 48) | 8] = 'Value must be 1 or 0'
IP_ADDR_FOR_NB_PLAN = (20L << 56) | (ipaddrs << 48) | 9; errors[(20L << 56) | (ipaddrs << 48) | 9] = 'IP addresses are specified for a namebased plan'
IP_INADDR_ANY = (20L << 56) | (ipaddrs << 48) | 10; errors[(20L << 56) | (ipaddrs << 48) | 10] = 'IP address 0.0.0.0 is not allowed'
IP_INADDR_NONE = (20L << 56) | (ipaddrs << 48) | 11; errors[(20L << 56) | (ipaddrs << 48) | 11] = 'IP address 255.255.255.255 is not allowed'
NB_INADDR_ANY = (20L << 56) | (nbaddrs << 48) | 12; errors[(20L << 56) | (nbaddrs << 48) | 12] = 'IP address 0.0.0.0 is not allowed'
NB_INADDR_NONE = (20L << 56) | (nbaddrs << 48) | 13; errors[(20L << 56) | (nbaddrs << 48) | 13] = 'IP address 255.255.255.255 is not allowed'
IP_ADDR_ALLOWED_RANGES = (20L << 56) | (ipaddrs << 48) | 14; errors[(20L << 56) | (ipaddrs << 48) | 14] = "IP address %(addr)s does not fall within any of the allowed IP ranges"
IP_ADDR_DISALLOWED_RANGES = (20L << 56) | (ipaddrs << 48) | 15; errors[(20L << 56) | (ipaddrs << 48) | 15] = "IP address %(addr)s falls within the disallowed range of IP address %(addr1)s to %(addr2)s"
WRONG_HOST_RESOLUTION = (20L << 56) | (ipaddrs << 48) | 16; errors[(20L << 56) | (ipaddrs << 48) | 16] = "Hostname %(domain)s resolves to %(ip)s which does not match the ipaddress %(addrs)s. This could be due to incorrectly entered IP address, misconfiguration in DNS settings, or incomplete DNS propagation."
IPADDR_BADCHARS = (20L << 56) | (ipaddrs << 48) | 17; errors[(20L << 56) | (ipaddrs << 48) | 17] = "IP address %(ipaddr)s contains invalid characters: %(badchars)s"
TOO_MANY_DOMAINS = (20L << 56) | (enabled << 48) | 18; errors[(20L << 56) | (enabled << 48) | 18] = "Can not create site as this will exceed the site limit. Current number of sites is %(current)d. Maximum allowed is %(max)d."
TOO_MANY_NB_DOMAINS = (20L << 56) | (namebased << 48) | 19; errors[(20L << 56) | (namebased << 48) | 19] = "Can not create site as this will exceed the name-based site limit. Current number of name-based sites is %(current)d. Maximum allowed is %(max)d."
TOO_MANY_NB_DOMAINS2 = (20L << 56) | (namebased << 48) | 20; errors[(20L << 56) | (namebased << 48) | 20] = "Can not switch site from namebased to IP-based as this will exceed the name-based site limit. Current number of name-based sites is %(current)d. Maximum allowed is %(max)d."
RESELLER_OVER_IPBASED = (20L << 56) | (total_ipbased << 48) | 21; errors[(20L << 56) | (total_ipbased << 48) | 21] = "Number of ip domains exceeds allowed limits."
RESELLER_OVER_NAMEBASED = (20L << 56) | (total_namebased << 48) | 22; errors[(20L << 56) | (total_namebased << 48) | 22] = "Number of name-based domains exceeds allowed limits."
TOTAL_NAMEBASED_NAN = (20L << 56) | (total_namebased << 48) | 23; errors[(20L << 56) | (total_namebased << 48) | 23] = "Name Based Sites value %(inval)s not a number."
TOTAL_IPBASED_NAN = (20L << 56) | (total_ipbased << 48) | 24; errors[(20L << 56) | (total_ipbased << 48) | 24] = "IP Based Sites value %(inval)s not a number."
ENABLED_IPINFO_NB_NAN = (20L << 56) | (enabled_namebased << 48) | 25; errors[(20L << 56) | (enabled_namebased << 48) | 25] = "IP Based Sites Enabled value %(inval)s not 0 or 1."
ENABLED_IPINFO_IP_NAN = (20L << 56) | (enabled_ipbased << 48) | 26; errors[(20L << 56) | (enabled_ipbased << 48) | 26] = "Name Based Sites value %(inval)s not 0 or 1."

ADD_ADDR = ((20L << 56) | (255L << 48)) | 0; errors[((20L << 56) | (255L << 48)) | 0] = "Failed to add IP address %(addr)s to interface %(intf)s. Error is %(errno)d.\nStandard Out: %(stdout)s\nStandard Error: %(stderr)s"
REM_ADDR = ((20L << 56) | (255L << 48)) | 1; errors[((20L << 56) | (255L << 48)) | 1] = "Failed to remove IP address %(addr)s from interface %(intf)s. Error is %(errno)d.\nStandard Out: %(stdout)s\nStandard Error: %(stderr)s"
COPY_HOSTS = ((20L << 56) | (255L << 48)) | 2; errors[((20L << 56) | (255L << 48)) | 2] = "Failed to copy %(from)s to %(to)s. File %(file)s. Error is %(errno)d : %(errstr)s"
ADD_IPS_TO_HOSTS = ((20L << 56) | (255L << 48)) | 3; errors[((20L << 56) | (255L << 48)) | 3] = "Failed to add site IP addresses to site %(file)s. Error is %(errno)d : %(errstr)s"
ADD_IPS_TO_HOSTS_CLOSE = ((20L << 56) | (255L << 48)) | 4; errors[((20L << 56) | (255L << 48)) | 4] = "Failed to close %(file)s after updating. Error is %(errno)d : %(errstr)s"
NO_IPRANGE_FILE = ((20L << 56) | (255L << 48)) | 5; errors[((20L << 56) | (255L << 48)) | 5] = "Failed to open IP range file %(file)s. Error is %(errno)d : %(errstr)s"
IPRANGE_READ = ((20L << 56) | (255L << 48)) | 6; errors[((20L << 56) | (255L << 48)) | 6] = "Failed to read information from IP range file %(file)s. Error is %(errno)d : %(errstr)s"
IPRANGE_CLOSE = ((20L << 56) | (255L << 48)) | 7; errors[((20L << 56) | (255L << 48)) | 7] = "Failed to close IP range file %(file)s. Error is %(errno)d : %(errstr)s"
IPRANGE_BAD_ADDR = ((20L << 56) | (255L << 48)) | 8; errors[((20L << 56) | (255L << 48)) | 8] = "Bad IP address %(addr)s on line %(line)d in file %(file)s"
IPRANGE_REVERSE = ((20L << 56) | (255L << 48)) | 9; errors[((20L << 56) | (255L << 48)) | 9] = "IP address %(addr2)s is less than IP address %(addr1)s on line %(line)d in file %(file)s"
IPRANGE_TOO_MANY_DOTS = ((20L << 56) | (255L << 48)) | 10; errors[((20L << 56) | (255L << 48)) | 10] = "Too many '.'s in IP address %(addr)s on line %(line)d in file %(file)s"
DB_REMOVE_OLD_IPINFO = ((20L << 56) | (255L << 48)) | 11; errors[((20L << 56) | (255L << 48)) | 11] = "Failed to remove old site ipinfo from database. DBERR: %(dberrstr)s"
DB_ADD_IPINFO = ((20L << 56) | (255L << 48)) | 12; errors[((20L << 56) | (255L << 48)) | 12] = "Failed to add site ipinfo to database. DBERR: %(dberrstr)s"
DB_UPDATE_IPINFO = ((20L << 56) | (255L << 48)) | 13; errors[((20L << 56) | (255L << 48)) | 13] = "Failed to update site ipinfo in database. DBERR: %(dberrstr)s"
