# This customization file may provides the vh_conf function
# to alter the way the virtualhosting containers for apache
# are generated per site. In addition, certain variables may
# be specified in order to alter the way the ensim-generated
# configuration files are created, in case you would prefer
# to not have to write the code yourself. Below is a discussion
# of the variables and functions available to be modified. This
# file is composed of python code, it is not parsed; all
# uncommented (lines that don't begin with '#') statements must
# be valid python.
#
# Note: if you wish to define either these variables of
# functions for all sites, you may copy this file into
# /usr/lib/ensim-python/site-packages/vh3/custom directory,
# renaming it to apache.py, and make your modifications.
# Please do not modify the file in its original location,
# as any changes made will be lost during an upgrade.
# After copying the file to the location above, Webppliance must
# be restarted before the customizations will start to take
# effect during site edits through the GUI.
#
# Note: if you wish to define either these variables or functions
# for a specific site, you may copy this file into the
# /home/virtual/site<n>/info/custom directory, renaming it
# to apache.py. Site-local customizations take priority over
# global customizations. That is, if the site-local
# customizations define one of the variables or functions
# listed below, then that variable or function will be used
# instead of the definition in the global customizations.

########################################
# Variables
########################################

# addrs is used to set the IP addresses for
# a virtualhost container. It must be a python
# list. It is not recommended that you
# customize this value. Modify this assignment
# and ucomment it to customize the addresses.
# example: addrs = [10.0.0.1, 10.0.0.2]

# port is an integer value used to set the
# port for a virtualhost container. The
# default is 80. Modify this assignment and
# uncomment it to set the port to something
# else.
# example: port = 8080
# port = 80

# extra_server_aliases is a string of additional
# server aliases that should be added to a namebased
# virtualhosting container. This string should not
# end with a newline. The default is blank. Modify
# this assignment and uncomment it to add additional
# aliases. If the virtualhosting container is IP-based,
# this setting will be ignored.
# example: extra_server_aliases = '*.mydomain.com *.mydomain.net'
# extra_server_aliases = ''

# serveradmin is the email address that will
# be used in the ServerAdmin directive of
# the virthost container. It defaults to the
# site administrator's email.
# serveradmin = user@domain

# user_dir is used to set the UserDir directive
# in the virthost container. Any path specified
# here will be appended to the proper virtual
# domain filesystem directory. The path must
# begin with a '/'. The default value is
# '/home/*/public_html'. If uncommented, this
# must be a valid python string. Note that this
# variable is no longer used; you need only modify
# the user_dir_alias in order to have changes take
# effect.
# user_dir = '/home/*/public_html'

# user_dir_alias is a companion to the user_dir
# variable. URIs of the form /users/<username>/...
# are mapped to /home/<username>/public_html/...
# by the setting '/home/$1/public_html/$2', which
# is the default. The $1 matches the <username>,
# and the $2 matches the rest of the URL. If
# uncommented, this must be a valid python string,
# and must begin with a '/'.
# user_alias_dir = '/home/$1/public_html/$2'

# docroot_permissions sets the directives that are
# contained in the document root <Directory ...>
# </Directory> block in the virthost container.
# If uncommented, this must be a valid python string.
# It does not require newline characters at either
# the beginning or the end.
# docroot_permissions = '\t\tAllow from all\n\t\tAllowOverride All'

# user_docroot_permissions sets the directives that are
# contained in the user document root <Directory ...>
# </Directory> block in the virthost container.
# If uncommented, this must be a valid python string.
# It does not require newline characters at either
# the beginning or the end.
# user_docroot_permissions = '\t\tAllow from all\n\t\tAllowOverride All'

# use_https_admin is a variable that determines whether
# the site admin and user admin pages use http or https.
# It may be set to 1 or 0 to choose https or http,
# respectively. The default value is 1.
#
# https_admin = 1

# site_admin_url is a variable that determines what
# URL will be redirected to the site administrator login
# page. It should be a string, and should begin with a '/'
# The default value is "/admin"
#
# site_admin_url = "/admin"

# user_admin_url is a variable that determines what
# URL will be redirected to the user administrator login
# page. It should be a string, and should begin with a '/'
# The default value is "/user"
#
# user_admin_url = "/user"

########################################
# Functions
########################################

# add_custom_global_directives allows you to add whatever additional
# global scope directives you'd like to the configuration generated
# by Ensim. For example, you may want to add global redirects that
# will send certain URIs to this virthost container. This function
# should return a valid python string, which does not need to begin or
# end with newlines. More advanced python programmers may
# take advantage of the fact that there are two
# customizations modules, one global and one local,
# and if both are defined, the one containing
# the definition of this function is passed the
# other module via the cust parameter.
#
# def add_custom_global_directives(site, newconf, oldconf, cust):

# add_custom_directives allows you to add whatever additional
# directives you'd like to the virthost container generated
# by Ensim. These directives will come at the end of the
# virthost container. This function should return a
# valid python string, which does not need to begin or
# end with newlines. More advanced python programmers may
# take advantage of the fact that there are two
# customizations modules, one global and one local,
# and if both are defined, the one containing
# the definition of this function is passed the
# other module via the cust parameter.
#
# def add_custom_directives(site, newconf, oldconf, cust):

# vh_conf is the function that is called
# to generate the apache virthost container.
# Below is an example
# of the configuration generated by default.
# If you choose to define this function,
# please pay attention to the lines marked
# with an (*) below. These lines are required
# in order for the service to work (the (*)
# should not be included in the content that
# you generate). Often there will be a comment
# above each of these lines explaining where to
# retrieve the information needed to generate
# them. More advanced python programmers may
# take advantage of the fact that there are two
# customizations modules, one global and one local,
# and if both are defined, the one containing
# the definition of this function is passed the
# other module via the cust parameter.
#
# Example configuration for vh_conf:
#
#    # These IP addresses are obtained from either
#    # newconf['ipinfo']['nbaddrs'] or newconf['ipinfo']['ipaddrs']
#    # depending on whether or not newconf['ipinfo']['namebased'] is
#    # set to '1' or '0', respectively. You may need to eval() the
#    # the value retrieved from the dictionary before you may use it
#    # as a list.
#(*) <VirtualHost 10.1.1.152:80 10.8.11.69:80 10.8.11.60:80>
#          # This value comes from newconf['apache']['webserver']. It is
#          # not recommended that you change this line, as it will interfere
#          # with bandwidth tracking.
#(*)       ServerName www.bar.com
#          # These values come from newconf['siteinfo']['domain'] and
#          # newconf['aliases']['aliases']. You may need to eval() the
#          # aliases value before you can use it as a list.
#(*)       ServerAlias ensim.com
#          # This value comes from newconf['siteinfo']['email']
#(*)       ServerAdmin blackhole@ensim.com
#          # The /home/virtual/site<n>/fst portion of the path
#          # is required. site<n> is the first argument to vh_conf.
#          # It is not recommended that you change this setting.
#(*)       DocumentRoot /home/virtual/site1/fst/var/www/html
#          # The next two lines pertain to suexec and are populated
#          # newconf['siteinfo']['admin']
#(*)       User admin1
#(*)       Group admin1
#          # The IP address in the next two lines should be the IP used for
#          # accessing the administration GUI. The domain value is
#          # retrieved from newconf['siteinfo']['domain']
#(*)       Redirect /admin https://10.1.1.152:19638/webhost/services/virtualhosting/siteadmin?ocw_login_domain=ensim.com
#(*)       Redirect /user https://10.1.1.152:19638/webhost/services/virtualhosting/useradmin?ocw_login_domain=ensim.com
#          # The site<n> below is obtained from the first argument to vh_conf
#(*)       UserDir /home/virtual/site1/fst/home/*/public_html
#          AliasMatch ^/users/([^/]+)/?(.*) /home/virtual/site1/fst/home/$1/public_html/$2
#          <Directory /home/virtual/site1/fst/var/www/html/>
#                  Allow from all
#                  AllowOverride All
#                  Order allow,deny
#          </Directory>
#          # The site<n> in the next three lines is obtained from the
#          # first argument to vh_conf
#(*)       SetEnv SITE_ROOT /home/virtual/site1/fst
#(*)       SetEnv SITE_HTMLROOT /home/virtual/site1/fst/var/www/html
#(*)       Include /etc/httpd/conf/site1
#    </VirtualHost>
#
# def vh_conf(site, newconf, oldconf, cust):
