# This customization file provides certain variables that affect
# the behavior of the subdomain implementation on WEBppliance 3.5.0
# 
# NOTE -- As of this writing, only global customizations are supported 
# for subdomains. That means, that only global customizations may be applied
# for subdomains, and that all such customizations affect *all* sites on the 
# webppliance server 
# 
# Please do not modify this file in its original location, because the changes
# you make may be lost during an upgrade. 
# 
# If you wish to change the default values of any of the variables, you may 
# copy this file as /usr/lib/ensim-python/site-packages/vh3/custom/subdomain.py
# 
# Also note that you must restart the webppliance GUI for any of these 
# customizations to take effect in the GUI
# 

# VARIABLES -- 
# 

# SUBDOMAIN_CONF_DIR
#   description:
#       the directory where the subdomain related maps are created
#   default:
# SUBDOMAIN_CONF_DIR='/etc/virtualhosting/subdomain/'
# 

# APACHE_SUBDOMAIN_TMPL
#   description:
#       the virtualhosting content that is included into every site's 
#       VirtualHost container. Defines the RewriteRules for subdomains.
#       The contents of this are written out into
#           /etc/httpd/conf/site<x>/subdomain,
#       whenever a site<x> is created/edited with subdomains enabled
#   default:
# APACHE_SUBDOMAIN_TMPL = '''
#     # basic server aliases for subdomain
#     ServerAlias %(aliases)s
# 
#     # this is required if we want to enable symlinks from the subdomain
#     # base directory (default /var/www/subdomain/)
#     # Also, note that as of Apache 1.3 none of the wildcards match a `/' 
#     # character, which more closely mimics the behavior of Unix shells
# 
#     # Subdomain Base Directory options
#     <Directory %(fst)s%(base)s/*/>
#         Options +FollowSymlinks
#         # AllowOverride All
#     </Directory>
# 
#     # Document Root options
#     <Directory %(fst)s%(base)s/*/html/>
#         Options +SymLinksIfOwnerMatch
#         AllowOverride AuthConfig Indexes
#     </Directory>
# 
#     # CGI SubDirectory Directory Options
#     <Directory %(fst)s%(base)s/*/html/%(CGIBIN)s/>
#         Options +ExecCGI
#         SetHandler cgi-script
#         AllowOverride All
#     </Directory>
# 
#     # The core rewrite rules
# 
#     RewriteEngine On
#     RewriteMap subdomains txt:%(REWRITEMAP)s
#     RewriteMap subdirectory_cgi txt:%(CGISUBMAP)s
#     RewriteMap scriptalias_cgi txt:%(CGIMAP)s
#     RewriteMap pub_subdomain_cgi txt:%(PUBLIC_SUBDOMAIN_CGI_MAP)s
# 
#     # Avoiding CGI directory spoofs at domain level -- cannot let http://domain.com/sub/cgi-bin/foo.cgi source
#     # 
#     # multiple slash problem
#     RewriteCond %%{REQUEST_URI} ^[^?]*//.*$
#     RewriteRule ^([^?]*)//(.*)$ $1/$2 [N]
#     RewriteCond ${pub_subdomain_cgi:$1} ^(.+)$
#     RewriteRule ^/(.*/cgi-bin)/(.+)$ %%1/$2 [T=application/x-httpd-cgi]
# 
#     # Per directory CGI access - execute CGI scripts from within 
#     # ${documentroot}/cgi-bin/ -- ALLOWING FOR WILDCARDS
#     RewriteCond %%{REQUEST_URI} ^/%(CGIBIN)s/
#     RewriteCond ${subdirectory_cgi:%%{HTTP_HOST}\:*} ^(.+)$
#     RewriteCond ${subdomains:%%{HTTP_HOST}} ^(.+)$
#     RewriteRule ^/%(CGIBIN)s/(.*)$ %%1/html/%(CGIBIN)s/$1 [T=application/x-httpd-cgi,L]
# 
#     # Per directory CGI access - execute CGI scripts from within 
#     # ${documentroot}/cgi-bin/
#     RewriteCond %%{REQUEST_URI} ^/%(CGIBIN)s/
#     RewriteCond %%{REQUEST_URI} \.([^\./]+)$
#     RewriteCond ${subdirectory_cgi:%%{HTTP_HOST}\:%%1} ^(.+)$
#     RewriteCond ${subdomains:%%{HTTP_HOST}} ^(.+)$
#     RewriteRule ^/%(CGIBIN)s/(.*)$ %%1/html/%(CGIBIN)s/$1 [T=application/x-httpd-cgi,L]
# 
#     # CGI script execution from the scriptalias directory
#     RewriteCond %%{REQUEST_URI} \.([^\./]+)$
#     RewriteCond ${scriptalias_cgi:%%{HTTP_HOST}\:%%1} ^(.+)$
#     RewriteCond ${subdomains:%%{HTTP_HOST}} ^(.+)$
#     RewriteRule ^/(.*)$ %%1/html/$1 [T=application/x-httpd-cgi,L]
# 
#     # To emulate mod_rewrite
#     RewriteCond ${subdomains:%%{HTTP_HOST}} ^(.+)$
#     RewriteCond %%1/html/%%{REQUEST_FILENAME} -d
#     RewriteRule ^(.+[^/])$ http://%%{HTTP_HOST}%%{REQUEST_URI}/ [R] [L]
# 
#     # Lastly, the fallback document root
#     RewriteCond ${subdomains:%%{HTTP_HOST}} ^(.+)$
#     RewriteCond %%{IS_SUBREQ} !true
#     RewriteCond %%{REQUEST_URI} !/interpreters/[-0-9a-zA-Z]+-script
#     RewriteRule ^/(.*)$ %%1/html/$1
# 
# '''
# 

# APACHE_SUBDOMAIN_PREVIEW_TMPL
#   description:
#       This defines the content that is written out into
#       /etc/httpd/conf/site<x>.preview/subdomain, 
#       whenever a site <x> is added/edited with subdomains enabled. 
#       This contains RewriteRules that enable the site's subdomains to 
#       be accessible via domain preview
#   default: 
# APACHE_SUBDOMAIN_PREVIEW_TMPL = '''
# 
#     # this is required if we want to enable symlinks from the subdomain
#     # base directory (default /var/www/subdomain/)
#     # Also, note that as of Apache 1.3 none of the wildcards match a `/' 
#     # character, which more closely mimics the behavior of Unix shells
# 
#     # Subdomain Base Directory options
#     <Directory %(fst)s%(base)s/*/>
#         Options +FollowSymlinks
#         # AllowOverride All
#     </Directory>
# 
#     # Document Root options
#     # NOTE: For CGI scripts `Via Script Extensions' the CGI scripts lie
#     # within the document root
#     <Directory %(fst)s%(base)s/*/html/>
#         Options +SymLinksIfOwnerMatch
#         # cjaeger: why not AllowOverride All?
#         AllowOverride AuthConfig Indexes
#     </Directory>
# 
#     # CGI SubDirectory Directory Options
#     <Directory %(fst)s%(base)s/*/html/%(CGIBIN)s/>
#         Options +ExecCGI
#         SetHandler cgi-script
#         AllowOverride All
#     </Directory>
# 
#     Alias /~admin%(siteidx)s/subdomain %(fst)s%(base)s
# 
# '''
# 

# REWRITEMAP
#   description:
#       the site-specific subdomain map. This maps a fully qualified subdomain
#       name to the absolute directory on the webppliance server, where the 
#       subdomain's content is stored.
#   default:
# REWRITEMAP = '/etc/virtualhosting/subdomain/%(site)s.map'
# 

# CGIMAP  (DEPRECATED)
#   description: 
#       this contains (or used to contain) the following mapping:
#       ...
#       subdomain_name:cgi_extension    *
#       ... 
#       for each subdomain_name in the site's subdomains, and for each 
#       cgi_extension enabled for that subdomain. cgi_extension can be *
#   default:
# CGIMAP ='/etc/virtualhosting/subdomain/%(site)s.cgimap'
# 

# CGISUBMAP
#   description: 
#       this contains the following mapping:
#       ...
#       subdomain_name:cgi_extension    *
#       ... 
#       for each subdomain_name in the site's subdomains, and for each 
#       cgi_extension enabled for that subdomain. cgi_extension can be *
#   default:
# CGISUBMAP ='/etc/virtualhosting/subdomain/%(site)s.cgisubmap'
# 

# OWNERMAP (DEPRECATED)
#   description: 
#       a Berkeley DB map that maps a subdomain to its owner
#   default:
# OWNERMAP ='/etc/virtualhosting/subdomain/%(site)s.ownermap'
# 

# CGI_OWNER_MAP 
#   description: 
#       a Berkeley DB map that maps a directory to its owner
#   default:
# CGI_OWNER_MAP ='/etc/virtualhosting/cgi/%(site)s.cgi_owner_map'
# 

# CGIBIN
#   description: 
#       the default (and only) url token that can be used to run subdomain
#       CGI scripts. This defaults to 'cgi-bin', which means that all internal
#       CGI subdirectories will be called 'cgi-bin', and all the subdomain CGI
#       URLs will look like http://subdomain.domain.com/cgi-bin/foo.cgi
#   default:
# CGIBIN ='cgi-bin'

# PLACEHOLDER_CONTENT
#   description: 
#       the content that is written out to the index.html file in the subdomain
#       directory, whenever a new subdomain is created. welcome page of sorts.
#   default:
# PLACEHOLDER_CONTENT='''
# <html>
# <head>
# <title>subdomain %(subdomain)s</title>
# </head>
# <body>
# this is a placeholder for the subdomain %(subdomain)s
# </body>
# </html>
# '''
# 

# HTTP_RELOAD_CMD
#   description:
#       the command that is used to reload apache whenever a subdomain is 
#       added to a site. as of this writing, opinion is divided as to whether
#       this should be 'reload-grc', or 'reload'. The default is 'reload-grc',
#       but the WEBppliance server administrator may want to change it, if too
#       many instances of 'long lost child came home' appear in the apache 
#       error logs
#   default:
# HTTP_RELOAD_CMD='! `/sbin/chkconfig httpd` || /etc/rc.d/init.d/httpd_app_init reload-grc'

# SKIP_GLOBAL_MAPPING
#   description: 
#       set this value to 1 only if you know what you are doing ! Setting this
#       to 1 will bypass all global uniqueness checks while subdomains are 
#       being added. 
#   default:
#  SKIP_GLOBAL_MAPPING=0
# 

# SUBDOMAIN_SYMLINK_BASE
#   description:
#       this is the directory, within the site's filesystem, where all the 
#       site's subdomains are symlink()ed from. 
#   default:
# SUBDOMAIN_SYMLINK_BASE='/var/subdomain'
# 

# HIDE_WILDCARD_OPTION
#   description: 
#       set this to 0 if you are an advanced user and/or if you wish to enable 
#       wildcard subdomains for any/all of your sites. Setting this value to 0
#       causes the 'Enable wildcards' checkbox to appear in the GUI on the 
#       Add/Edit domain page. The advantage of using subdomain wildcards, is 
#       that *you don't need to reload apache everytime you add a subdomain* !
#   default:
# HIDE_WILDCARD_OPTION=1
# 
